@@ -28,6 +28,30 @@ extern "C"
 {
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
+#ifdef HAVE_LIBSWRESAMPLE
+#include <libswresample/swresample.h>
+#elif defined HAVE_LIBAVRESAMPLE
+#include <libavresample/avresample.h>
+#endif
+#include <libavutil/opt.h>
+#include <libavutil/mathematics.h>
+#ifndef AVCODEC_MAX_AUDIO_FRAME_SIZE
+#define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000 // 1 second of 48khz 32bit audio
+#endif
+#ifdef HAVE_AVCODECID
+#define CodecID AVCodecID
+#define CODEC_ID_NONE AV_CODEC_ID_NONE
+#define CODEC_ID_H264 AV_CODEC_ID_H264
+#define CODEC_ID_FLV1 AV_CODEC_ID_FLV1
+#define CODEC_ID_VP6F AV_CODEC_ID_VP6F
+#define CODEC_ID_VP6A AV_CODEC_ID_VP6A
+#define CODEC_ID_AAC AV_CODEC_ID_AAC
+#define CODEC_ID_MP3 AV_CODEC_ID_MP3
+#define CODEC_ID_PCM_S16LE AV_CODEC_ID_PCM_S16LE
+#define CODEC_ID_PCM_F32BE AV_CODEC_ID_PCM_F32BE
+#define CODEC_ID_PCM_F32LE AV_CODEC_ID_PCM_F32LE
+#define CODEC_ID_ADPCM_SWF AV_CODEC_ID_ADPCM_SWF
+#endif
 #define MAX_AUDIO_FRAME_SIZE AVCODEC_MAX_AUDIO_FRAME_SIZE
 }
 #else
@@ -39,8 +63,19 @@ extern "C"
 namespace lightspark
 {
 
-enum LS_VIDEO_CODEC { H264=0, H263, VP6 };
-enum LS_AUDIO_CODEC { LINEAR_PCM_PLATFORM_ENDIAN=0, ADPCM=1, MP3=2, LINEAR_PCM_LE=3, AAC=10 };
+enum LS_VIDEO_CODEC { H264=0, H263, VP6, VP6A };
+// "Audio coding formats" from Chapter 11 in SWF documentation (except for LINEAR_PCM_FLOAT_BE)
+enum LS_AUDIO_CODEC { CODEC_NONE=-1, LINEAR_PCM_PLATFORM_ENDIAN=0, ADPCM=1, MP3=2, LINEAR_PCM_LE=3, AAC=10, LINEAR_PCM_FLOAT_BE = 100 };
+
+class AudioFormat
+{
+public:
+	AudioFormat(LS_AUDIO_CODEC co, int sr, int ch):codec(co),sampleRate(sr),channels(ch) {}
+	LS_AUDIO_CODEC codec;
+	int sampleRate;
+	int channels;
+};
+class NetStream;
 
 class Decoder
 {
@@ -66,11 +101,12 @@ public:
 class VideoDecoder: public Decoder, public ITextureUploadable
 {
 public:
-	VideoDecoder():frameRate(0),frameWidth(0),frameHeight(0),fenceCount(0),resizeGLBuffers(false){}
-	virtual ~VideoDecoder(){};
+	VideoDecoder():frameRate(0),framesdecoded(0),framesdropped(0),frameWidth(0),frameHeight(0),fenceCount(0),resizeGLBuffers(false),markedForDeletion(false){}
+	virtual ~VideoDecoder(){}
+	virtual void switchCodec(LS_VIDEO_CODEC codecId, uint8_t* initdata, uint32_t datalen, double frameRateHint)=0;
 	virtual bool decodeData(uint8_t* data, uint32_t datalen, uint32_t time)=0;
 	virtual bool discardFrame()=0;
-	virtual void skipUntil(uint32_t time)=0;
+	virtual uint32_t skipUntil(uint32_t time)=0;
 	virtual void skipAll()=0;
 	uint32_t getWidth()
 	{
@@ -81,6 +117,8 @@ public:
 		return frameHeight;
 	}
 	double frameRate;
+	uint32_t framesdecoded;
+	uint32_t framesdropped;
 	/*
 		Useful to avoid destruction of the object while a pending upload is waiting
 	*/
@@ -89,6 +127,7 @@ public:
 	void sizeNeeded(uint32_t& w, uint32_t& h) const;
 	const TextureChunk& getTexture();
 	void uploadFence();
+	void markForDestruction();
 protected:
 	TextureChunk videoTexture;
 	uint32_t frameWidth;
@@ -102,6 +141,7 @@ protected:
 	LS_VIDEO_CODEC videoCodec;
 private:
 	bool resizeGLBuffers;
+	bool markedForDeletion;
 };
 
 class NullVideoDecoder: public VideoDecoder
@@ -109,21 +149,23 @@ class NullVideoDecoder: public VideoDecoder
 public:
 	NullVideoDecoder() {status=VALID;}
 	~NullVideoDecoder() { while(fenceCount); }
-	bool decodeData(uint8_t* data, uint32_t datalen, uint32_t time){return false;}
-	bool discardFrame(){return false;}
-	void skipUntil(uint32_t time){}
-	void skipAll(){}
-	void setFlushing()
+	void switchCodec(LS_VIDEO_CODEC codecId, uint8_t* initdata, uint32_t datalen, double frameRateHint) override {}
+	bool decodeData(uint8_t* data, uint32_t datalen, uint32_t time) override {return false;}
+	bool discardFrame() override {return false;}
+	uint32_t skipUntil(uint32_t time) override { return 0;}
+	void skipAll() override {}
+	void setFlushing() override
 	{
 		flushing=true;
 	}
 	//ITextureUploadable interface
-	void upload(uint8_t* data, uint32_t w, uint32_t h) const
+	void upload(uint8_t* data, uint32_t w, uint32_t h) const override
 	{
 	}
 };
 
 #ifdef ENABLE_LIBAVCODEC
+#define FFMPEGVIDEODECODERBUFFERSIZE 80
 class FFMpegVideoDecoder: public VideoDecoder
 {
 private:
@@ -134,7 +176,7 @@ private:
 	public:
 		uint8_t* ch[3];
 		uint32_t time;
-		YUVBuffer():time(0){ch[0]=NULL;ch[1]=NULL;ch[2]=NULL;}
+		YUVBuffer():time(0){ch[0]=nullptr;ch[1]=nullptr;ch[2]=nullptr;}
 		~YUVBuffer()
 		{
 			if(ch[0])
@@ -156,7 +198,7 @@ private:
 	bool ownedContext;
 	uint32_t curBuffer;
 	AVCodecContext* codecContext;
-	BlockingCircularQueue<YUVBuffer,80> buffers;
+	BlockingCircularQueue<YUVBuffer,FFMPEGVIDEODECODERBUFFERSIZE> buffers;
 	Mutex mutex;
 	AVFrame* frameIn;
 	void copyFrameToBuffers(const AVFrame* frameIn, uint32_t time);
@@ -168,17 +210,22 @@ public:
 	/*
 	   Specialized constructor used by FFMpegStreamDecoder
 	*/
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 40, 101)
+	FFMpegVideoDecoder(AVCodecParameters* codecPar, double frameRateHint);
+#else
 	FFMpegVideoDecoder(AVCodecContext* codecContext, double frameRateHint);
+#endif
 	~FFMpegVideoDecoder();
 	/*
 	   Specialized decoding used by FFMpegStreamDecoder
 	*/
 	bool decodePacket(AVPacket* pkt, uint32_t time);
-	bool decodeData(uint8_t* data, uint32_t datalen, uint32_t time);
-	bool discardFrame();
-	void skipUntil(uint32_t time);
-	void skipAll();
-	void setFlushing()
+	void switchCodec(LS_VIDEO_CODEC codecId, uint8_t* initdata, uint32_t datalen, double frameRateHint) override;
+	bool decodeData(uint8_t* data, uint32_t datalen, uint32_t time) override;
+	bool discardFrame() override;
+	uint32_t skipUntil(uint32_t time) override;
+	void skipAll() override;
+	void setFlushing() override
 	{
 		flushing=true;
 		if(buffers.isEmpty())
@@ -188,7 +235,7 @@ public:
 		}
 	}
 	//ITextureUploadable interface
-	void upload(uint8_t* data, uint32_t w, uint32_t h) const;
+	void upload(uint8_t* data, uint32_t w, uint32_t h) const override;
 };
 #endif
 
@@ -221,7 +268,8 @@ public:
 	void* operator new(size_t);
 	void operator delete(void*);
 	AudioDecoder():sampleRate(0),channelCount(0),initialTime(-1){}
-	virtual ~AudioDecoder(){};
+	virtual ~AudioDecoder(){}
+	virtual void switchCodec(LS_AUDIO_CODEC codecId, uint8_t* initdata, uint32_t datalen)=0;
 	virtual uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time)=0;
 	bool hasDecodedFrames() const
 	{
@@ -268,30 +316,47 @@ public:
 		sampleRate=44100;
 		channelCount=2;
 	}
+	void switchCodec(LS_AUDIO_CODEC codecId, uint8_t* initdata, uint32_t datalen){}
 	uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time){return 0;}
 };
 
 #ifdef ENABLE_LIBAVCODEC
+class EngineData;
 class FFMpegAudioDecoder: public AudioDecoder
 {
 private:
+	EngineData* engine;
 	bool ownedContext;
 	AVCodecContext* codecContext;
+#ifdef HAVE_LIBSWRESAMPLE
+	SwrContext* resamplecontext;
+#elif defined HAVE_LIBAVRESAMPLE
+	AVAudioResampleContext* resamplecontext;
+#endif
+	std::vector<uint8_t> overflowBuffer;
 	bool fillDataAndCheckValidity();
-#if HAVE_AVCODEC_DECODE_AUDIO4
+	CodecID LSToFFMpegCodec(LS_AUDIO_CODEC lscodec);
+#ifdef HAVE_AVCODEC_DECODE_AUDIO4
 	AVFrame* frameIn;
+	int resampleFrameToS16(FrameSamples& curTail);
 #endif
 public:
-	FFMpegAudioDecoder(LS_AUDIO_CODEC codec, uint8_t* initdata, uint32_t datalen);
+	FFMpegAudioDecoder(EngineData* eng,LS_AUDIO_CODEC codec, uint8_t* initdata, uint32_t datalen);
+	FFMpegAudioDecoder(EngineData* eng,LS_AUDIO_CODEC codec, int sampleRate, int channels, bool);
 	/*
 	   Specialized constructor used by FFMpegStreamDecoder
 	*/
-	FFMpegAudioDecoder(AVCodecContext* codecContext);
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 40, 101)
+	FFMpegAudioDecoder(EngineData* eng,AVCodecParameters* codecPar);
+#else
+	FFMpegAudioDecoder(EngineData* eng,AVCodecContext* codecContext);
+#endif
 	~FFMpegAudioDecoder();
 	/*
 	   Specialized decoding used by FFMpegStreamDecoder
 	*/
 	uint32_t decodePacket(AVPacket* pkt, uint32_t time);
+	void switchCodec(LS_AUDIO_CODEC audioCodec, uint8_t* initdata, uint32_t datalen);
 	uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time);
 };
 #endif
@@ -299,22 +364,24 @@ public:
 class StreamDecoder
 {
 public:
-	StreamDecoder():audioDecoder(NULL),videoDecoder(NULL),valid(false){}
+	StreamDecoder():audioDecoder(NULL),videoDecoder(NULL),valid(false),hasvideo(false){}
 	virtual ~StreamDecoder();
 	virtual bool decodeNextFrame() = 0;
-	virtual bool getMetadataInteger(const char* name, uint32_t& ret) const=0;
-	virtual bool getMetadataDouble(const char* name, double& ret) const=0;
+	virtual void jumpToPosition(number_t position) = 0;
 	bool isValid() const { return valid; }
 	AudioDecoder* audioDecoder;
 	VideoDecoder* videoDecoder;
+	bool hasVideo() const { return hasvideo; }
 protected:
 	bool valid;
+	bool hasvideo;
 };
 
 #ifdef ENABLE_LIBAVCODEC
 class FFMpegStreamDecoder: public StreamDecoder
 {
 private:
+	NetStream* netstream;
 	bool audioFound;
 	bool videoFound;
 	std::istream& stream;
@@ -325,7 +392,7 @@ private:
 	FFMpegAudioDecoder* customAudioDecoder;
 	FFMpegVideoDecoder* customVideoDecoder;
 	//Helpers for custom I/O of libavformat
-	uint8_t avioBuffer[4096];
+	uint8_t* avioBuffer;
 	static int avioReadPacket(void* t, uint8_t* buf, int buf_size);
 	//NOTE: this will become AVIOContext in FFMpeg 0.7
 #if LIBAVUTIL_VERSION_MAJOR < 51
@@ -333,14 +400,15 @@ private:
 #else
 	AVIOContext* avioContext;
 #endif
+	int availablestreamlength;
 public:
-	FFMpegStreamDecoder(std::istream& s);
+	FFMpegStreamDecoder(NetStream* ns,EngineData* eng,std::istream& s, AudioFormat* format = NULL, int streamsize = -1);
 	~FFMpegStreamDecoder();
-	bool decodeNextFrame();
-	bool getMetadataInteger(const char* name, uint32_t& ret) const;
-	bool getMetadataDouble(const char* name, double& ret) const;
+	void jumpToPosition(number_t position) override;
+	bool decodeNextFrame() override;
+	int getAudioSampleRate();
 };
 #endif
 
-};
+}
 #endif /* BACKENDS_DECODER */
