@@ -20,173 +20,35 @@
 #include <cassert>
 
 #include "swf.h"
+#include "abc.h"
 #include "backends/graphics.h"
 #include "logger.h"
 #include "exceptions.h"
 #include "backends/rendering.h"
 #include "backends/config.h"
 #include "compat.h"
+#include "scripting/flash/geom/flashgeom.h"
 #include "scripting/flash/text/flashtext.h"
 #include "scripting/flash/display/BitmapData.h"
+#include <pango/pangocairo.h>
 
 using namespace lightspark;
 
-void TextureBuffer::setAllocSize(uint32_t w, uint32_t h)
-{
-	if(getRenderThread()->hasNPOTTextures)
-	{
-		allocWidth=w;
-		allocHeight=h;
-		//Now adjust for the requested alignment
-		if((allocWidth%horizontalAlignment))
-		{
-			allocWidth+=horizontalAlignment;
-			allocWidth-=(allocWidth%horizontalAlignment);
-		}
-		if((allocHeight%verticalAlignment))
-		{
-			allocHeight+=verticalAlignment;
-			allocHeight-=(allocHeight%verticalAlignment);
-		}
-	}
-	else
-	{
-		allocWidth=nearestPOT(w);
-		allocHeight=nearestPOT(h);
-		//Assert that the requested alignment is satisfied
-		assert((allocWidth%horizontalAlignment)==0);
-		assert((allocHeight%verticalAlignment)==0);
-	}
-}
-
-uint32_t TextureBuffer::nearestPOT(uint32_t a) const
-{
-	if(a==0)
-		return 0;
-	uint32_t ret=1;
-	while(ret<a)
-		ret<<=1;
-	return ret;
-}
-
-TextureBuffer::TextureBuffer(bool initNow, uint32_t w, uint32_t h, GLenum f):texId(0),filtering(f),allocWidth(0),allocHeight(0),
-	width(w),height(h),horizontalAlignment(1),verticalAlignment(1),inited(false)
-{
-	if(initNow)
-		init(w, h, f);
-}
-
-void TextureBuffer::init(uint32_t w, uint32_t h, GLenum f)
-{
-	assert(!inited);
-	inited=true;
-
-	setAllocSize(w,h);
-	width=w;
-	height=h;
-	filtering=f;
-	
-	assert(texId==0);
-	glGenTextures(1,&texId);
-	assert(texId!=0);
-	
-	assert(filtering==GL_NEAREST || filtering==GL_LINEAR);
-	
-	//If the previous call has not failed these should not fail (in specs, we trust)
-	glBindTexture(GL_TEXTURE_2D,texId);
-	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,filtering);
-	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,filtering);
-	//Wrapping should not be very useful, we use textures carefully
-	//glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
-	
-	//Allocate the texture
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, allocWidth, allocHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, 0);
-	
-	glBindTexture(GL_TEXTURE_2D,0);
-	
-	if(GLRenderContext::handleGLErrors())
-	{
-		LOG(LOG_ERROR,_("OpenGL error in TextureBuffer::init"));
-		throw RunTimeException("OpenGL error in TextureBuffer::init");
-	}
-}
-
-void TextureBuffer::resize(uint32_t w, uint32_t h)
-{
-	if(width!=w || height!=h)
-	{
-		if(w>allocWidth || h>allocHeight) //Destination texture should be reallocated
-		{
-			glBindTexture(GL_TEXTURE_2D,texId);
-			LOG(LOG_CALLS,_("Reallocating texture to size ") << w << 'x' << h);
-			setAllocSize(w,h);
-			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, allocWidth, allocHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, 0);
-			if(GLRenderContext::handleGLErrors())
-			{
-				LOG(LOG_ERROR,_("OpenGL error in TextureBuffer::resize"));
-				throw RunTimeException("OpenGL error in TextureBuffer::resize");
-			}
-		}
-		width=w;
-		height=h;
-	}
-}
-
-void TextureBuffer::setRequestedAlignment(uint32_t w, uint32_t h)
-{
-	assert(w && h);
-	horizontalAlignment=w;
-	verticalAlignment=h;
-}
-
-void TextureBuffer::setTexScale(GLuint uniformLocation)
-{
-	float v1=width;
-	float v2=height;
-	v1/=allocWidth;
-	v2/=allocHeight;
-	glUniform2f(uniformLocation,v1,v2);
-}
-
-void TextureBuffer::bind()
-{
-	glBindTexture(GL_TEXTURE_2D,texId);
-}
-
-void TextureBuffer::unbind()
-{
-	glBindTexture(GL_TEXTURE_2D,0);
-}
-
-void TextureBuffer::shutdown()
-{
-	if(inited)
-	{
-		glDeleteTextures(1,&texId);
-		inited=false;
-	}
-}
-
-TextureBuffer::~TextureBuffer()
-{
-	shutdown();
-}
-
 TextureChunk::TextureChunk(uint32_t w, uint32_t h)
 {
-	width=w;
-	height=h;
+	width=w+(w/CHUNKSIZE_REAL+1)*2;
+	height=h+(h/CHUNKSIZE_REAL+1)*2;
 	if(w==0 || h==0)
 	{
-		chunks=NULL;
+		chunks=nullptr;
 		return;
 	}
-	const uint32_t blocksW=(w+CHUNKSIZE-1)/CHUNKSIZE;
-	const uint32_t blocksH=(h+CHUNKSIZE-1)/CHUNKSIZE;
+	const uint32_t blocksW=(width+CHUNKSIZE-1)/CHUNKSIZE;
+	const uint32_t blocksH=(height+CHUNKSIZE-1)/CHUNKSIZE;
 	chunks=new uint32_t[blocksW*blocksH];
 }
 
-TextureChunk::TextureChunk(const TextureChunk& r):chunks(NULL),texId(0),width(r.width),height(r.height)
+TextureChunk::TextureChunk(const TextureChunk& r):chunks(nullptr),texId(0),width(r.width),height(r.height)
 {
 	*this = r;
 	return;
@@ -211,7 +73,7 @@ TextureChunk& TextureChunk::operator=(const TextureChunk& r)
 		memcpy(chunks, r.chunks, blocksW*blocksH*4);
 	}
 	else
-		chunks=NULL;
+		chunks=nullptr;
 	return *this;
 }
 
@@ -225,8 +87,9 @@ void TextureChunk::makeEmpty()
 	width=0;
 	height=0;
 	texId=0;
-	delete[] chunks;
-	chunks=NULL;
+	if (chunks)
+		delete[] chunks;
+	chunks=nullptr;
 }
 
 bool TextureChunk::resizeIfLargeEnough(uint32_t w, uint32_t h)
@@ -236,13 +99,15 @@ bool TextureChunk::resizeIfLargeEnough(uint32_t w, uint32_t h)
 		//The texture collapsed, release the resources
 		getSys()->getRenderThread()->releaseTexture(*this);
 		delete[] chunks;
-		chunks=NULL;
+		chunks=nullptr;
 		width=w;
 		height=h;
 		return true;
 	}
 	const uint32_t blocksW=(width+CHUNKSIZE-1)/CHUNKSIZE;
 	const uint32_t blocksH=(height+CHUNKSIZE-1)/CHUNKSIZE;
+	w += (w/CHUNKSIZE_REAL+1)*2;
+	h += (h/CHUNKSIZE_REAL+1)*2;
 	if(w<=blocksW*CHUNKSIZE && h<=blocksH*CHUNKSIZE)
 	{
 		width=w;
@@ -252,9 +117,15 @@ bool TextureChunk::resizeIfLargeEnough(uint32_t w, uint32_t h)
 	return false;
 }
 
-CairoRenderer::CairoRenderer(const MATRIX& _m, int32_t _x, int32_t _y, int32_t _w, int32_t _h,
-		float _s, float _a, const std::vector<MaskData>& _ms)
-	: IDrawable(_w, _h, _x, _y, _a, _ms), scaleFactor(_s), matrix(_m)
+CairoRenderer::CairoRenderer(const MATRIX& _m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r, float _xs, float _ys, bool _im, bool _hm,
+		float _s, float _a, const std::vector<MaskData>& _ms,
+		float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier,
+		float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset,
+		bool _smoothing, number_t _xstart, number_t _ystart)
+	: IDrawable(_w, _h, _x, _y, _rw, _rh, _rx, _ry, _r, _xs, _ys, _im, _hm,_a, _ms,
+				_redMultiplier,_greenMultiplier,_blueMultiplier,_alphaMultiplier,
+				_redOffset,_greenOffset,_blueOffset,_alphaOffset)
+	, scaleFactor(_s),smoothing(_smoothing), matrix(_m),xstart(_xstart),ystart(_ystart)
 {
 }
 
@@ -272,17 +143,16 @@ void CairoTokenRenderer::quadraticBezier(cairo_t* cr, double control_x, double c
 	               end_x, end_y);
 }
 
-cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection)
+cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection, float scalex, float scaley, bool isMask)
 {
-	cairo_pattern_t* pattern = NULL;
+	cairo_pattern_t* pattern = nullptr;
 
 	switch(style.FillStyleType)
 	{
 		case SOLID_FILL:
 		{
 			const RGBA& color = style.Color;
-			pattern = cairo_pattern_create_rgba(color.rf(), color.gf(),
-			                                    color.bf(), color.af());
+			pattern = cairo_pattern_create_rgba(color.rf(), color.gf(),color.bf(), isMask ? 1.0 : color.af());
 			break;
 		}
 		case LINEAR_GRADIENT:
@@ -299,8 +169,13 @@ cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, do
 			}
 
 			MATRIX tmp=style.Matrix;
+			tmp.x0 -= style.ShapeBounds.Xmin/20;
+			tmp.y0 -= style.ShapeBounds.Ymin/20;
+			cairo_matrix_scale(&tmp,scalex,scaley);
 			tmp.x0/=scaleCorrection;
 			tmp.y0/=scaleCorrection;
+			tmp.x0*=scalex;
+			tmp.y0*=scaley;
 			// The dimensions of the pattern space are specified in SWF specs
 			// as a 32768x32768 box centered at (0,0)
 			if (style.FillStyleType == LINEAR_GRADIENT)
@@ -316,7 +191,7 @@ cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, do
 				double x1,y1; //Point on the circle edge at 0°
 				tmp.multiply2D(0, 0,x0,y0);
 				tmp.multiply2D(16384.0, 0,x1,y1);
-				double radius=sqrt(x1*x1+y1*y1);
+				double radius=x1-x0;
 				pattern = cairo_pattern_create_radial(x0, y0, 0, x0, y0, radius);
 			}
 
@@ -330,10 +205,8 @@ cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, do
 			for(uint32_t i=0;i<grad.GradientRecords.size();i++)
 			{
 				double ratio = grad.GradientRecords[i].Ratio / 255.0;
-				const RGBA& color=style.Gradient.GradientRecords[i].Color;
-
-				cairo_pattern_add_color_stop_rgba(pattern, ratio,
-				    color.rf(), color.gf(), color.bf(), color.af());
+				const RGBA& color=grad.GradientRecords[i].Color;
+				cairo_pattern_add_color_stop_rgba(pattern, ratio,color.rf(), color.gf(),color.bf(), color.af());
 			}
 			break;
 		}
@@ -345,20 +218,26 @@ cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, do
 		{
 			_NR<BitmapContainer> bm(style.bitmap);
 			if(bm.isNull())
-				return NULL;
+				return nullptr;
+			if (!style.Matrix.isInvertible())
+				return nullptr;
 
+			cairo_surface_t* surface = nullptr;
 			//Do an explicit cast, the data will not be modified
-			cairo_surface_t* surface = cairo_image_surface_create_for_data ((uint8_t*)bm->getData(),
-								CAIRO_FORMAT_ARGB32,
-								bm->getWidth(),
-								bm->getHeight(),
-								cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, bm->getWidth()));
+			surface = cairo_image_surface_create_for_data ((uint8_t*)bm->getData(),
+									CAIRO_FORMAT_ARGB32,
+									bm->getWidth(),
+									bm->getHeight(),
+									cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, bm->getWidth()));
 
 			pattern = cairo_pattern_create_for_surface(surface);
 			cairo_surface_destroy(surface);
 
 			//Make a copy to invert it
 			cairo_matrix_t mat=style.Matrix;
+			mat.x0 -= style.ShapeBounds.Xmin/20;
+			mat.y0 -= style.ShapeBounds.Ymin/20;
+			cairo_matrix_scale(&mat,scalex,scaley);
 			cairo_status_t st = cairo_matrix_invert(&mat);
 			assert(st == CAIRO_STATUS_SUCCESS);
 			(void)st; // silence warning about unused variable
@@ -383,19 +262,22 @@ cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, do
 		}
 		default:
 			LOG(LOG_NOT_IMPLEMENTED, "Unsupported fill style " << (int)style.FillStyleType);
-			return NULL;
+			return nullptr;
 	}
 
 	return pattern;
 }
 
-bool CairoTokenRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken>& tokens, double scaleCorrection, bool skipPaint)
+bool CairoTokenRenderer::cairoPathFromTokens(cairo_t* cr, const tokensVector& tokens, double scaleCorrection, bool skipPaint,float scalex,float scaley,number_t xstart, number_t ystart,bool isMask)
 {
 	cairo_scale(cr, scaleCorrection, scaleCorrection);
 
 	bool empty=true;
 
 	cairo_t *stroke_cr = cairo_create(cairo_get_group_target(cr));
+	cairo_matrix_t mat;
+	cairo_get_matrix(cr,&mat);
+	cairo_set_matrix(stroke_cr, &mat);
 	cairo_push_group(stroke_cr);
 
 	// Make sure not to draw anything until a fill is set.
@@ -403,134 +285,187 @@ bool CairoTokenRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<Geom
 	cairo_set_operator(cr, CAIRO_OPERATOR_DEST);
 
 	#define PATH(operation, args...) \
-		operation(cr, ## args); \
-		operation(stroke_cr, ## args);
+		operation(instroke?stroke_cr:cr, ## args);
 
-	for(uint32_t i=0;i<tokens.size();i++)
+	bool instroke = false;
+	int tokentype = 1;
+	while (tokentype)
 	{
-		switch(tokens[i].type)
+		std::vector<_NR<GeomToken>, reporter_allocator<_NR<GeomToken>>>::const_iterator it;
+		std::vector<_NR<GeomToken>, reporter_allocator<_NR<GeomToken>>>::const_iterator itend;
+		switch(tokentype)
 		{
-			case MOVE:
-				PATH(cairo_move_to, tokens[i].p1.x, tokens[i].p1.y);
+			case 1:
+				it = tokens.filltokens.begin();
+				itend = tokens.filltokens.end();
+				tokentype++;
 				break;
-			case STRAIGHT:
-				PATH(cairo_line_to, tokens[i].p1.x, tokens[i].p1.y);
-				empty = false;
+			case 2:
+				it = tokens.stroketokens.begin();
+				itend = tokens.stroketokens.end();
+				tokentype++;
 				break;
-			case CURVE_QUADRATIC:
-				PATH(quadraticBezier,
-				   tokens[i].p1.x, tokens[i].p1.y,
-				   tokens[i].p2.x, tokens[i].p2.y);
-				empty = false;
-				break;
-			case CURVE_CUBIC:
-				PATH(cairo_curve_to,
-				   tokens[i].p1.x, tokens[i].p1.y,
-				   tokens[i].p2.x, tokens[i].p2.y,
-				   tokens[i].p3.x, tokens[i].p3.y);
-				empty = false;
-				break;
-			case SET_FILL:
-			{
-				if(skipPaint)
-					break;
-
-				cairo_fill(cr);
-
-				cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-
-				const FILLSTYLE& style = tokens[i].fillStyle;
-				cairo_pattern_t* pattern = FILLSTYLEToCairo(style, scaleCorrection);
-				if(pattern)
-				{
-					cairo_set_source(cr, pattern);
-					// Destroy the first reference.
-					cairo_pattern_destroy(pattern);
-				}
-
-				break;
-			}
-			case SET_STROKE:
-			{
-				if(skipPaint)
-					break;
-
-				cairo_stroke(stroke_cr);
-
-				const LINESTYLE2& style = tokens[i].lineStyle;
-				const RGBA& color = style.Color;
-
-				cairo_set_operator(stroke_cr, CAIRO_OPERATOR_OVER);
-				cairo_set_source_rgba(stroke_cr, color.rf(), color.gf(), color.bf(), color.af());
-
-				// TODO: EndCapStyle
-				if (style.StartCapStyle == 0)
-					cairo_set_line_cap(stroke_cr, CAIRO_LINE_CAP_ROUND);
-				else if (style.StartCapStyle == 1)
-					cairo_set_line_cap(stroke_cr, CAIRO_LINE_CAP_BUTT);
-				else if (style.StartCapStyle == 2)
-					cairo_set_line_cap(stroke_cr, CAIRO_LINE_CAP_SQUARE);
-
-				if (style.JointStyle == 0)
-					cairo_set_line_join(stroke_cr, CAIRO_LINE_JOIN_ROUND);
-				else if (style.JointStyle == 1)
-					cairo_set_line_join(stroke_cr, CAIRO_LINE_JOIN_BEVEL);
-				else if (style.JointStyle == 2) {
-					cairo_set_line_join(stroke_cr, CAIRO_LINE_JOIN_MITER);
-					cairo_set_miter_limit(stroke_cr, style.MiterLimitFactor);
-				}
-
-				//Width == 0 should be a hairline, but
-				//cairo does not support hairlines.
-				//Line width 1 is not a perfect
-				//substitute, because it is affected
-				//by transformations.
-				if (style.Width == 0)
-					cairo_set_line_width(stroke_cr, 1);
-				else
-					cairo_set_line_width(stroke_cr, (double)(style.Width / 20.0));
-				break;
-			}
-
-			case CLEAR_FILL:
-			case FILL_KEEP_SOURCE:
-				if(skipPaint)
-					break;
-
-				cairo_fill(cr);
-
-				if(tokens[i].type==CLEAR_FILL)
-					// Clear source.
-					cairo_set_operator(cr, CAIRO_OPERATOR_DEST);
-				break;
-			case CLEAR_STROKE:
-				if(skipPaint)
-					break;
-
-				cairo_stroke(stroke_cr);
-
-				// Clear source.
-				cairo_set_operator(stroke_cr, CAIRO_OPERATOR_DEST);
-				break;
-			case FILL_TRANSFORM_TEXTURE:
-			{
-				if(skipPaint)
-					break;
-
-				cairo_matrix_t origmat;
-				cairo_pattern_t* pattern;
-				pattern=cairo_get_source(cr);
-				cairo_pattern_get_matrix(pattern, &origmat);
-
-				cairo_pattern_set_matrix(pattern, &tokens[i].textureTransform);
-
-				cairo_fill(cr);
-
-				cairo_pattern_set_matrix(pattern, &origmat);
-				break;
-			}
 			default:
-				assert(false);
+				tokentype = 0;
+				break;
+		}
+		if (tokentype == 0)
+			break;
+		for (;it != itend; it++)
+		{
+			switch((*it)->type)
+			{
+				case MOVE:
+					PATH(cairo_move_to, (*it)->p1.x*scalex-xstart*scalex, (*it)->p1.y*scaley-ystart*scaley);
+					break;
+				case STRAIGHT:
+					PATH(cairo_line_to, (*it)->p1.x*scalex-xstart*scalex, (*it)->p1.y*scaley-ystart*scaley);
+					empty = false;
+					break;
+				case CURVE_QUADRATIC:
+					PATH(quadraticBezier,
+					   (*it)->p1.x*scalex-xstart*scalex, (*it)->p1.y*scaley-ystart*scaley,
+					   (*it)->p2.x*scalex-xstart*scalex, (*it)->p2.y*scaley-ystart*scaley);
+					empty = false;
+					break;
+				case CURVE_CUBIC:
+					PATH(cairo_curve_to,
+					   (*it)->p1.x*scalex-xstart*scalex, (*it)->p1.y*scaley-ystart*scaley,
+					   (*it)->p2.x*scalex-xstart*scalex, (*it)->p2.y*scaley-ystart*scaley,
+					   (*it)->p3.x*scalex-xstart*scalex, (*it)->p3.y*scaley-ystart*scaley);
+					empty = false;
+					break;
+				case SET_FILL:
+				{
+					if(skipPaint)
+						break;
+	
+					cairo_fill(cr);
+	
+					cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+	
+					const FILLSTYLE& style = (*it)->fillStyle;
+					if (style.FillStyleType == NON_SMOOTHED_CLIPPED_BITMAP ||
+						style.FillStyleType == NON_SMOOTHED_REPEATING_BITMAP)
+						cairo_set_antialias(cr,CAIRO_ANTIALIAS_NONE);
+					cairo_pattern_t* pattern = FILLSTYLEToCairo(style, scaleCorrection,scalex,scaley,isMask);
+					if(pattern)
+					{
+						cairo_set_source(cr, pattern);
+						// Destroy the first reference.
+						cairo_pattern_destroy(pattern);
+					}
+	
+					break;
+				}
+				case SET_STROKE:
+				{
+					instroke = true;
+					if(skipPaint)
+						break;
+	
+					cairo_stroke(stroke_cr);
+	
+					const LINESTYLE2& style = (*it)->lineStyle;
+	
+					cairo_set_operator(stroke_cr, CAIRO_OPERATOR_OVER);
+					if (style.HasFillFlag)
+					{
+						if (style.FillType.FillStyleType == NON_SMOOTHED_CLIPPED_BITMAP ||
+							style.FillType.FillStyleType == NON_SMOOTHED_REPEATING_BITMAP)
+							cairo_set_antialias(cr,CAIRO_ANTIALIAS_NONE);
+						cairo_pattern_t* pattern = FILLSTYLEToCairo(style.FillType, scaleCorrection,scalex,scaley,isMask);
+						if (pattern)
+						{
+							cairo_set_source(stroke_cr, pattern);
+							cairo_pattern_destroy(pattern);
+						}
+					} else {
+						const RGBA& color = style.Color;
+						float r,g,b,a;
+						r = color.rf();
+						g = color.gf();
+						b = color.bf();
+						a = isMask ? 1.0 : color.af();
+						cairo_set_source_rgba(stroke_cr, r, g, b, a);
+					}
+	
+					// TODO: EndCapStyle
+					if (style.StartCapStyle == 0)
+						cairo_set_line_cap(stroke_cr, CAIRO_LINE_CAP_ROUND);
+					else if (style.StartCapStyle == 1)
+						cairo_set_line_cap(stroke_cr, CAIRO_LINE_CAP_BUTT);
+					else if (style.StartCapStyle == 2)
+						cairo_set_line_cap(stroke_cr, CAIRO_LINE_CAP_SQUARE);
+	
+					if (style.JointStyle == 0)
+						cairo_set_line_join(stroke_cr, CAIRO_LINE_JOIN_ROUND);
+					else if (style.JointStyle == 1)
+						cairo_set_line_join(stroke_cr, CAIRO_LINE_JOIN_BEVEL);
+					else if (style.JointStyle == 2) {
+						cairo_set_line_join(stroke_cr, CAIRO_LINE_JOIN_MITER);
+						cairo_set_miter_limit(stroke_cr, style.MiterLimitFactor);
+					}
+	
+					//Width == 0 should be a hairline, but
+					//cairo does not support hairlines.
+					//Line width 1 is not a perfect
+					//substitute, because it is affected
+					//by transformations.
+					if (style.Width == 0)
+						cairo_set_line_width(stroke_cr, 1);
+					else if (style.Width < 20) // would result in line with < 1 what seems to lead to no rendering at all
+						cairo_set_line_width(stroke_cr, 5);
+					else 
+					{
+						double linewidth = (double)(style.Width / 20.0 / scaleCorrection);
+						//cairo_device_to_user_distance(stroke_cr, &linewidth, &linewidth);
+						cairo_set_line_width(stroke_cr, linewidth);
+					}
+					break;
+				}
+	
+				case CLEAR_FILL:
+				case FILL_KEEP_SOURCE:
+					if(skipPaint)
+						break;
+	
+					cairo_fill(cr);
+	
+					if((*it)->type==CLEAR_FILL)
+						// Clear source.
+						cairo_set_operator(cr, CAIRO_OPERATOR_DEST);
+					break;
+				case CLEAR_STROKE:
+					instroke = false;
+					if(skipPaint)
+						break;
+	
+					cairo_stroke(stroke_cr);
+	
+					// Clear source.
+					cairo_set_operator(stroke_cr, CAIRO_OPERATOR_DEST);
+					break;
+				case FILL_TRANSFORM_TEXTURE:
+				{
+					if(skipPaint)
+						break;
+	
+					cairo_matrix_t origmat;
+					cairo_pattern_t* pattern;
+					pattern=cairo_get_source(cr);
+					cairo_pattern_get_matrix(pattern, &origmat);
+	
+					cairo_pattern_set_matrix(pattern, &(*it)->textureTransform);
+	
+					cairo_fill(cr);
+	
+					cairo_pattern_set_matrix(pattern, &origmat);
+					break;
+				}
+				default:
+					assert(false);
+			}
 		}
 	}
 
@@ -539,7 +474,8 @@ bool CairoTokenRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<Geom
 	if(!skipPaint)
 	{
 		cairo_fill(cr);
-		cairo_stroke(stroke_cr);
+		if (instroke)
+			cairo_stroke(stroke_cr);
 	}
 
 	cairo_pattern_t *stroke_pattern = cairo_pop_group(stroke_cr);
@@ -570,97 +506,57 @@ cairo_surface_t* CairoRenderer::allocateSurface(uint8_t*& buf)
 	return cairo_image_surface_create_for_data(buf, CAIRO_FORMAT_ARGB32, width, height, cairoWidthStride);
 }
 
-void CairoTokenRenderer::executeDraw(cairo_t* cr)
+void CairoTokenRenderer::executeDraw(cairo_t* cr, float scalex, float scaley)
 {
+	cairo_set_antialias(cr,smoothing ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
 	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
 	cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
 
-	cairoPathFromTokens(cr, tokens, scaleFactor, false);
+	cairoPathFromTokens(cr, tokens, scaleFactor, false,scalex, scaley,xstart,ystart,isMask);
 }
 
-#ifdef HAVE_NEW_GLIBMM_THREAD_API
-StaticRecMutex CairoRenderer::cairoMutex;
-#else
-StaticRecMutex CairoRenderer::cairoMutex = GLIBMM_STATIC_REC_MUTEX_INIT;
-#endif
-
-uint8_t* CairoRenderer::getPixelBuffer()
+uint8_t* CairoRenderer::getPixelBuffer(float scalex, float scaley)
 {
-	RecMutex::Lock l(cairoMutex);
 	if(width==0 || height==0 || !Config::getConfig()->isRenderingEnabled())
-		return NULL;
+		return nullptr;
 
-	int32_t windowWidth=getSys()->getRenderThread()->windowWidth;
-	int32_t windowHeight=getSys()->getRenderThread()->windowHeight;
-	//Discard stuff that it's outside the visible part
-	if(xOffset >= windowWidth || yOffset >= windowHeight
-		|| xOffset + width <= 0 || yOffset + height <= 0)
-	{
-		width=0;
-		height=0;
-		return NULL;
-	}
+	uint8_t* ret=nullptr;
 
-	if(xOffset<0)
-	{
-		width+=xOffset;
-		xOffset=0;
-	}
-	if(yOffset<0)
-	{
-		height+=yOffset;
-		yOffset=0;
-	}
-
-	//Clip the size to the screen borders
-	if((xOffset>0) && (width+xOffset) > windowWidth)
-		width=windowWidth-xOffset;
-	if((yOffset>0) && (height+yOffset) > windowHeight)
-		height=windowHeight-yOffset;
-	uint8_t* ret=NULL;
 	cairo_surface_t* cairoSurface=allocateSurface(ret);
 
 	cairo_t* cr=cairo_create(cairoSurface);
 	cairo_surface_destroy(cairoSurface); /* cr has an reference to it */
 	cairoClean(cr);
-
-	//Make sure the rendering starts at 0,0 in surface coordinates
-	//This also guarantees that all the shape fills in width/height pixels
-	//We don't translate for negative offsets as we don't want to see what's in negative coords
-	if(xOffset >= 0)
-		matrix.x0-=xOffset;
-	if(yOffset >= 0)
-		matrix.y0-=yOffset;
+	cairo_set_antialias(cr,smoothing ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
 
 	//Apply all the masks to clip the drawn part
 	for(uint32_t i=0;i<masks.size();i++)
 	{
 		if(masks[i].maskMode != HARD_MASK)
 			continue;
-		masks[i].m->applyCairoMask(cr,xOffset,yOffset);
+		masks[i].m->applyCairoMask(cr,xOffset,yOffset,scalex,scaley);
 	}
 
-	cairo_set_matrix(cr, &matrix);
-	executeDraw(cr);
+	executeDraw(cr,scalex, scaley);
 
-	cairo_surface_t* maskSurface = NULL;
-	uint8_t* maskRawData = NULL;
-	cairo_t* maskCr = NULL;
-	int32_t maskXOffset = 0;
-	int32_t maskYOffset = 0;
+	cairo_surface_t* maskSurface = nullptr;
+	uint8_t* maskRawData = nullptr;
+	cairo_t* maskCr = nullptr;
+	int32_t maskXOffset = 1;
+	int32_t maskYOffset = 1;
 	//Also apply the soft masks
 	for(uint32_t i=0;i<masks.size();i++)
 	{
 		if(masks[i].maskMode != SOFT_MASK)
 			continue;
 		//TODO: this may be optimized if needed
-		uint8_t* maskData = masks[i].m->getPixelBuffer();
-		if(maskData==NULL)
+		uint8_t* maskData = masks[i].m->getPixelBuffer(scalex,scaley);
+		if(maskData==nullptr)
 			continue;
 
 		cairo_surface_t* tmp = cairo_image_surface_create_for_data(maskData,CAIRO_FORMAT_ARGB32,
 				masks[i].m->getWidth(),masks[i].m->getHeight(),masks[i].m->getWidth()*4);
-		if(maskSurface==NULL)
+		if(maskSurface==nullptr)
 		{
 			maskSurface = tmp;
 			maskRawData = maskData;
@@ -695,14 +591,14 @@ uint8_t* CairoRenderer::getPixelBuffer()
 	return ret;
 }
 
-bool CairoTokenRenderer::hitTest(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y)
+bool CairoTokenRenderer::hitTest(const tokensVector& tokens, float scaleFactor, number_t x, number_t y)
 {
-	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(NULL, CAIRO_FORMAT_ARGB32, 0, 0, 0);
+	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(nullptr, CAIRO_FORMAT_ARGB32, 0, 0, 0);
 	cairo_t *cr=cairo_create(cairoSurface);
 	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-	cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
+	cairo_set_fill_rule(cr, CAIRO_FILL_RULE_WINDING);
 
-	bool empty=cairoPathFromTokens(cr, tokens, scaleFactor, true);
+	bool empty=cairoPathFromTokens(cr, tokens, scaleFactor, true,1.0,1.0,0,0,true);
 	bool ret=false;
 	if(!empty)
 	{
@@ -717,36 +613,52 @@ bool CairoTokenRenderer::hitTest(const std::vector<GeomToken>& tokens, float sca
 	return ret;
 }
 
-void CairoTokenRenderer::applyCairoMask(cairo_t* cr,int32_t xOffset,int32_t yOffset) const
+void CairoTokenRenderer::applyCairoMask(cairo_t* cr,int32_t xOffset,int32_t yOffset, float scalex, float scaley) const
 {
-	cairo_matrix_t tmp=matrix;
-	tmp.x0-=xOffset;
-	tmp.y0-=yOffset;
-	cairo_set_matrix(cr, &tmp);
-	cairoPathFromTokens(cr, tokens, scaleFactor, true);
+	cairo_matrix_t mat;
+	cairo_get_matrix(cr,&mat);
+	cairo_translate(cr,xOffset,yOffset);
+	cairo_set_matrix(cr,&mat);
+	
+	cairoPathFromTokens(cr, tokens, scaleFactor, true,scalex,scaley,xstart,ystart,true);
 	cairo_clip(cr);
 }
 
+CairoTokenRenderer::CairoTokenRenderer(const tokensVector &_g, const MATRIX &_m, int32_t _x, int32_t _y, int32_t _w, int32_t _h
+									   , int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r
+									   , float _xs, float _ys, bool _im, bool _hm, float _s, float _a
+									   , const std::vector<IDrawable::MaskData> &_ms
+									   , float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier
+									   , float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset
+									   , bool _smoothing, number_t _xmin, number_t _ymin)
+	: CairoRenderer(_m,_x,_y,_w,_h,_rx,_ry,_rw,_rh,_r,_xs,_ys,_im,_hm,_s,_a,_ms
+					, _redMultiplier,_greenMultiplier,_blueMultiplier,_alphaMultiplier
+					, _redOffset,_greenOffset,_blueOffset,_alphaOffset
+					,_smoothing,_xmin,_ymin),tokens(_g)
+{
+}
+
 void CairoRenderer::convertBitmapWithAlphaToCairo(std::vector<uint8_t, reporter_allocator<uint8_t>>& data, uint8_t* inData, uint32_t width,
-		uint32_t height, size_t* dataSize, size_t* stride)
+												  uint32_t height, size_t* dataSize, size_t* stride, bool frompng)
 {
 	*stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
 	*dataSize = *stride * height;
 	data.resize(*dataSize, 0);
 	uint8_t* outData=&data[0];
-	uint32_t* inData32 = (uint32_t*)inData;
 
 	for(uint32_t i = 0; i < height; i++)
 	{
 		for(uint32_t j = 0; j < width; j++)
 		{
 			uint32_t* outDataPos = (uint32_t*)(outData+i*(*stride)) + j;
-			*outDataPos = GINT32_FROM_BE( *(inData32+(i*width+j)) );
+			// PNGs are always decoded in RGBA
+			*outDataPos = frompng ? inData[i*(*stride)+j*4+3]<<24 | inData[i*(*stride)+j*4  ]<<16 | inData[i*(*stride)+j*4+1]<<8 | inData[i*(*stride)+j*4+2]
+								  : inData[i*(*stride)+j*4  ]<<24 | inData[i*(*stride)+j*4+1]<<16 | inData[i*(*stride)+j*4+2]<<8 | inData[i*(*stride)+j*4+3];
 		}
 	}
 }
 
-inline void CairoRenderer::copyRGB15To24(uint8_t* dest, uint8_t* src)
+inline void CairoRenderer::copyRGB15To24(uint32_t& dest, uint8_t* src)
 {
 	// highest bit is ignored
 	uint8_t r = (src[0] & 0x7C) >> 2;
@@ -757,18 +669,20 @@ inline void CairoRenderer::copyRGB15To24(uint8_t* dest, uint8_t* src)
 	g = g*255/31;
 	b = b*255/31;
 
-	dest[0] = r;
-	dest[1] = g;
-	dest[2] = b;
+	dest |= uint32_t(r)<<16;
+	dest |= uint32_t(g)<<8;
+	dest |= uint32_t(b);
 }
 
-inline void CairoRenderer::copyRGB24To24(uint8_t* dest, uint8_t* src)
+inline void CairoRenderer::copyRGB24To24(uint32_t& dest, uint8_t* src)
 {
-	memcpy(dest, src, 3);
+	dest |= uint32_t(src[0])<<16;
+	dest |= uint32_t(src[1])<<8;
+	dest |= uint32_t(src[2]);
 }
 
 void CairoRenderer::convertBitmapToCairo(std::vector<uint8_t, reporter_allocator<uint8_t>>& data, uint8_t* inData, uint32_t width,
-					 uint32_t height, size_t* dataSize, size_t* stride, bool rgb15)
+					 uint32_t height, size_t* dataSize, size_t* stride, uint32_t bpp)
 {
 	*stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
 	*dataSize = *stride * height;
@@ -779,26 +693,26 @@ void CairoRenderer::convertBitmapToCairo(std::vector<uint8_t, reporter_allocator
 		for(uint32_t j = 0; j < width; j++)
 		{
 			uint32_t* outDataPos = (uint32_t*)(outData+i*(*stride)) + j;
-			uint32_t pdata = 0xFF;
-			/* the alpha channel is set to opaque above */
-			uint8_t* rgbData = ((uint8_t*)&pdata)+1;
-			/* copy the RGB bytes to rgbData */
-			if(rgb15)
-				copyRGB15To24(rgbData, inData+(i*width+j)*2);
-			else
-				copyRGB24To24(rgbData, inData+(i*width+j)*3);
-			/* cairo needs this in host endianess */
-			*outDataPos = GINT32_FROM_BE(pdata);
+			// set the alpha channel to opaque
+			uint32_t pdata = uint32_t(0xFF)<<24;
+			// copy the RGB bytes to pdata
+			switch (bpp)
+			{
+				case 2:
+					copyRGB15To24(pdata, inData+(i*width+j)*2);
+					break;
+				case 3:
+					copyRGB24To24(pdata, inData+(i*width+j)*3);
+					break;
+				case 4:
+					copyRGB24To24(pdata, inData+(i*width+j)*4+1);
+					break;
+			}
+			*outDataPos = pdata;
 		}
 	}
 }
 
-#ifdef HAVE_NEW_GLIBMM_THREAD_API
-StaticMutex CairoPangoRenderer::pangoMutex;
-#else
-StaticMutex CairoPangoRenderer::pangoMutex = GLIBMM_STATIC_MUTEX_INIT;
-#endif
-
 void CairoPangoRenderer::pangoLayoutFromData(PangoLayout* layout, const TextData& tData)
 {
 	PangoFontDescription* desc;
@@ -838,12 +752,6 @@ void CairoPangoRenderer::pangoLayoutFromData(PangoLayout* layout, const TextData
 		pango_layout_set_width(layout,PANGO_SCALE*tData.width);
 		pango_layout_set_wrap(layout,PANGO_WRAP_WORD);//I think this is what Adobe does
 	}
-	//In case autoSize is NONE, we also have the height
-	if(tData.autoSize == TextData::AUTO_SIZE::AS_NONE)
-	{
-		pango_layout_set_width(layout,PANGO_SCALE*tData.width);
-		pango_layout_set_height(layout,PANGO_SCALE*tData.height);//TODO:Not sure what Pango does if the text is too long to fit
-	}
 
 	/* setup font description */
 	desc = pango_font_description_new();
@@ -853,26 +761,47 @@ void CairoPangoRenderer::pangoLayoutFromData(PangoLayout* layout, const TextData
 	pango_font_description_free(desc);
 }
 
-void CairoPangoRenderer::executeDraw(cairo_t* cr)
+void CairoPangoRenderer::executeDraw(cairo_t* cr, float /*scalex*/, float /*scaley*/)
 {
-	/* TODO: pango is not fully thread-safe,
-	 * but we may be able to use finer grained locking.
-	 */
-	Locker l(pangoMutex);
 	PangoLayout* layout;
 
 	layout = pango_cairo_create_layout(cr);
 	pangoLayoutFromData(layout, textData);
 
+	int xpos=0;
+	switch(textData.autoSize)
+	{
+		case TextData::AUTO_SIZE::AS_RIGHT:
+			xpos = textData.width-textData.textWidth;
+			break;
+		case TextData::AUTO_SIZE::AS_CENTER:
+			xpos = (textData.width-textData.textWidth)/2;
+			break;
+		default:
+			break;
+	}
+	
 	if(textData.background)
 	{
 		cairo_set_source_rgb (cr, textData.backgroundColor.Red/255., textData.backgroundColor.Green/255., textData.backgroundColor.Blue/255.);
 		cairo_paint(cr);
 	}
 
+	/* text scroll position */
+	int32_t translateX = textData.scrollH;
+	int32_t translateY = 0;
+	if (textData.scrollV > 1)
+	{
+		translateY = -PANGO_PIXELS(lineExtents(layout, textData.scrollV-1).y);
+	}
+
 	/* draw the text */
+	cairo_translate(cr, xpos, 0);
 	cairo_set_source_rgb (cr, textData.textColor.Red/255., textData.textColor.Green/255., textData.textColor.Blue/255.);
+	cairo_translate(cr, translateX, translateY);
 	pango_cairo_show_layout(cr, layout);
+	cairo_translate(cr, -translateX, -translateY);
+	cairo_translate(cr, -xpos, 0);
 
 	if(textData.border)
 	{
@@ -881,14 +810,30 @@ void CairoPangoRenderer::executeDraw(cairo_t* cr)
 		cairo_rectangle(cr, 0, 0, textData.width, textData.height);
 		cairo_stroke_preserve(cr);
 	}
+	if(textData.caretblinkstate)
+	{
+		uint32_t w,h,tw,th;
+		tw=0;
+		tiny_string currenttext = textData.text;
+		if (caretIndex < currenttext.numChars())
+		{
+			textData.text = textData.text.substr(0,caretIndex);
+		}
+		getBounds(textData,w,h,tw,th);
+		textData.text = currenttext;
+		tw+=xpos;
+		cairo_set_source_rgb(cr, 0, 0, 0);
+		cairo_set_line_width(cr, 2);
+		cairo_move_to(cr,tw,2);
+		cairo_line_to(cr,tw, textData.height-2);
+		cairo_stroke_preserve(cr);
+	}
 
 	g_object_unref(layout);
 }
 
 bool CairoPangoRenderer::getBounds(const TextData& _textData, uint32_t& w, uint32_t& h, uint32_t& tw, uint32_t& th)
 {
-	//TODO:check locking
-	Locker l(pangoMutex);
 	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(NULL, CAIRO_FORMAT_ARGB32, 0, 0, 0);
 	cairo_t *cr=cairo_create(cairoSurface);
 
@@ -917,31 +862,100 @@ bool CairoPangoRenderer::getBounds(const TextData& _textData, uint32_t& w, uint3
 	return (h!=0) && (w!=0);
 }
 
-void CairoPangoRenderer::applyCairoMask(cairo_t* cr, int32_t xOffset, int32_t yOffset) const
+PangoRectangle CairoPangoRenderer::lineExtents(PangoLayout *layout, int lineNumber)
+{
+	PangoRectangle rect;
+	memset(&rect, 0, sizeof(PangoRectangle));
+	int i = 0;
+	PangoLayoutIter* lineIter = pango_layout_get_iter(layout);
+	do
+	{
+		if (i == lineNumber)
+		{
+			pango_layout_iter_get_line_extents(lineIter, NULL, &rect);
+			break;
+		}
+
+		i++;
+	} while (pango_layout_iter_next_line(lineIter));
+	pango_layout_iter_free(lineIter);
+
+	return rect;
+}
+
+std::vector<LineData> CairoPangoRenderer::getLineData(const TextData& _textData)
+{
+	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(NULL, CAIRO_FORMAT_ARGB32, 0, 0, 0);
+	cairo_t *cr=cairo_create(cairoSurface);
+
+	PangoLayout* layout;
+	layout = pango_cairo_create_layout(cr);
+	pangoLayoutFromData(layout, _textData);
+
+	int XOffset = _textData.scrollH;
+	int YOffset = PANGO_PIXELS(lineExtents(layout, _textData.scrollV-1).y);
+	std::vector<LineData> data;
+	data.reserve(pango_layout_get_line_count(layout));
+	PangoLayoutIter* lineIter = pango_layout_get_iter(layout);
+	do
+	{
+		PangoRectangle rect;
+		pango_layout_iter_get_line_extents(lineIter, NULL, &rect);
+		PangoLayoutLine* line = pango_layout_iter_get_line(lineIter);
+		data.emplace_back(PANGO_PIXELS(rect.x) - XOffset,
+				  PANGO_PIXELS(rect.y) - YOffset,
+				  PANGO_PIXELS(rect.width),
+				  PANGO_PIXELS(rect.height),
+				  _textData.text.bytePosToIndex(line->start_index),
+				  _textData.text.substr_bytes(line->start_index, line->length).numChars(),
+				  PANGO_PIXELS(PANGO_ASCENT(rect)),
+				  PANGO_PIXELS(PANGO_DESCENT(rect)),
+				  PANGO_PIXELS(PANGO_LBEARING(rect)),
+				  0); // FIXME
+	} while (pango_layout_iter_next_line(lineIter));
+	pango_layout_iter_free(lineIter);
+
+	g_object_unref(layout);
+	cairo_destroy(cr);
+	cairo_surface_destroy(cairoSurface);
+
+	return data;
+}
+
+void CairoPangoRenderer::applyCairoMask(cairo_t* cr, int32_t xOffset, int32_t yOffset, float scalex, float scaley) const
 {
 	assert(false);
 }
 
-AsyncDrawJob::AsyncDrawJob(IDrawable* d, _R<DisplayObject> o):drawable(d),owner(o),surfaceBytes(NULL),uploadNeeded(false)
+AsyncDrawJob::AsyncDrawJob(IDrawable* d, _R<DisplayObject> o):drawable(d),owner(o),surfaceBytes(nullptr),uploadNeeded(false)
 {
 }
 
 AsyncDrawJob::~AsyncDrawJob()
 {
+	owner->getSystemState()->AsyncDrawJobCompleted(this);
 	delete drawable;
-	delete[] surfaceBytes;
+	if (surfaceBytes)
+		delete[] surfaceBytes;
 }
 
 void AsyncDrawJob::execute()
 {
-	surfaceBytes=drawable->getPixelBuffer();
-	if(surfaceBytes)
+	SystemState* sys = owner->getSystemState();
+	float scalex;
+	float scaley;
+	int offx,offy;
+	sys->stageCoordinateMapping(sys->getRenderThread()->windowWidth,sys->getRenderThread()->windowHeight,offx,offy, scalex,scaley);
+
+	if(!threadAborting)
+		surfaceBytes=drawable->getPixelBuffer(scalex,scaley);
+	if(!threadAborting && surfaceBytes)
 		uploadNeeded=true;
 }
 
 void AsyncDrawJob::threadAbort()
 {
-	//Nothing special to be done
+	uploadNeeded=false;
 }
 
 void AsyncDrawJob::jobFence()
@@ -949,7 +963,10 @@ void AsyncDrawJob::jobFence()
 	//If the data must be uploaded (there were no errors) the Job add itself to the upload queue.
 	//Otherwise it destroys itself
 	if(uploadNeeded)
-		getSys()->getRenderThread()->addUploadJob(this);
+	{
+		uploadNeeded=false;
+		owner->getSystemState()->getRenderThread()->addUploadJob(this);
+	}
 	else
 		delete this;
 }
@@ -974,16 +991,44 @@ const TextureChunk& AsyncDrawJob::getTexture()
 	uint32_t width=drawable->getWidth();
 	uint32_t height=drawable->getHeight();
 	//Verify that the texture is large enough
-	if(!surface.tex.resizeIfLargeEnough(width, height))
-		surface.tex=getSys()->getRenderThread()->allocateTexture(width, height,false);
+	if (!surface.tex)
+	{
+		surface.tex=new TextureChunk();
+		surface.isChunkOwner=true;
+	}
+	if(!surface.tex->resizeIfLargeEnough(width, height))
+		*surface.tex=owner->getSystemState()->getRenderThread()->allocateTexture(width, height,false);
 	surface.xOffset=drawable->getXOffset();
 	surface.yOffset=drawable->getYOffset();
+	surface.xOffsetTransformed=drawable->getXOffsetTransformed();
+	surface.yOffsetTransformed=drawable->getYOffsetTransformed();
+	surface.widthTransformed=drawable->getWidthTransformed();
+	surface.heightTransformed=drawable->getHeightTransformed();
 	surface.alpha=drawable->getAlpha();
-	return surface.tex;
+	surface.rotation=drawable->getRotation();
+	surface.xscale = drawable->getXScale();
+	surface.yscale = drawable->getYScale();
+	surface.isMask = drawable->getIsMask();
+	surface.hasMask = drawable->getHasMask();
+	surface.redMultiplier=drawable->getRedMultiplier();
+	surface.greenMultiplier=drawable->getGreenMultiplier();
+	surface.blueMultiplier=drawable->getBlueMultiplier();
+	surface.alphaMultiplier=drawable->getAlphaMultiplier();
+	surface.redOffset=drawable->getRedOffset();
+	surface.greenOffset=drawable->getGreenOffset();
+	surface.blueOffset=drawable->getBlueOffset();
+	surface.alphaOffset=drawable->getAlphaOffset();
+	return *surface.tex;
 }
 
 void AsyncDrawJob::uploadFence()
 {
+	// ensure that the owner is moved to freelist in vm thread
+	if (getVm(owner->getSystemState()))
+	{
+		owner->incRef();
+		getVm(owner->getSystemState())->addDeletableObject(owner.getPtr());
+	}
 	delete this;
 }
 
@@ -991,3 +1036,14 @@ void SoftwareInvalidateQueue::addToInvalidateQueue(_R<DisplayObject> d)
 {
 	queue.emplace_back(d);
 }
+
+IDrawable::~IDrawable()
+{
+	auto it = masks.begin();
+	while (it != masks.end())
+	{
+		delete it->m;
+		it->m=nullptr;
+		it++;
+	}
+}
