@@ -20,10 +20,10 @@
 #ifndef BACKENDS_GRAPHICS_H
 #define BACKENDS_GRAPHICS_H 1
 
+#define CHUNKSIZE_REAL 126 // 1 pixel on each side is used for clamping to edge
 #define CHUNKSIZE 128
 
 #include "compat.h"
-#include "backends/lsopengl.h"
 #include <vector>
 #include "swftypes.h"
 #include "threading.h"
@@ -37,86 +37,7 @@ namespace lightspark
 
 class DisplayObject;
 class InvalidateQueue;
-
-class TextureBuffer
-{
-private:
-	GLuint texId;
-	GLenum filtering;
-	uint32_t allocWidth;
-	uint32_t allocHeight;
-	uint32_t width;
-	uint32_t height;
-	uint32_t horizontalAlignment;
-	uint32_t verticalAlignment;
-	bool inited;
-	uint32_t nearestPOT(uint32_t a) const;
-	void setAllocSize(uint32_t w, uint32_t h);
-public:
-	/**
-	  	TextureBuffer constructor
-
-		@param initNow Create right now the texture (can be true only if created inside the Render Thread)
-		@param width The requested width
-		@param height The requested height
-		@param filtering The requested texture filtering from OpenGL enumeration
-	*/
-	TextureBuffer(bool initNow, uint32_t width=0, uint32_t height=0, GLenum filtering=GL_NEAREST);
-	/**
-	  	TextureBuffer destructor
-
-		Destroys the GL resources allocated for this texture
-		@pre Should be run inside the RenderThread or shutdown should be already run
-	*/
-	~TextureBuffer();
-	/**
-	   	Return the texture id
-
-		@ret The OpenGL texture id
-	*/
-	GLuint getId() {return texId;}
-	/**
-	  	Initialize the texture using new values
-
-		@param width The requested width
-		@param height The requested height
-		@param filtering The requested texture filtering from OpenGL enumeration
-		@pre Running inside the RenderThread
-	*/
-	void init(uint32_t width, uint32_t height, GLenum filtering=GL_NEAREST);
-	/**
-	  	Frees the GL resources
-
-		@pre Running inside the RenderThread
-	*/
-	void shutdown();
-	/**
-		Bind as the current texture
-
-		@pre Running inside the RenderThread
-	*/
-	void bind();
-	/**
-		Unbind the current texture
-
-		@pre Running inside the RenderThread
-	*/
-	void unbind();
-	/**
-		Set the given uniform with the coordinate scale of the current texture
-
-		@pre Running inside the RenderThread
-	*/
-	void setTexScale(GLuint uniformLocation);
-
-	void resize(uint32_t width, uint32_t height);
-	/**
-		Request a minimum alignment for width and height
-	*/
-	void setRequestedAlignment(uint32_t w, uint32_t h);
-	uint32_t getAllocWidth() const { return allocWidth;}
-	uint32_t getAllocHeight() const { return allocHeight;}
-};
+class ColorTransform;
 
 class TextureChunk
 {
@@ -134,7 +55,7 @@ private:
 	uint32_t texId;
 	TextureChunk(uint32_t w, uint32_t h);
 public:
-	TextureChunk():chunks(NULL),texId(0),width(0),height(0){}
+	TextureChunk():chunks(nullptr),texId(0),width(0),height(0){}
 	TextureChunk(const TextureChunk& r);
 	TextureChunk& operator=(const TextureChunk& r);
 	~TextureChunk();
@@ -149,13 +70,39 @@ public:
 class CachedSurface
 {
 public:
-	CachedSurface():xOffset(0),yOffset(0),alpha(1.0){}
-	TextureChunk tex;
+	CachedSurface():tex(nullptr),xOffset(0),yOffset(0),xOffsetTransformed(0),yOffsetTransformed(0),widthTransformed(0),heightTransformed(0),alpha(1.0),rotation(0.0),xscale(1.0),yscale(1.0)
+	  , redMultiplier(1.0), greenMultiplier(1.0), blueMultiplier(1.0), alphaMultiplier(1.0), redOffset(0.0), greenOffset(0.0), blueOffset(0.0), alphaOffset(0.0)
+	  ,isMask(false),hasMask(false),isChunkOwner(true){}
+	~CachedSurface()
+	{
+		if (isChunkOwner && tex)
+			delete tex;
+	}
+	TextureChunk* tex;
 	int32_t xOffset;
 	int32_t yOffset;
+	int32_t xOffsetTransformed;
+	int32_t yOffsetTransformed;
+	int32_t widthTransformed;
+	int32_t heightTransformed;
 	float alpha;
+	float rotation;
+	float xscale;
+	float yscale;
+	float redMultiplier;
+	float greenMultiplier;
+	float blueMultiplier;
+	float alphaMultiplier;
+	float redOffset;
+	float greenOffset;
+	float blueOffset;
+	float alphaOffset;
+	bool isMask;
+	bool hasMask;
+	bool isChunkOwner;
 };
 
+
 class ITextureUploadable
 {
 protected:
@@ -199,27 +146,71 @@ protected:
 	   The minimal y coordinate for all the points being drawn, in local coordinates
 	*/
 	int32_t yOffset;
+	int32_t xOffsetTransformed;
+	int32_t yOffsetTransformed;
+	int32_t widthTransformed;
+	int32_t heightTransformed;
+	float rotation;
 	float alpha;
+	float xscale;
+	float yscale;
+	float redMultiplier;
+	float greenMultiplier;
+	float blueMultiplier;
+	float alphaMultiplier;
+	float redOffset;
+	float greenOffset;
+	float blueOffset;
+	float alphaOffset;
+	bool isMask;
+	bool hasMask;
 public:
-	IDrawable(int32_t w, int32_t h, int32_t x, int32_t y, float a, const std::vector<MaskData>& m):
-		masks(m),width(w),height(h),xOffset(x),yOffset(y),alpha(a){}
-	virtual ~IDrawable(){}
+	IDrawable(int32_t w, int32_t h, int32_t x, int32_t y,
+		int32_t rw, int32_t rh, int32_t rx, int32_t ry, float r,
+		float xs, float ys,
+		bool im, bool hm,
+		float a, const std::vector<MaskData>& m,
+		float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier,
+		float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset):
+		masks(m),width(w),height(h),xOffset(x),yOffset(y),xOffsetTransformed(rx),yOffsetTransformed(ry),widthTransformed(rw),heightTransformed(rh),rotation(r),
+		alpha(a),xscale(xs),yscale(ys),
+		redMultiplier(_redMultiplier),greenMultiplier(_greenMultiplier),blueMultiplier(_blueMultiplier),alphaMultiplier(_alphaMultiplier),
+		redOffset(_redOffset),greenOffset(_greenOffset),blueOffset(_blueOffset),alphaOffset(_alphaOffset),
+		isMask(im),hasMask(hm) {}
+	virtual ~IDrawable();
 	/*
 	 * This method returns a raster buffer of the image
 	 * The various implementation are responsible for applying the
 	 * masks
 	 */
-	virtual uint8_t* getPixelBuffer()=0;
+	virtual uint8_t* getPixelBuffer(float scalex,float scaley)=0;
 	/*
 	 * This method creates a cairo path that can be used as a mask for
 	 * another object
 	 */
-	virtual void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY) const = 0;
+	virtual void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY, float scalex, float scaley) const = 0;
 	int32_t getWidth() const { return width; }
 	int32_t getHeight() const { return height; }
+	int32_t getWidthTransformed() const { return widthTransformed; }
+	int32_t getHeightTransformed() const { return heightTransformed; }
 	int32_t getXOffset() const { return xOffset; }
 	int32_t getYOffset() const { return yOffset; }
+	int32_t getXOffsetTransformed() const { return xOffsetTransformed; }
+	int32_t getYOffsetTransformed() const { return yOffsetTransformed; }
+	float getRotation() const { return rotation; }
 	float getAlpha() const { return alpha; }
+	float getXScale() const { return xscale; }
+	float getYScale() const { return yscale; }
+	bool getIsMask() const { return isMask; }
+	bool getHasMask() const { return hasMask; }
+	float getRedMultiplier() const { return redMultiplier; }
+	float getGreenMultiplier() const { return greenMultiplier; }
+	float getBlueMultiplier() const { return blueMultiplier; }
+	float getAlphaMultiplier() const { return alphaMultiplier; }
+	float getRedOffset() const { return redOffset; }
+	float getGreenOffset() const { return greenOffset; }
+	float getBlueOffset() const { return blueOffset; }
+	float getAlphaOffset() const { return alphaOffset; }
 };
 
 class AsyncDrawJob: public IThreadJob, public ITextureUploadable
@@ -251,6 +242,7 @@ public:
 	void sizeNeeded(uint32_t& w, uint32_t& h) const;
 	const TextureChunk& getTexture();
 	void uploadFence();
+	DisplayObject* getOwner() { return owner.getPtr(); }
 };
 
 /**
@@ -265,54 +257,56 @@ protected:
 	   Useful to adapt points defined in pixels and twips (1/20 of pixel)
 	*/
 	const float scaleFactor;
+	bool smoothing;
 	/**
 	  The whole transformation matrix that is applied to the rendered object
 	*/
 	MATRIX matrix;
-	/*
-	 * There are reports (http://lists.freedesktop.org/archives/cairo/2011-September/022247.html)
-	 * that cairo is not threadsafe, and I have encountered some spurious crashes, too.
-	 * So we use a global lock for all cairo calls until this issue is sorted out.
-	 * TODO: CairoRenderes are enqueued as IThreadJobs, therefore this mutex
-	 *       will serialize the thread pool when all thread pool workers are executing CairoRenderers!
-	 */
-	static StaticRecMutex cairoMutex;
+	number_t xstart;
+	number_t ystart;
 	static void cairoClean(cairo_t* cr);
 	cairo_surface_t* allocateSurface(uint8_t*& buf);
-	virtual void executeDraw(cairo_t* cr)=0;
-	static void copyRGB15To24(uint8_t* dest, uint8_t* src);
-	static void copyRGB24To24(uint8_t* dest, uint8_t* src);
+	virtual void executeDraw(cairo_t* cr, float scalex, float scaley)=0;
+	static void copyRGB15To24(uint32_t& dest, uint8_t* src);
+	static void copyRGB24To24(uint32_t& dest, uint8_t* src);
 public:
-	CairoRenderer(const MATRIX& _m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a, const std::vector<MaskData>& m);
+	CairoRenderer(const MATRIX& _m, int32_t _x, int32_t _y, int32_t _w, int32_t _h
+				  , int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r
+				  , float _xs, float _ys
+				  , bool _im, bool _hm
+				  , float _s, float _a, const std::vector<MaskData>& m
+				  , float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier
+				  , float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset
+				  , bool _smoothing,number_t _xstart,number_t _ystart);
 	//IDrawable interface
-	uint8_t* getPixelBuffer();
+	uint8_t* getPixelBuffer(float scalex, float scaley);
 	/*
 	 * Converts data (which is in RGB format) to the format internally used by cairo.
 	 */
 	static void convertBitmapToCairo(std::vector<uint8_t, reporter_allocator<uint8_t>>& data, uint8_t* inData, uint32_t width,
-					 uint32_t height, size_t* dataSize, size_t* stride, bool rgb15);
+					 uint32_t height, size_t* dataSize, size_t* stride, uint32_t bpp);
 	/*
-	 * Converts data (which is in ARGB format) to the format internally used by cairo.
+	 * Converts data (which is in ARGB or RGBA(png) format) to the format internally used by cairo.
 	 */
 	static void convertBitmapWithAlphaToCairo(std::vector<uint8_t, reporter_allocator<uint8_t>>& data, uint8_t* inData, uint32_t width,
-			uint32_t height, size_t* dataSize, size_t* stride);
+			uint32_t height, size_t* dataSize, size_t* stride, bool frompng);
 };
 
 class CairoTokenRenderer : public CairoRenderer
 {
 private:
-	static cairo_pattern_t* FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection);
-	static bool cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken>& tokens, double scaleCorrection, bool skipFill);
+	static cairo_pattern_t* FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection, float scalex, float scaley, bool isMask);
+	static bool cairoPathFromTokens(cairo_t* cr, const tokensVector &tokens, double scaleCorrection, bool skipFill, float scalex, float scaley, number_t xstart, number_t ystart, bool isMask);
 	static void quadraticBezier(cairo_t* cr, double control_x, double control_y, double end_x, double end_y);
 	/*
 	   The tokens to be drawn
 	*/
-	const std::vector<GeomToken> tokens;
+	const tokensVector tokens;
 	/*
 	 * This is run by CairoRenderer::execute()
 	 */
-	void executeDraw(cairo_t* cr);
-	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY) const;
+	void executeDraw(cairo_t* cr, float scalex, float scaley);
+	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY, float scalex, float scaley) const;
 public:
 	/*
 	   CairoTokenRenderer constructor
@@ -324,11 +318,18 @@ public:
 	   @param _s The scale factor to be applied in both the x and y axis
 	   @param _a The alpha factor to be applied
 	   @param _ms The masks that must be applied
+	   @param _smoothing indicates if the tokens should be rendered with antialiasing
 	*/
-	CairoTokenRenderer(const std::vector<GeomToken>& _g, const MATRIX& _m,
+	CairoTokenRenderer(const tokensVector& _g, const MATRIX& _m,
 			int32_t _x, int32_t _y, int32_t _w, int32_t _h,
-		    float _s, float _a, const std::vector<MaskData>& _ms)
-		: CairoRenderer(_m,_x,_y,_w,_h,_s,_a,_ms),tokens(_g){}
+			int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r,
+			float _xs, float _ys,
+			bool _im, bool _hm,
+			float _s, float _a, const std::vector<MaskData>& _ms,
+			float _redMultiplier, float _greenMultiplier, float _blueMultiplier, float _alphaMultiplier,
+			float _redOffset, float _greenOffset, float _blueOffset, float _alphaOffset,
+			bool _smoothing,
+			number_t _xmin, number_t _ymin);
 	/*
 	   Hit testing helper. Uses cairo to find if a point in inside the shape
 
@@ -337,22 +338,26 @@ public:
 	   @param x The X in local coordinates
 	   @param y The Y in local coordinates
 	*/
-	static bool hitTest(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y);
+	static bool hitTest(const tokensVector& tokens, float scaleFactor, number_t x, number_t y);
 };
 
 class TextData
 {
 public:
 	/* the default values are from the spec for flash.text.TextField and flash.text.TextFormat */
-	TextData() : width(100), height(100), textWidth(0), textHeight(0), font("Times New Roman"), background(false), backgroundColor(0xFFFFFF),
+	TextData() : width(100), height(100),leading(0), textWidth(0), textHeight(0), font("Times New Roman"),fontID(UINT32_MAX), scrollH(0), scrollV(1), background(false), backgroundColor(0xFFFFFF),
 		border(false), borderColor(0x000000), multiline(false), textColor(0x000000),
-		autoSize(AS_NONE), fontSize(12), wordWrap(false) {}
+		autoSize(AS_NONE), fontSize(12), wordWrap(false),caretblinkstate(false) {}
 	uint32_t width;
 	uint32_t height;
+	int32_t leading;
 	uint32_t textWidth;
 	uint32_t textHeight;
 	tiny_string text;
 	tiny_string font;
+	uint32_t fontID;
+	int32_t scrollH; // pixels, 0-based
+	int32_t scrollV; // lines, 1-based
 	bool background;
 	RGB backgroundColor;
 	bool border;
@@ -363,34 +368,69 @@ public:
 	AUTO_SIZE autoSize;
 	uint32_t fontSize;
 	bool wordWrap;
+	bool caretblinkstate;
+};
+
+class LineData {
+public:
+	LineData(int32_t x, int32_t y, int32_t _width,
+		 int32_t _height, int32_t _firstCharOffset, int32_t _length,
+		 number_t _ascent, number_t _descent, number_t _leading,
+		 number_t _indent):
+		extents(x, x+_width, y, y+_height), 
+		firstCharOffset(_firstCharOffset), length(_length),
+		ascent(_ascent), descent(_descent), leading(_leading),
+		indent(_indent) {}
+	// position and size
+	RECT extents;
+	// Offset of the first character on this line
+	int32_t firstCharOffset;
+	// length of the line in characters
+	int32_t length;
+	number_t ascent;
+	number_t descent;
+	number_t leading;
+	number_t indent;
 };
 
 class CairoPangoRenderer : public CairoRenderer
 {
-	static StaticMutex pangoMutex;
 	/*
 	 * This is run by CairoRenderer::execute()
 	 */
-	void executeDraw(cairo_t* cr);
+	void executeDraw(cairo_t* cr, float scalex, float scaley);
 	TextData textData;
+	uint32_t caretIndex;
 	static void pangoLayoutFromData(PangoLayout* layout, const TextData& tData);
-	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY) const;
+	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY, float scalex, float scaley) const;
+	static PangoRectangle lineExtents(PangoLayout *layout, int lineNumber);
 public:
 	CairoPangoRenderer(const TextData& _textData, const MATRIX& _m,
-			int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a, const std::vector<MaskData>& _ms)
-		: CairoRenderer(_m,_x,_y,_w,_h,_s,_a,_ms), textData(_textData) {}
+			int32_t _x, int32_t _y, int32_t _w, int32_t _h,
+			int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r,
+			float _xs, float _ys,
+			bool _im, bool _hm,
+			float _s, float _a, const std::vector<MaskData>& _ms,
+			float _redMultiplier, float _greenMultiplier, float _blueMultiplier, float _alphaMultiplier,
+			float _redOffset, float _greenOffset, float _blueOffset, float _alphaOffset,
+			bool _smoothing,number_t _xmin,number_t _ymin,uint32_t _ci)
+		: CairoRenderer(_m,_x,_y,_w,_h,_rx,_ry,_rw,_rh,_r,_xs, _ys,_im,_hm,_s,_a,_ms,
+						_redMultiplier, _greenMultiplier, _blueMultiplier, _alphaMultiplier,
+						_redOffset, _greenOffset, _blueOffset, _alphaOffset,
+						_smoothing,_xmin,_ymin), textData(_textData),caretIndex(_ci) {}
 	/**
 		Helper. Uses Pango to find the size of the textdata
 		@param _texttData The textData being tested
 		@param w,h,tw,th are the (text)width and (text)height of the textData.
 	*/
 	static bool getBounds(const TextData& _textData, uint32_t& w, uint32_t& h, uint32_t& tw, uint32_t& th);
+	static std::vector<LineData> getLineData(const TextData& _textData);
 };
 
 class InvalidateQueue
 {
 public:
-	virtual ~InvalidateQueue(){};
+	virtual ~InvalidateQueue(){}
 	//Invalidation queue management
 	virtual void addToInvalidateQueue(_R<DisplayObject> d) = 0;
 };
@@ -402,5 +442,5 @@ public:
 	void addToInvalidateQueue(_R<DisplayObject> d);
 };
 
-};
+}
 #endif /* BACKENDS_GRAPHICS_H */
