@@ -26,10 +26,9 @@
 #include "platforms/engineutils.h"
 #include "swftypes.h"
 #include "smartrefs.h"
+#include "scripting/flash/ui/keycodes.h"
 #include <vector>
 
-#include "scripting/class.h"
-
 namespace lightspark
 {
 
@@ -39,30 +38,26 @@ class InteractiveObject;
 class Sprite;
 class MouseEvent;
 
-struct KeyNameCodePair {
-	const char *keyname;
-	unsigned int keycode;
-};
-
 class InputThread
 {
+friend class Stage;
 private:
 	SystemState* m_sys;
 	EngineData* engineData;
-	Thread* t;
 	bool terminated;
-	bool threaded;
-	bool worker(GdkEvent *event);
+	// this is called from mainloopthread
+	bool worker(SDL_Event *event);
 
 	std::vector<InteractiveObject* > listeners;
 	Mutex mutexListeners;
 	Mutex mutexDragged;
 
-	std::vector<KeyNameCodePair> keyNamesAndCodes;
-
 	_NR<Sprite> curDragged;
 	_NR<InteractiveObject> currentMouseOver;
 	_NR<InteractiveObject> lastMouseDownTarget;
+	SDL_Keymod lastKeymod;
+	SDL_Keycode lastKeyDown;
+	SDL_Keycode lastKeyUp;
 	const RECT* dragLimit;
 	Vector2f dragOffset;
 	class MaskData
@@ -73,18 +68,20 @@ private:
 		MaskData(DisplayObject* _d, const MATRIX& _m):d(_d),m(_m){}
 	};
 	_NR<InteractiveObject> getMouseTarget(uint32_t x, uint32_t y, DisplayObject::HIT_TYPE type);
-	void handleMouseDown(uint32_t x, uint32_t y, unsigned int buttonState);
-	void handleMouseDoubleClick(uint32_t x, uint32_t y, unsigned int buttonState);
-	void handleMouseUp(uint32_t x, uint32_t y, unsigned int buttonState);
-	void handleMouseMove(uint32_t x, uint32_t y, unsigned int buttonState);
-	void handleScrollEvent(uint32_t x, uint32_t y, GdkScrollDirection direction, unsigned int buttonState);
+	void handleMouseDown(uint32_t x, uint32_t y, SDL_Keymod buttonState,bool pressed);
+	void handleMouseDoubleClick(uint32_t x, uint32_t y, SDL_Keymod buttonState,bool pressed);
+	void handleMouseUp(uint32_t x, uint32_t y, SDL_Keymod buttonState, bool pressed, uint8_t button);
+	void handleMouseMove(uint32_t x, uint32_t y, SDL_Keymod buttonState,bool pressed);
+	void handleScrollEvent(uint32_t x, uint32_t y, uint32_t direction, SDL_Keymod buttonState,bool pressed);
+	void handleMouseLeave();
 
-	void initKeyTable();
-	bool handleKeyboardShortcuts(const GdkEventKey *keyevent);
-	void sendKeyEvent(const GdkEventKey *keyevent);
+	bool handleKeyboardShortcuts(const SDL_KeyboardEvent *keyevent);
+	void sendKeyEvent(const SDL_KeyboardEvent *keyevent);
 
-	Spinlock inputDataSpinlock;
+	bool handleContextMenuEvent(SDL_Event* event);
+	Mutex inputDataSpinlock;
 	Vector2 mousePos;
+	bool button1pressed;
 public:
 	InputThread(SystemState* s);
 	~InputThread();
@@ -94,14 +91,31 @@ public:
 	void removeListener(InteractiveObject* ob);
 	void startDrag(_R<Sprite> s, const RECT* limit, Vector2f dragOffset);
 	void stopDrag(Sprite* s);
-	const std::vector<KeyNameCodePair>& getKeyNamesAndCodes();
 
 	Vector2 getMousePos()
 	{
-		SpinlockLocker locker(inputDataSpinlock);
+		Locker locker(inputDataSpinlock);
 		return mousePos;
 	}
+	bool getLeftButtonPressed()
+	{
+		Locker locker(mutexListeners);
+		if (button1pressed)
+		{
+			button1pressed=false;
+			return true;
+		}
+		return false;
+	}
+	bool handleEvent(SDL_Event *event)
+	{
+		return worker(event);
+	}
+	AS3KeyCode getLastKeyDown();
+	AS3KeyCode getLastKeyUp();
+	SDL_Keycode getLastKeyCode();
+	SDL_Keymod getLastKeyMod();
 };
 
-};
+}
 #endif /* BACKENDS_INPUT_H */
