@@ -21,7 +21,6 @@
 #define BACKENDS_LSOPENGL_H 1
 
 #ifdef ENABLE_GLES2
-	#define SUPPORT_X11 1 //Needed for SGX/OMAP GL stack
 	#include <EGL/egl.h>
 	#include <GLES2/gl2.h>
 	#include <GLES2/gl2ext.h>
@@ -35,18 +34,14 @@
 
 	//there are no multiple buffers in GLES 2.0
 	#define glDrawBuffer(x)
-	#define glBindBuffer(...)
-	#define glBufferData(...)
+//	#define glBindBuffer(...)
+//	#define glBufferData(...)
 	#define glPixelStorei(...)
 #else
 	//GLEW_NO_GLU tells glew.h to not include glu.h. Required to
 	//compile on systems without glu.h.
 	#define GLEW_NO_GLU
 	#include <GL/glew.h>
-	#ifndef _WIN32
-		#include <GL/glx.h>
-	#endif
-	#undef None // conflicts with libxml++ headers
 #endif
 
 #endif /* BACKENDS_LSOPENGL_H */
