@@ -38,9 +38,9 @@
 using namespace std;
 using namespace lightspark;
 
-#define LSGL_MATRIX_SIZE (16*sizeof(GLfloat))
+#define LSGL_MATRIX_SIZE (16*sizeof(float))
 
-const GLfloat RenderContext::lsIdentityMatrix[16] = {
+const float RenderContext::lsIdentityMatrix[16] = {
 								1, 0, 0, 0,
 								0, 1, 0, 0,
 								0, 0, 1, 0,
@@ -54,7 +54,7 @@ RenderContext::RenderContext(CONTEXT_TYPE t):contextType(t)
 	lsglLoadIdentity();
 }
 
-void RenderContext::lsglLoadMatrixf(const GLfloat *m)
+void RenderContext::lsglLoadMatrixf(const float *m)
 {
 	memcpy(lsMVPMatrix, m, LSGL_MATRIX_SIZE);
 }
@@ -64,14 +64,14 @@ void RenderContext::lsglLoadIdentity()
 	lsglLoadMatrixf(lsIdentityMatrix);
 }
 
-void RenderContext::lsglMultMatrixf(const GLfloat *m)
+void RenderContext::lsglMultMatrixf(const float *m)
 {
-	GLfloat tmp[16];
+	float tmp[16];
 	for(int i=0;i<4;i++)
 	{
 		for(int j=0;j<4;j++)
 		{
-			GLfloat sum=0;
+			float sum=0;
 			for (int k=0;k<4;k++)
 			{
 				sum += lsMVPMatrix[i+k*4]*m[j*4+k];
@@ -82,9 +82,9 @@ void RenderContext::lsglMultMatrixf(const GLfloat *m)
 	memcpy(lsMVPMatrix, tmp, LSGL_MATRIX_SIZE);
 }
 
-void RenderContext::lsglScalef(GLfloat scaleX, GLfloat scaleY, GLfloat scaleZ)
+void RenderContext::lsglScalef(float scaleX, float scaleY, float scaleZ)
 {
-	static GLfloat scale[16];
+	static float scale[16];
 
 	memcpy(scale, lsIdentityMatrix, LSGL_MATRIX_SIZE);
 	scale[0] = scaleX;
@@ -93,9 +93,9 @@ void RenderContext::lsglScalef(GLfloat scaleX, GLfloat scaleY, GLfloat scaleZ)
 	lsglMultMatrixf(scale);
 }
 
-void RenderContext::lsglTranslatef(GLfloat translateX, GLfloat translateY, GLfloat translateZ)
+void RenderContext::lsglTranslatef(float translateX, float translateY, float translateZ)
 {
-	static GLfloat trans[16];
+	static float trans[16];
 
 	memcpy(trans, lsIdentityMatrix, LSGL_MATRIX_SIZE);
 	trans[12] = translateX;
@@ -104,9 +104,9 @@ void RenderContext::lsglTranslatef(GLfloat translateX, GLfloat translateY, GLflo
 	lsglMultMatrixf(trans);
 }
 
-void GLRenderContext::lsglOrtho(GLfloat l, GLfloat r, GLfloat b, GLfloat t, GLfloat n, GLfloat f)
+void GLRenderContext::lsglOrtho(float l, float r, float b, float t, float n, float f)
 {
-	GLfloat ortho[16];
+	float ortho[16];
 	memset(ortho, 0, sizeof(ortho));
 	ortho[0] = 2/(r-l);
 	ortho[5] = 2/(t-b);
@@ -124,54 +124,93 @@ const CachedSurface& GLRenderContext::getCachedSurface(const DisplayObject* d) c
 	return d->cachedSurface;
 }
 
-void GLRenderContext::renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h,
-			float alpha, COLOR_MODE colorMode)
+void GLRenderContext::setProperties(AS_BLENDMODE blendmode)
 {
+	// TODO handle other blend modes ,maybe with shaders ? (see https://github.com/jamieowen/glsl-blend)
+	switch (blendmode)
+	{
+		case BLENDMODE_NORMAL:
+			engineData->exec_glBlendFunc(BLEND_ONE,BLEND_ONE_MINUS_SRC_ALPHA);
+			break;
+		case BLENDMODE_MULTIPLY:
+			engineData->exec_glBlendFunc(BLEND_DST_COLOR,BLEND_ONE_MINUS_SRC_ALPHA);
+			break;
+		case BLENDMODE_ADD:
+			engineData->exec_glBlendFunc(BLEND_ONE,BLEND_ONE);
+			break;
+		case BLENDMODE_SCREEN:
+			engineData->exec_glBlendFunc(BLEND_ONE,BLEND_ONE_MINUS_SRC_COLOR);
+			break;
+		default:
+			LOG(LOG_NOT_IMPLEMENTED,"renderTextured of blend mode "<<(int)blendmode);
+			break;
+	}
+}
+void GLRenderContext::renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h,
+			float alpha, COLOR_MODE colorMode, float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
+									 float redMultiplier,float greenMultiplier,float blueMultiplier,float alphaMultiplier,
+									 float redOffset,float greenOffset,float blueOffset,float alphaOffset,
+									 bool isMask, bool hasMask)
+{
+	if (isMask)
+	{
+		engineData->exec_glBindFramebuffer_GL_FRAMEBUFFER(maskframebuffer);
+		engineData->exec_glClearColor(0,0,0,0);
+		engineData->exec_glClear_GL_COLOR_BUFFER_BIT();
+		engineData->exec_glUniform1f(maskUniform, 0);
+	}
+	else
+	{
+		engineData->exec_glUniform1f(maskUniform, hasMask ? 1 : 0);
+	}
 	//Set color mode
-	glUniform1f(yuvUniform, (colorMode==YUV_MODE)?1:0);
+	engineData->exec_glUniform1f(yuvUniform, (colorMode==YUV_MODE)?1:0);
 	//Set alpha
-	glUniform1f(alphaUniform, alpha);
+	engineData->exec_glUniform1f(alphaUniform, alpha);
+	// set rotation, scaling and translation
+	// TODO there may be a more elegant way to handle this, but I'm new to shader programming...
+	engineData->exec_glUniform1f(rotateUniform, rotate*M_PI/180.0);
+	engineData->exec_glUniform2f(beforeRotateUniform, float(w)/2.0,float(h)/2.0);
+	engineData->exec_glUniform2f(afterRotateUniform, float(widthtransformed)/2.0,float(heighttransformed)/2.0);
+	engineData->exec_glUniform2f(startPositionUniform, xtransformed,ytransformed);
+	engineData->exec_glUniform2f(scaleUniform, xscale,yscale);
+	engineData->exec_glUniform4f(colortransMultiplyUniform, redMultiplier,greenMultiplier,blueMultiplier,alphaMultiplier);
+	engineData->exec_glUniform4f(colortransAddUniform, redOffset/255.0,greenOffset/255.0,blueOffset/255.0,alphaOffset/255.0);
 	//Set matrix
 	setMatrixUniform(LSGL_MODELVIEW);
 
-	glBindTexture(GL_TEXTURE_2D, largeTextures[chunk.texId].id);
+	engineData->exec_glBindTexture_GL_TEXTURE_2D(largeTextures[chunk.texId].id);
 	const uint32_t blocksPerSide=largeTextureSize/CHUNKSIZE;
-	uint32_t startX, startY, endX, endY;
+	float startX, startY, endX, endY;
 	assert(chunk.getNumberOfChunks()==((chunk.width+CHUNKSIZE-1)/CHUNKSIZE)*((chunk.height+CHUNKSIZE-1)/CHUNKSIZE));
 
 	uint32_t curChunk=0;
 	//The 4 corners of each texture are specified as the vertices of 2 triangles,
 	//so there are 6 vertices per quad, two of them duplicated (the diagonal)
 	//Allocate the data on the stack to reduce heap fragmentation
-	GLfloat *vertex_coords = g_newa(GLfloat,chunk.getNumberOfChunks()*12);
-	GLfloat *texture_coords = g_newa(GLfloat,chunk.getNumberOfChunks()*12);
+	float *vertex_coords = g_newa(float,chunk.getNumberOfChunks()*12);
+	float *texture_coords = g_newa(float,chunk.getNumberOfChunks()*12);
 	for(uint32_t i=0, k=0;i<chunk.height;i+=CHUNKSIZE)
 	{
-		startY=h*i/chunk.height;
-		endY=min(h*(i+CHUNKSIZE)/chunk.height,h);
-		//Take yOffset into account
-		startY = (y<0)?startY:y+startY;
-		endY = (y<0)?endY:y+endY;
+		startY=float(h*i)/float(chunk.height);
+		endY=min(float(h*(i+CHUNKSIZE))/float(chunk.height),float(h));
 		for(uint32_t j=0;j<chunk.width;j+=CHUNKSIZE)
 		{
-			startX=w*j/chunk.width;
-			endX=min(w*(j+CHUNKSIZE)/chunk.width,w);
-			//Take xOffset into account
-			startX = (x<0)?startX:x+startX;
-			endX = (x<0)?endX:x+endX;
+			startX=float(w*j)/float(chunk.width);
+			endX=min(float(w*(j+CHUNKSIZE))/float(chunk.width),float(w));
 			const uint32_t curChunkId=chunk.chunks[curChunk];
 			const uint32_t blockX=((curChunkId%blocksPerSide)*CHUNKSIZE);
 			const uint32_t blockY=((curChunkId/blocksPerSide)*CHUNKSIZE);
 			const uint32_t availX=min(int(chunk.width-j),CHUNKSIZE);
 			const uint32_t availY=min(int(chunk.height-i),CHUNKSIZE);
-			float startU=blockX;
-			startU/=largeTextureSize;
-			float startV=blockY;
-			startV/=largeTextureSize;
-			float endU=blockX+availX;
-			endU/=largeTextureSize;
-			float endV=blockY+availY;
-			endV/=largeTextureSize;
+			float startU=blockX + 1;
+			startU/=float(largeTextureSize);
+			float startV=blockY + 1;
+			startV/=float(largeTextureSize);
+			float endU=blockX+availX - 1;
+			endU/=float(largeTextureSize);
+			float endV=blockY+availY - 1;
+			endV/=float(largeTextureSize);
 
 			//Upper-right triangle of the quad
 			texture_coords[k] = startU;
@@ -211,24 +250,24 @@ void GLRenderContext::renderTextured(const TextureChunk& chunk, int32_t x, int32
 		}
 	}
 
-	glVertexAttribPointer(VERTEX_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, vertex_coords);
-	glVertexAttribPointer(TEXCOORD_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, texture_coords);
-	glEnableVertexAttribArray(VERTEX_ATTRIB);
-	glEnableVertexAttribArray(TEXCOORD_ATTRIB);
-	glDrawArrays(GL_TRIANGLES, 0, curChunk*6);
-	glDisableVertexAttribArray(VERTEX_ATTRIB);
-	glDisableVertexAttribArray(TEXCOORD_ATTRIB);
-	handleGLErrors();
+	engineData->exec_glVertexAttribPointer(VERTEX_ATTRIB, 0, vertex_coords,FLOAT_2);
+	engineData->exec_glVertexAttribPointer(TEXCOORD_ATTRIB, 0, texture_coords,FLOAT_2);
+	engineData->exec_glEnableVertexAttribArray(VERTEX_ATTRIB);
+	engineData->exec_glEnableVertexAttribArray(TEXCOORD_ATTRIB);
+	engineData->exec_glDrawArrays_GL_TRIANGLES( 0, curChunk*6);
+	engineData->exec_glDisableVertexAttribArray(VERTEX_ATTRIB);
+	engineData->exec_glDisableVertexAttribArray(TEXCOORD_ATTRIB);
+	if (isMask)
+		engineData->exec_glBindFramebuffer_GL_FRAMEBUFFER(0);
 }
 
-bool GLRenderContext::handleGLErrors()
+int GLRenderContext::errorCount = 0;
+bool GLRenderContext::handleGLErrors() const
 {
-	int errorCount = 0;
-	GLenum err;
+	uint32_t err;
 	while(1)
 	{
-		err=glGetError();
-		if(err!=GL_NO_ERROR)
+		if(engineData && engineData->getGLError(err))
 		{
 			errorCount++;
 			LOG(LOG_ERROR,_("GL error ")<< err);
@@ -246,16 +285,17 @@ bool GLRenderContext::handleGLErrors()
 
 void GLRenderContext::setMatrixUniform(LSGL_MATRIX m) const
 {
-	GLint uni = (m == LSGL_MODELVIEW) ? modelviewMatrixUniform:projectionMatrixUniform;
+	int uni = (m == LSGL_MODELVIEW) ? modelviewMatrixUniform:projectionMatrixUniform;
 
-	glUniformMatrix4fv(uni, 1, GL_FALSE, lsMVPMatrix);
+	engineData->exec_glUniformMatrix4fv(uni, 1, false, lsMVPMatrix);
 }
 
-CairoRenderContext::CairoRenderContext(uint8_t* buf, uint32_t width, uint32_t height):RenderContext(CAIRO)
+CairoRenderContext::CairoRenderContext(uint8_t* buf, uint32_t width, uint32_t height, bool smoothing):RenderContext(CAIRO)
 {
 	cairo_surface_t* cairoSurface=getCairoSurfaceForData(buf, width, height);
 	cr=cairo_create(cairoSurface);
 	cairo_surface_destroy(cairoSurface); /* cr has an reference to it */
+	cairo_set_antialias(cr,smoothing ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
 }
 
 CairoRenderContext::~CairoRenderContext()
@@ -263,8 +303,8 @@ CairoRenderContext::~CairoRenderContext()
 	for(auto it=customSurfaces.begin();it!=customSurfaces.end();it++)
 	{
 		//Delete and reset here the buffer memory stored in chunks
-		delete[] it->second.tex.chunks;
-		it->second.tex.chunks=NULL;
+		delete[] it->second.tex->chunks;
+		it->second.tex->chunks=nullptr;
 	}
 	cairo_destroy(cr);
 }
@@ -309,23 +349,33 @@ void CairoRenderContext::transformedBlit(const MATRIX& m, uint8_t* sourceBuf, ui
 }
 
 void CairoRenderContext::renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h,
-			float alpha, COLOR_MODE colorMode)
+			float alpha, COLOR_MODE colorMode, float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
+			float redMultiplier, float greenMultiplier, float blueMultiplier, float alphaMultiplier,
+			float redOffset, float greenOffset, float blueOffset, float alphaOffset,
+			bool isMask, bool hasMask)
 {
-	//TODO: support alpha, and colorMode
+	if (alpha != 1.0)
+		LOG(LOG_NOT_IMPLEMENTED,"CairoRenderContext.renderTextured alpha not implemented:"<<alpha);
+	if (colorMode != RGB_MODE)
+		LOG(LOG_NOT_IMPLEMENTED,"CairoRenderContext.renderTextured colorMode not implemented:"<<(int)colorMode);
 	uint8_t* buf=(uint8_t*)chunk.chunks;
 	cairo_surface_t* chunkSurface = getCairoSurfaceForData(buf, chunk.width, chunk.height);
 	cairo_pattern_t* chunkPattern = cairo_pattern_create_for_surface(chunkSurface);
 	cairo_surface_destroy(chunkSurface);
 	cairo_pattern_set_filter(chunkPattern, CAIRO_FILTER_BILINEAR);
 	cairo_pattern_set_extend(chunkPattern, CAIRO_EXTEND_NONE);
+	cairo_save(cr);
 	cairo_matrix_t matrix;
-	//TODO: Support scaling
-	cairo_matrix_init_translate(&matrix, -x, -y);
-	cairo_pattern_set_matrix(chunkPattern, &matrix);
+	cairo_matrix_init_translate(&matrix,xtransformed,ytransformed);
+	cairo_matrix_scale(&matrix, xscale, yscale);
+	cairo_matrix_rotate(&matrix,rotate*M_PI/180.0);
+	cairo_set_matrix(cr,&matrix);
+
 	cairo_set_source(cr, chunkPattern);
 	cairo_pattern_destroy(chunkPattern);
-	cairo_rectangle(cr, x, y, w, h);
+	cairo_rectangle(cr, 0, 0, w, h);
 	cairo_fill(cr);
+	cairo_restore(cr);
 }
 
 const CachedSurface& CairoRenderContext::getCachedSurface(const DisplayObject* d) const
@@ -339,11 +389,52 @@ const CachedSurface& CairoRenderContext::getCachedSurface(const DisplayObject* d
 	return ret->second;
 }
 
+void CairoRenderContext::setProperties(AS_BLENDMODE blendmode)
+{
+	switch (blendmode)
+	{
+		case BLENDMODE_NORMAL:
+			break;
+		case BLENDMODE_MULTIPLY:
+			cairo_set_operator(cr,CAIRO_OPERATOR_MULTIPLY);
+			break;
+		case BLENDMODE_ADD:
+			cairo_set_operator(cr,CAIRO_OPERATOR_ADD);
+			break;
+		case BLENDMODE_SCREEN:
+			cairo_set_operator(cr,CAIRO_OPERATOR_SCREEN);
+			break;
+		case BLENDMODE_LAYER:
+			cairo_set_operator(cr,CAIRO_OPERATOR_OVER);
+			break;
+		case BLENDMODE_DARKEN:
+			cairo_set_operator(cr,CAIRO_OPERATOR_DARKEN);
+			break;
+		case BLENDMODE_DIFFERENCE:
+			cairo_set_operator(cr,CAIRO_OPERATOR_DIFFERENCE);
+			break;
+		case BLENDMODE_HARDLIGHT:
+			cairo_set_operator(cr,CAIRO_OPERATOR_HARD_LIGHT);
+			break;
+		case BLENDMODE_LIGHTEN:
+			cairo_set_operator(cr,CAIRO_OPERATOR_LIGHTEN);
+			break;
+		case BLENDMODE_OVERLAY:
+			cairo_set_operator(cr,CAIRO_OPERATOR_OVERLAY);
+			break;
+		default:
+			LOG(LOG_NOT_IMPLEMENTED,"renderTextured of blend mode "<<(int)blendmode);
+			break;
+	}
+}
+
 CachedSurface& CairoRenderContext::allocateCustomSurface(const DisplayObject* d, uint8_t* texBuf)
 {
 	auto ret=customSurfaces.insert(make_pair(d, CachedSurface()));
-	assert(ret.second);
+//	assert(ret.second);
 	CachedSurface& surface=ret.first->second;
-	surface.tex.chunks=(uint32_t*)texBuf;
+	if (surface.tex==nullptr)
+		surface.tex=new TextureChunk();
+	surface.tex->chunks=(uint32_t*)texBuf;
 	return surface;
 }
