@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2011-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2011-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -21,8 +21,8 @@
 #define BACKENDS_RENDERING_CONTEXT_H 1
 
 #include <stack>
-#include "backends/lsopengl.h"
 #include "backends/graphics.h"
+#include "platforms/engineutils.h"
 
 namespace lightspark
 {
@@ -36,11 +36,11 @@ class RenderContext
 {
 protected:
 	/* Modelview matrix manipulation */
-	static const GLfloat lsIdentityMatrix[16];
-	GLfloat lsMVPMatrix[16];
-	std::stack<GLfloat*> lsglMatrixStack;
+	static const float lsIdentityMatrix[16];
+	float lsMVPMatrix[16];
+	std::stack<float*> lsglMatrixStack;
 	~RenderContext(){}
-	void lsglMultMatrixf(const GLfloat *m);
+	void lsglMultMatrixf(const float *m);
 public:
 	enum CONTEXT_TYPE { CAIRO=0, GL };
 	RenderContext(CONTEXT_TYPE t);
@@ -48,9 +48,10 @@ public:
 
 	/* Modelview matrix manipulation */
 	void lsglLoadIdentity();
-	void lsglLoadMatrixf(const GLfloat *m);
-	void lsglScalef(GLfloat scaleX, GLfloat scaleY, GLfloat scaleZ);
-	void lsglTranslatef(GLfloat translateX, GLfloat translateY, GLfloat translateZ);
+	void lsglLoadMatrixf(const float *m);
+	void lsglScalef(float scaleX, float scaleY, float scaleZ);
+	void lsglTranslatef(float translateX, float translateY, float translateZ);
+	void lsglRotatef(float angle);
 
 	enum COLOR_MODE { RGB_MODE=0, YUV_MODE };
 	enum MASK_MODE { NO_MASK = 0, ENABLE_MASK };
@@ -59,21 +60,38 @@ public:
 		Render a quad of given size using the given chunk
 	*/
 	virtual void renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h,
-			float alpha, COLOR_MODE colorMode)=0;
+			float alpha, COLOR_MODE colorMode,float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
+			float redMultiplier, float greenMultiplier, float blueMultiplier, float alphaMultiplier,
+			float redOffset, float greenOffset, float blueOffset, float alphaOffset,
+			bool isMask, bool hasMask)=0;
 	/**
 	 * Get the right CachedSurface from an object
 	 */
 	virtual const CachedSurface& getCachedSurface(const DisplayObject* obj) const=0;
+	virtual void setProperties(AS_BLENDMODE blendmode) = 0;
 };
 
 class GLRenderContext: public RenderContext
 {
+private:
+	static int errorCount;
 protected:
-	GLint projectionMatrixUniform;
-	GLint modelviewMatrixUniform;
+	EngineData* engineData;
+	int projectionMatrixUniform;
+	int modelviewMatrixUniform;
 
-	GLint yuvUniform;
-	GLint alphaUniform;
+	int yuvUniform;
+	int alphaUniform;
+	int rotateUniform;
+	int beforeRotateUniform;
+	int startPositionUniform;
+	int afterRotateUniform;
+	int scaleUniform;
+	int maskUniform;
+	int colortransMultiplyUniform;
+	int colortransAddUniform;
+	uint32_t maskframebuffer;
+	uint32_t maskTextureID;
 
 	/* Textures */
 	Mutex mutexLargeTexture;
@@ -81,7 +99,7 @@ protected:
 	class LargeTexture
 	{
 	public:
-		GLuint id;
+		uint32_t id;
 		uint8_t* bitmap;
 		LargeTexture(uint8_t* b):id(-1),bitmap(b){}
 		~LargeTexture(){/*delete[] bitmap;*/}
@@ -90,27 +108,32 @@ protected:
 
 	~GLRenderContext(){}
 
+public:
 	enum LSGL_MATRIX {LSGL_PROJECTION=0, LSGL_MODELVIEW};
 	/*
 	 * Uploads the current matrix as the specified type.
 	 */
 	void setMatrixUniform(LSGL_MATRIX m) const;
-public:
-	GLRenderContext() : RenderContext(GL), largeTextureSize(0)
+	GLRenderContext() : RenderContext(GL),engineData(NULL), largeTextureSize(0)
 	{
 	}
-	void lsglOrtho(GLfloat l, GLfloat r, GLfloat b, GLfloat t, GLfloat n, GLfloat f);
+	void SetEngineData(EngineData* data) { engineData = data;}
+	void lsglOrtho(float l, float r, float b, float t, float n, float f);
 
 	void renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h,
-			float alpha, COLOR_MODE colorMode);
+			float alpha, COLOR_MODE colorMode, float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
+			float redMultiplier, float greenMultiplier, float blueMultiplier, float alphaMultiplier,
+			float redOffset, float greenOffset, float blueOffset, float alphaOffset,
+			bool isMask, bool hasMask) override;
 	/**
 	 * Get the right CachedSurface from an object
 	 * In the OpenGL case we just get the CachedSurface inside the object itself
 	 */
 	const CachedSurface& getCachedSurface(const DisplayObject* obj) const;
+	void setProperties(AS_BLENDMODE blendmode);
 
 	/* Utility */
-	static bool handleGLErrors();
+	bool handleGLErrors() const;
 };
 
 class CairoRenderContext: public RenderContext
@@ -124,15 +147,19 @@ private:
 	 */
 	static const CachedSurface invalidSurface;
 public:
-	CairoRenderContext(uint8_t* buf, uint32_t width, uint32_t height);
+	CairoRenderContext(uint8_t* buf, uint32_t width, uint32_t height,bool smoothing);
 	virtual ~CairoRenderContext();
 	void renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h,
-			float alpha, COLOR_MODE colorMode);
+			float alpha, COLOR_MODE colorMode, float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
+			float redMultiplier, float greenMultiplier, float blueMultiplier, float alphaMultiplier,
+			float redOffset, float greenOffset, float blueOffset, float alphaOffset,
+			bool isMask, bool hasMask) override;
 	/**
 	 * Get the right CachedSurface from an object
 	 * In the Cairo case we get the right CachedSurface out of the map
 	 */
 	const CachedSurface& getCachedSurface(const DisplayObject* obj) const;
+	void setProperties(AS_BLENDMODE blendmode);
 
 	/**
 	 * The CairoRenderContext acquires the ownership of the buffer
