@@ -64,7 +64,7 @@ SecurityManager::SecurityManager():
  */
 SecurityManager::~SecurityManager()
 {
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 
 	URLPFileMapIt i = pendingURLPFiles.begin();
 	for(; i != pendingURLPFiles.end(); ++i)
@@ -104,7 +104,7 @@ PolicyFile* SecurityManager::addPolicyFile(const URLInfo& url)
  */
 URLPolicyFile* SecurityManager::addURLPolicyFile(const URLInfo& url)
 {
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 
 	URLPolicyFile* file = new URLPolicyFile(url);
 	if(file->isValid())
@@ -128,7 +128,7 @@ URLPolicyFile* SecurityManager::addURLPolicyFile(const URLInfo& url)
  */
 SocketPolicyFile* SecurityManager::addSocketPolicyFile(const URLInfo& url)
 {
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 
 	SocketPolicyFile* file = new SocketPolicyFile(url);
 	if(file->isValid())
@@ -143,7 +143,7 @@ SocketPolicyFile* SecurityManager::addSocketPolicyFile(const URLInfo& url)
 template <class T>
 T* SecurityManager::getPolicyFileByURL(std::multimap<tiny_string, T*>& pendingFiles, std::multimap<tiny_string, T*>& loadedFiles, const URLInfo& url)
 {
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 	std::pair< typename std::multimap<tiny_string, T*>::iterator, typename std::multimap<tiny_string, T*>::iterator > range;
 	typename std::multimap<tiny_string, T*>::iterator i;
 
@@ -205,7 +205,7 @@ SocketPolicyFile* SecurityManager::getSocketPolicyFileByURL(const URLInfo& url)
 template <class T>
 void SecurityManager::loadPolicyFile(std::multimap<tiny_string, T*>& pendingFiles, std::multimap<tiny_string, T*>& loadedFiles, PolicyFile *file)
 {
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 
 	if(pendingFiles.count(file->getURL().getHostname()) > 0)
 	{
@@ -269,7 +269,7 @@ std::list<T*> *SecurityManager::searchPolicyFiles(const URLInfo& url,
 {
 	std::list<T*> *result = new std::list<T*>;
 
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 	//Check if the master policy file is loaded.
 	//If another user-added relevant policy file is already loaded, 
 	//it's master will have already been loaded too (to check if it is allowed).
@@ -483,17 +483,17 @@ void SecurityManager::checkURLStaticAndThrow(const URLInfo& url,
 			allowedSandboxesLocal, restrictLocalDirectory);
 	//Network sandboxes can't access local files (this should be a SecurityErrorEvent)
 	if(evaluationResult == SecurityManager::NA_REMOTE_SANDBOX)
-		throw Class<SecurityError>::getInstanceS("SecurityError: "
+		throw Class<SecurityError>::getInstanceS(getSys(),"SecurityError: "
 				"connect to network");
 	//Local-with-filesystem sandbox can't access network
 	else if(evaluationResult == SecurityManager::NA_LOCAL_SANDBOX)
-		throw Class<SecurityError>::getInstanceS("SecurityError: "
+		throw Class<SecurityError>::getInstanceS(getSys(),"SecurityError: "
 				"connect to local file");
 	else if(evaluationResult == SecurityManager::NA_PORT)
-		throw Class<SecurityError>::getInstanceS("SecurityError: "
+		throw Class<SecurityError>::getInstanceS(getSys(),"SecurityError: "
 				"connect to restricted port");
 	else if(evaluationResult == SecurityManager::NA_RESTRICT_LOCAL_DIRECTORY)
-		throw Class<SecurityError>::getInstanceS("SecurityError: "
+		throw Class<SecurityError>::getInstanceS(getSys(),"SecurityError: "
 				"not allowed to navigate up for local files");
 }
 
@@ -575,10 +575,18 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluatePortURL(const URLInfo
 SecurityManager::EVALUATIONRESULT SecurityManager::evaluatePoliciesURL(const URLInfo& url,
 		bool loadPendingPolicies)
 {
+	//This check doesn't apply to data generation mode
+	if(url.isEmpty())
+		return ALLOWED;
+
 	//This check doesn't apply to local files
 	if(url.getProtocol() == "file" && getSys()->mainClip->getOrigin().getProtocol() == "file")
 		return ALLOWED;
 
+	// no need to check if we trust the source
+	if (getSys()->mainClip->getOrigin().getProtocol() == "file" && sandboxType == LOCAL_TRUSTED)
+		return ALLOWED;
+
 	//Streaming from RTMP is always allowed (see
 	//http://forums.adobe.com/thread/422391)
 	if(url.isRTMP())
@@ -599,7 +607,7 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluatePoliciesURL(const URL
 	//Search for the policy files to check
 	URLPFileList* files = searchURLPolicyFiles(url, loadPendingPolicies);
 
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 
 	//Check the policy files
 	if(files != NULL)
@@ -616,7 +624,7 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluatePoliciesURL(const URL
 		}
 	}
 
-	LOG(LOG_INFO, _("SECURITY: DISALLOWED: No policy file explicitly allowed access"));
+	LOG(LOG_INFO, _("SECURITY: DISALLOWED: No policy file explicitly allowed access:")<<url);
 	delete files;
 
 	return NA_CROSSDOMAIN_POLICY;
@@ -635,7 +643,7 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateSocketConnection(cons
 	//Search for the policy files to check
 	SocketPFileList* files = searchSocketPolicyFiles(url, loadPendingPolicies);
 
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 
 	//Check the policy files
 	if(files != NULL)
@@ -671,6 +679,9 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateSocketConnection(cons
 SecurityManager::EVALUATIONRESULT SecurityManager::evaluateHeader(const URLInfo& url,
 		const tiny_string& header, bool loadPendingPolicies)
 {
+	//This check doesn't apply to data generation mode
+	if (url.isEmpty())
+		return ALLOWED;
 	//This check doesn't apply to local files
 	if(url.getProtocol() == "file" && getSys()->mainClip->getOrigin().getProtocol() == "file")
 		return ALLOWED;
@@ -717,7 +728,7 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateHeader(const URLInfo&
 	//Search for the policy files to check
 	URLPFileList* files = searchURLPolicyFiles(url, loadPendingPolicies);
 
-	RecMutex::Lock l(mutex);
+	Locker l(mutex);
 
 	//Check the policy files
 	if(files != NULL)
@@ -762,7 +773,7 @@ PolicyFile::PolicyFile(URLInfo _url, TYPE _type):
  */
 PolicyFile::~PolicyFile()
 {
-	Mutex::Lock l(mutex);
+	Locker l(mutex);
 	for(list<PolicyAllowAccessFrom*>::iterator i = allowAccessFrom.begin();
 			i != allowAccessFrom.end(); ++i)
 		delete (*i);
@@ -780,6 +791,7 @@ PolicyFile::~PolicyFile()
  */
 void PolicyFile::load()
 {
+	Locker l(mutex);
 	//TODO: support download timeout handling
 	
 	//Invalid URLPolicyFile or already loaded, ignore this call
@@ -793,7 +805,6 @@ void PolicyFile::load()
 	if (!isIgnored())
 		valid = retrievePolicyFile(policy);
 
-	Mutex::Lock l(mutex);
 
 	if (isLoaded())
 	{
@@ -934,7 +945,7 @@ URLPolicyFile::URLPolicyFile(const URLInfo& _url):
  */
 URLPolicyFile::~URLPolicyFile()
 {
-	Mutex::Lock l(mutex);
+	Locker l(mutex);
 	for(list<PolicyAllowHTTPRequestHeadersFrom*>::iterator i = allowHTTPRequestHeadersFrom.begin();
 			i != allowHTTPRequestHeadersFrom.end(); ++i)
 		delete (*i);
@@ -950,7 +961,7 @@ URLPolicyFile::~URLPolicyFile()
  */
 URLPolicyFile* URLPolicyFile::getMasterPolicyFile()
 {
-	Mutex::Lock l(mutex);
+	Locker l(mutex);
 
 	if(isMaster())
 		return this;
@@ -1001,7 +1012,7 @@ bool URLPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 	bool ok = true;
 
 	//No caching needed for this download, we don't expect very big files
-	Downloader* downloader=getSys()->downloadManager->download(url, false, NULL);
+	Downloader* downloader=getSys()->downloadManager->download(url, _MR(new MemoryStreamCache(getSys())), NULL);
 
 	//Wait until the file is fetched
 	downloader->waitForTermination();
@@ -1023,7 +1034,8 @@ bool URLPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 
 	//Policy files must have on of the following content-types to be valid:
 	//text/*, application/xml or application/xhtml+xml
-	tiny_string contentType = downloader->getHeader("content-type");
+	std::list<tiny_string> contenttypelist = downloader->getHeader("content-type").split(';');
+	tiny_string contentType = contenttypelist.size() == 0 ? "" : contenttypelist.front();
 	if(ok && (subtype == HTTP || subtype == HTTPS) && 
 	   contentType.substr(0, 5) != "text/" &&
 	   contentType != "application/xml" &&
@@ -1052,11 +1064,13 @@ bool URLPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 
 	if (ok)
 	{
-		istream s(downloader);
+		std::streambuf *sbuf = downloader->getCache()->createReader();
+		istream s(sbuf);
 		size_t bufLength = downloader->getLength();
 		size_t offset = outData.size();
 		outData.resize(offset+bufLength);
 		s.read((char*)&outData[offset], bufLength);
+		delete sbuf;
 	}
 
 	getSys()->downloadManager->destroy(downloader);
@@ -1116,7 +1130,8 @@ bool URLPolicyFile::allowsAccessFrom(const URLInfo& requestingUrl, const URLInfo
 	for(; i != allowAccessFrom.end(); ++i)
 	{
 		//This allow-access-from entry applies to our domain AND it allows our domain
-		if((*i)->allowsAccessFrom(requestingUrl))
+		// we allow access to https urls even if the main url is not https
+		if((*i)->allowsAccessFrom(requestingUrl,0,requestingUrl.getProtocol()=="https"))
 			return true;
 	}
 	return false;
@@ -1223,8 +1238,6 @@ bool SocketPolicyFile::isMaster() const
 
 SocketPolicyFile* SocketPolicyFile::getMasterPolicyFile()
 {
-	Mutex::Lock l(mutex);
-
 	if(isMaster())
 		return this;
 
@@ -1288,29 +1301,29 @@ bool SocketPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 	const char *socket_policy_cmd = "<policy-file-request/>\0";
 	unsigned int socket_policy_cmd_len = strlen(socket_policy_cmd)+1;
 	ssize_t nbytes = sock.sendAll(socket_policy_cmd, socket_policy_cmd_len);
-	if (nbytes != socket_policy_cmd_len)
+	if (nbytes != (int)socket_policy_cmd_len)
 	{
 		return false;
 	}
 
-	nbytes = 0;
 	do
 	{
 		char buf[4096];
 		nbytes = sock.receive(buf, sizeof buf);
-		outData.insert(outData.end(), buf, buf + nbytes);
+		if (nbytes > 0)
+			outData.insert(outData.end(), buf, buf + nbytes);
 	}
 	while (nbytes > 0);
 
-	if (nbytes < 0)
+	if (nbytes < 0 && outData.size() == 0)
 	{
 		// error reading from socket
 		return false;
 	}
 
 	// The policy file is considered invalid if the last character
-	// is not '\0'
-	if (outData.size() == 0 || outData[outData.size()-1] != '\0')
+	// is not '\0' or '>' or '\n'
+	if (outData.size() == 0 || (outData[outData.size()-1] != '\0' && outData[outData.size()-1] != '>' && outData[outData.size()-1] != '\n'))
 	{
 		return false;
 	}
@@ -1459,7 +1472,7 @@ PolicyAllowAccessFrom::~PolicyAllowAccessFrom()
  * \param url The URL to check this entry against
  * \return \c true if this entry allows the given URL access, otherwise \c false
  */
-bool PolicyAllowAccessFrom::allowsAccessFrom(const URLInfo& url, uint16_t toPort) const
+bool PolicyAllowAccessFrom::allowsAccessFrom(const URLInfo& url, uint16_t toPort, bool bCheckHttps) const
 {
 	//TODO: resolve domain names using DNS before checking for a match?
 	//See section 1.5.9 in specification
@@ -1469,14 +1482,16 @@ bool PolicyAllowAccessFrom::allowsAccessFrom(const URLInfo& url, uint16_t toPort
 		return false;
 
 	//Check if the requesting URL is secure, if needed
-	if(file->getType() == PolicyFile::URL && 
-			dynamic_cast<URLPolicyFile*>(file)->getSubtype() == URLPolicyFile::HTTPS && 
-			secure && url.getProtocol() != "https")
-		return false;
-	if(file->getType() == PolicyFile::SOCKET && 
-			secure && url.getProtocol() != "https")
-		return false;
-
+	if (bCheckHttps)
+	{
+		if(file->getType() == PolicyFile::URL && 
+				dynamic_cast<URLPolicyFile*>(file)->getSubtype() == URLPolicyFile::HTTPS && 
+				secure && url.getProtocol() != "https")
+			return false;
+		if(file->getType() == PolicyFile::SOCKET && 
+				secure && url.getProtocol() != "https")
+			return false;
+	}
 	//Check for to-ports (only applies to SOCKET connections)
 	if(file->getType() == PolicyFile::SOCKET)
 	{
