@@ -17,97 +17,247 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
+#include <glib.h>
 #include "version.h"
 #include "backends/security.h"
+#include "backends/config.h"
 #include "swf.h"
 #include "logger.h"
 #include "platforms/engineutils.h"
-#ifndef _WIN32
-#	include <sys/resource.h>
-#	include <gdk/gdkx.h>
-#endif
 #include "compat.h"
+#include <SDL2/SDL.h>
+#include "flash/utils/ByteArray.h"
+#include <sys/stat.h>
+#include "parsing/streams.h"
 
-
+#ifdef __MINGW32__
+    #ifndef PATH_MAX
+    #define PATH_MAX _MAX_PATH
+    #endif
+    #define realpath(N,R) _fullpath((R),(N),_MAX_PATH)
+#endif
 using namespace std;
 using namespace lightspark;
 
 class StandaloneEngineData: public EngineData
 {
-	guint destroyHandlerId;
-public:
-	StandaloneEngineData() : destroyHandlerId(0)
+	SDL_GLContext mSDLContext;
+	void removedir(const char* dir)
 	{
-#ifndef _WIN32
-		visual = XVisualIDFromVisual(gdk_x11_visual_get_xvisual(gdk_visual_get_system()));
-#endif
+		GDir* d = g_dir_open(dir,0,nullptr);
+		if (!d)
+			return;
+		while (true)
+		{
+			const char* filename = g_dir_read_name (d);
+			if (!filename)
+				return;
+			string path = dir;
+			path += G_DIR_SEPARATOR_S;
+			path += filename;
+			struct stat st;
+			stat(path.c_str(), &st);
+			if(st.st_mode & S_IFDIR)
+				removedir(path.c_str());
+			else
+				::remove(path.c_str());
+		}
+		g_dir_close(d);
+		removedir(dir);
 	}
-	static void destroyWidget(GtkWidget* widget)
+	bool isvalidfilename(const tiny_string& filename)
 	{
-		gdk_threads_enter();
-		gtk_widget_destroy(widget);
-		gdk_threads_leave();
+		if (filename.find("./") != tiny_string::npos || filename == ".." ||
+				filename.find("~") != tiny_string::npos)
+			return false;
+		return true;
+	}
+	size_t getfilesize(const char* filepath)
+	{
+		struct stat st;
+		stat(filepath, &st);
+		return st.st_size;
+	}
+public:
+	StandaloneEngineData(const tiny_string& datapath)
+	{
+		sharedObjectDatapath=datapath;
 	}
 	~StandaloneEngineData()
 	{
-		if(widget)
+		removedir(Config::getConfig()->getDataDirectory().c_str());
+	}
+	SDL_Window* createWidget(uint32_t w, uint32_t h)
+	{
+		SDL_Window* window = SDL_CreateWindow("Lightspark",SDL_WINDOWPOS_UNDEFINED,SDL_WINDOWPOS_UNDEFINED,w,h,SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE);
+		if (window == 0)
 		{
-			if(destroyHandlerId)
-				g_signal_handler_disconnect(widget, destroyHandlerId);
-
-			runInGtkThread(sigc::bind(&destroyWidget,widget));
+			// try creation of window without opengl support
+			window = SDL_CreateWindow("Lightspark",SDL_WINDOWPOS_UNDEFINED,SDL_WINDOWPOS_UNDEFINED,w,h,SDL_WINDOW_RESIZABLE);
+			if (window == 0)
+				LOG(LOG_ERROR,"createWidget failed:"<<SDL_GetError());
 		}
-	}
-	static void StandaloneDestroy(GtkWidget *widget, gpointer data)
-	{
-		StandaloneEngineData* e = reinterpret_cast<StandaloneEngineData*>(data);
-		RecMutex::Lock l(e->mutex);
-		/* no need to destroy it - it's already done */
-		e->widget = NULL;
-		getSys()->setShutdownFlag();
-	}
-	GtkWidget* createGtkWidget()
-	{
-		GtkWidget* window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title((GtkWindow*)window,"Lightspark");
-		destroyHandlerId = g_signal_connect(window,"destroy",G_CALLBACK(StandaloneDestroy),this);
 		return window;
 	}
-	GdkNativeWindow getWindowForGnash()
+	
+	uint32_t getWindowForGnash() override
 	{
 		/* passing and invalid window id to gnash makes
 		 * it create its own window */
 		return 0;
 	}
-	void stopMainDownload() {}
-	bool isSizable() const
+	void stopMainDownload() override {}
+	bool isSizable() const override
 	{
 		return true;
 	}
-        void grabFocus()
+	void grabFocus() override
 	{
 		/* Nothing to do because the standalone main window is
 		 * always focused */
 	}
-        void openPageInBrowser(const tiny_string& url, const tiny_string& window)
+	void openPageInBrowser(const tiny_string& url, const tiny_string& window) override
 	{
 		LOG(LOG_NOT_IMPLEMENTED, "openPageInBrowser not implemented in the standalone mode");
 	}
+	bool getScreenData(SDL_DisplayMode* screen) override
+	{
+		if (SDL_GetDesktopDisplayMode(0, screen) != 0) {
+			LOG(LOG_ERROR,"Capabilities: SDL_GetDesktopDisplayMode failed:"<<SDL_GetError());
+			return false;
+		}
+		return true;
+	}
+	double getScreenDPI() override
+	{
+#if SDL_VERSION_ATLEAST(2, 0, 4)
+		float ddpi;
+		float hdpi;
+		float vdpi;
+		SDL_GetDisplayDPI(SDL_GetWindowDisplayIndex(widget),&ddpi,&hdpi,&vdpi);
+		return ddpi;
+#else
+		LOG(LOG_NOT_IMPLEMENTED,"getScreenDPI needs SDL version >= 2.0.4");
+		return 96.0;
+#endif
+	}
+	void DoSwapBuffers() override
+	{
+		uint32_t err;
+		if (getGLError(err))
+			LOG(LOG_ERROR,"swapbuffers:"<<widget<<" "<<err);
+		SDL_GL_SwapWindow(widget);
+	}
+	void InitOpenGL() override
+	{
+		mSDLContext = SDL_GL_CreateContext(widget);
+		if (!mSDLContext)
+			LOG(LOG_ERROR,"failed to create openGL context:"<<SDL_GetError());
+		initGLEW();
+	}
+	void DeinitOpenGL() override
+	{
+		SDL_GL_DeleteContext(mSDLContext);
+	}
+	bool FileExists(const tiny_string& filename) override
+	{
+		if (!isvalidfilename(filename))
+			return false;
+		std::string p = Config::getConfig()->getDataDirectory();
+		if (p.empty())
+			return false;
+		p += G_DIR_SEPARATOR_S;
+		p += filename.raw_buf();
+		return g_file_test(p.c_str(),G_FILE_TEST_EXISTS);
+	}
+	tiny_string FileRead(const tiny_string& filename) override
+	{
+		if (!isvalidfilename(filename))
+			return "";
+		std::string p = Config::getConfig()->getDataDirectory();
+		if (p.empty())
+			return "";
+
+		p += G_DIR_SEPARATOR_S;
+		p += filename.raw_buf();
+		if (!g_file_test(p.c_str(),G_FILE_TEST_EXISTS))
+			return "";
+		uint32_t len = getfilesize(p.c_str());
+		std::ifstream file;
+		
+		file.open(p, std::ios::in|std::ios::binary);
+		
+		tiny_string res(file,len);
+		file.close();
+		return res;
+	}
+	void FileWrite(const tiny_string& filename, const tiny_string& data) override
+	{
+		if (!isvalidfilename(filename))
+			return;
+		std::string p = Config::getConfig()->getDataDirectory();
+		if (p.empty())
+			return;
+		p += G_DIR_SEPARATOR_S;
+		p += filename.raw_buf();
+		std::ofstream file;
+		
+		file.open(p, std::ios::out|std::ios::binary);
+		file << data;
+		file.close();
+	}
+	void FileReadByteArray(const tiny_string &filename,ByteArray* res) override
+	{
+		if (!isvalidfilename(filename))
+			return;
+		std::string p = Config::getConfig()->getDataDirectory();
+		if (p.empty())
+			return;
+		p += G_DIR_SEPARATOR_S;
+		p += filename.raw_buf();
+		if (!g_file_test(p.c_str(),G_FILE_TEST_EXISTS))
+			return;
+		uint32_t len = getfilesize(p.c_str());
+		std::ifstream file;
+		uint8_t buf[len];
+		file.open(p, std::ios::in|std::ios::binary);
+		file.read((char*)buf,len);
+		res->writeBytes(buf,len);
+		file.close();
+	}
+	
+	void FileWriteByteArray(const tiny_string &filename, ByteArray *data) override
+	{
+		if (!isvalidfilename(filename))
+			return;
+		std::string p = Config::getConfig()->getDataDirectory();
+		if (p.empty())
+			return;
+		p += G_DIR_SEPARATOR_S;
+		p += filename.raw_buf();
+		std::ofstream file;
+		
+		file.open(p, std::ios::out|std::ios::binary|std::ios::trunc);
+		uint8_t* buf = data->getBuffer(data->getLength(),false);
+		file.write((char*)buf,data->getLength());
+		file.close();
+	}
 };
 
 int main(int argc, char* argv[])
 {
-	char* fileName=NULL;
-	char* url=NULL;
-	char* paramsFileName=NULL;
+	char* fileName=nullptr;
+	char* url=nullptr;
+	char* paramsFileName=nullptr;
 #ifdef PROFILING_SUPPORT
-	char* profilingFileName=NULL;
+	char* profilingFileName=nullptr;
 #endif
-	char *HTTPcookie=NULL;
+	char *HTTPcookie=nullptr;
 	SecurityManager::SANDBOXTYPE sandboxType=SecurityManager::LOCAL_WITH_FILE;
 	bool useInterpreter=true;
 	bool useFastInterpreter=false;
 	bool useJit=false;
+	bool ignoreUnhandledExceptions = false;
 	SystemState::ERROR_TYPE exitOnError=SystemState::ERROR_PARSING;
 	LOG_LEVEL log_level=LOG_INFO;
 	SystemState::FLASH_MODE flashMode=SystemState::FLASH;
@@ -123,14 +273,6 @@ int main(int argc, char* argv[])
 
 	LOG(LOG_INFO,"Lightspark version " << VERSION << " Copyright 2009-2013 Alessandro Pignotti and others");
 
-	//Make GTK thread enabled
-#ifdef HAVE_G_THREAD_INIT
-	g_thread_init(NULL);
-#endif
-	gdk_threads_init();
-	//Give GTK a chance to parse its own options
-	gtk_init (&argc, &argv);
-
 	for(int i=1;i<argc;i++)
 	{
 		if(strcmp(argv[i],"-u")==0 || 
@@ -139,7 +281,7 @@ int main(int argc, char* argv[])
 			i++;
 			if(i==argc)
 			{
-				fileName=NULL;
+				fileName=nullptr;
 				break;
 			}
 
@@ -147,18 +289,22 @@ int main(int argc, char* argv[])
 		}
 		else if(strcmp(argv[i],"--air")==0)
 			flashMode=SystemState::AIR;
+		else if(strcmp(argv[i],"--avmplus")==0)
+			flashMode=SystemState::AVMPLUS;
 		else if(strcmp(argv[i],"-ni")==0 || strcmp(argv[i],"--disable-interpreter")==0)
 			useInterpreter=false;
 		else if(strcmp(argv[i],"-fi")==0 || strcmp(argv[i],"--enable-fast-interpreter")==0)
 			useFastInterpreter=true;
 		else if(strcmp(argv[i],"-j")==0 || strcmp(argv[i],"--enable-jit")==0)
 			useJit=true;
+		else if(strcmp(argv[i],"-ne")==0 || strcmp(argv[i],"--ignore-unhandled-exceptions")==0)
+			ignoreUnhandledExceptions=true;
 		else if(strcmp(argv[i],"-l")==0 || strcmp(argv[i],"--log-level")==0)
 		{
 			i++;
 			if(i==argc)
 			{
-				fileName=NULL;
+				fileName=nullptr;
 				break;
 			}
 
@@ -170,7 +316,7 @@ int main(int argc, char* argv[])
 			i++;
 			if(i==argc)
 			{
-				fileName=NULL;
+				fileName=nullptr;
 				break;
 			}
 			paramsFileName=argv[i];
@@ -194,7 +340,7 @@ int main(int argc, char* argv[])
 			i++;
 			if(i==argc)
 			{
-				fileName=NULL;
+				fileName=nullptr;
 				break;
 			}
 			if(strncmp(argv[i], "remote", 6) == 0)
@@ -215,12 +361,17 @@ int main(int argc, char* argv[])
 		{
 			exitOnError = SystemState::ERROR_ANY;
 		}
+		else if(strcmp(argv[i],"--disable-rendering")==0)
+		{
+			EngineData::enablerendering = false;
+		}
+		
 		else if(strcmp(argv[i],"--HTTP-cookies")==0)
 		{
 			i++;
 			if(i==argc)
 			{
-				fileName=NULL;
+				fileName=nullptr;
 				break;
 			}
 			HTTPcookie=argv[i];
@@ -230,37 +381,34 @@ int main(int argc, char* argv[])
 			//No options flag, so set the swf file name
 			if(fileName) //If already set, exit in error status
 			{
-				fileName=NULL;
+				fileName=nullptr;
 				break;
 			}
 			fileName=argv[i];
 		}
 	}
 
-	if(fileName==NULL)
+	if(fileName==nullptr)
 	{
 		LOG(LOG_ERROR, "Usage: " << argv[0] << " [--url|-u http://loader.url/file.swf]" <<
-			" [--disable-interpreter|-ni] [--enable-fast-interpreter|-fi] [--enable-jit|-j]" <<
+			" [--disable-interpreter|-ni] [--enable-fast-interpreter|-fi]" <<
+#ifdef LLVM_ENABLED
+			" [--enable-jit|-j]" <<
+#endif
 			" [--log-level|-l 0-4] [--parameters-file|-p params-file] [--security-sandbox|-s sandbox]" <<
-			" [--exit-on-error] [--HTTP-cookies cookie] [--air]" <<
+			" [--exit-on-error] [--HTTP-cookies cookie] [--air] [--avmplus] [--disable-rendering]" <<
 #ifdef PROFILING_SUPPORT
 			" [--profiling-output|-o profiling-file]" <<
 #endif
+			" [--ignore-unhandled-exceptions|-ne]"
+			" [--version|-v]" <<
 			" <file.swf>");
 		exit(1);
 	}
 
-#ifndef _WIN32
-	struct rlimit rl;
-	getrlimit(RLIMIT_AS,&rl);
-	rl.rlim_cur=400000000;
-	rl.rlim_max=rl.rlim_cur;
-	//setrlimit(RLIMIT_AS,&rl);
-
-#endif
-
 	Log::setLogLevel(log_level);
-	ifstream f(fileName, ios::in|ios::binary);
+	lsfilereader r(fileName);
+	istream f(&r);
 	f.seekg(0, ios::end);
 	uint32_t fileSize=f.tellg();
 	f.seekg(0, ios::beg);
@@ -273,12 +421,19 @@ int main(int argc, char* argv[])
 	cout.exceptions( ios::failbit | ios::badbit);
 	cerr.exceptions( ios::failbit | ios::badbit);
 	SystemState::staticInit();
-	EngineData::startGTKMain();
+	if (!EngineData::startSDLMain())
+	{
+		LOG(LOG_ERROR,"SDL initialization failed, aborting");
+		SystemState::staticDeinit();
+		exit(3);
+	}
 	//NOTE: see SystemState declaration
 	SystemState* sys = new SystemState(fileSize, flashMode);
 	ParseThread* pt = new ParseThread(f, sys->mainClip);
 	setTLSSys(sys);
 	sys->setDownloadedPath(fileName);
+	sys->allowFullscreen=true;
+	sys->allowFullscreenInteractive=true;
 
 	//This setting allows qualifying filename-only paths to fully qualified paths
 	//When the URL parameter is set, set the root URL to the given parameter
@@ -286,13 +441,18 @@ int main(int argc, char* argv[])
 	{
 		sys->mainClip->setOrigin(url, fileName);
 		sys->parseParametersFromURL(sys->mainClip->getOrigin());
-		sandboxType = SecurityManager::REMOTE;
+		if (sandboxType != SecurityManager::REMOTE &&
+		    sys->mainClip->getOrigin().getProtocol() != "file")
+		{
+			LOG(LOG_INFO, _("Switching to remote sandbox because of remote url"));
+			sandboxType = SecurityManager::REMOTE;
+		}
 	}
 #ifndef _WIN32
 	//When running in a local sandbox, set the root URL to the current working dir
 	else if(sandboxType != SecurityManager::REMOTE)
 	{
-		char * cwd = get_current_dir_name();
+		char * cwd = g_get_current_dir();
 		string cwdStr = string("file://") + string(cwd);
 		free(cwd);
 		cwdStr += "/";
@@ -314,6 +474,7 @@ int main(int argc, char* argv[])
 	sys->useInterpreter=useInterpreter;
 	sys->useFastInterpreter=useFastInterpreter;
 	sys->useJit=useJit;
+	sys->ignoreUnhandledExceptions=ignoreUnhandledExceptions;
 	sys->exitOnError=exitOnError;
 	if(paramsFileName)
 		sys->parseParametersFromFile(paramsFileName);
@@ -324,7 +485,19 @@ int main(int argc, char* argv[])
 	if(HTTPcookie)
 		sys->setCookies(HTTPcookie);
 
-	sys->setParamsAndEngine(new StandaloneEngineData(), true);
+	// create path for shared object local storage
+	char absolutepath[PATH_MAX];
+	realpath(fileName,absolutepath);
+	tiny_string homedir(g_get_home_dir());
+	tiny_string filedatapath = absolutepath;
+	if (filedatapath.find(homedir) == 0) // remove home dir, if file is located below home dir
+		filedatapath = filedatapath.substr_bytes(homedir.numBytes(),UINT32_MAX);
+	tiny_string sharedobjectdatapath = Config::getConfig()->getCacheDirectory();
+	sharedobjectdatapath += G_DIR_SEPARATOR_S;
+	sharedobjectdatapath += "data";
+	sharedobjectdatapath += filedatapath;
+
+	sys->setParamsAndEngine(new StandaloneEngineData(sharedobjectdatapath), true);
 
 	sys->securityManager->setSandboxType(sandboxType);
 	if(sandboxType == SecurityManager::REMOTE)
@@ -345,11 +518,18 @@ int main(int argc, char* argv[])
 	 * SystemState::setShutdownFlag.
 	 */
 	sys->destroy();
+	bool isonerror = sys->exitOnError==SystemState::ERROR_ANY && sys->isOnError();
+	SDL_Event event;
+	SDL_zero(event);
+	event.type = LS_USEREVENT_QUIT;
+	SDL_PushEvent(&event);
+	SDL_WaitThread(EngineData::mainLoopThread,nullptr);
+
 	delete pt;
 	delete sys;
 
 	SystemState::staticDeinit();
-	EngineData::quitGTKMain();
-	return 0;
+	
+	return isonerror ? 1 : 0;
 }
 
