@@ -46,8 +46,36 @@ enum markers_type
 	xml_doc_marker = 0x7,
 	date_marker = 0x8,
 	array_marker = 0x9,
-	object_marker = 0xa,
-	xml_marker = 0xb
+	object_marker = 0xA,
+	xml_marker = 0xB,
+	byte_array_marker = 0x0C,
+	vector_int_marker = 0x0D,
+	vector_uint_marker = 0x0E,
+	vector_double_marker = 0x0F,
+	vector_object_marker = 0x10,
+	dictionary_marker = 0x11
+};
+
+enum amf0_markers_type
+{
+	amf0_number_marker       = 0x00,
+	amf0_boolean_marker      = 0x01,
+	amf0_string_marker       = 0x02,
+	amf0_object_marker       = 0x03,
+	amf0_movieclip_marker    = 0x04,
+	amf0_null_marker         = 0x05,
+	amf0_undefined_marker    = 0x06,
+	amf0_reference_marker    = 0x07,
+	amf0_ecma_array_marker   = 0x08,
+	amf0_object_end_marker   = 0x09,
+	amf0_strict_array_marker = 0x0A,
+	amf0_date_marker         = 0x0B,
+	amf0_long_string_marker  = 0x0C,
+	amf0_unsupported_marker  = 0x0D,
+	amf0_recordset_marker    = 0x0E,
+	amf0_xml_document_marker = 0x0F,
+	amf0_typed_object_marker = 0x10,
+	amf0_avmplus_object_marker = 0x11
 };
 
 class TraitsRef
@@ -64,22 +92,43 @@ class Amf3Deserializer
 private:
 	ByteArray* input;
 	tiny_string parseStringVR(std::vector<tiny_string>& stringMap) const;
-	_R<ASObject> parseObject(std::vector<tiny_string>& stringMap,
-			std::vector<ASObject*>& objMap,
+	
+	asAtom parseObject(std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
 			std::vector<TraitsRef>& traitsMap) const;
-	_R<ASObject> parseArray(std::vector<tiny_string>& stringMap,
-			std::vector<ASObject*>& objMap,
+	asAtom parseArray(std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
 			std::vector<TraitsRef>& traitsMap) const;
-	_R<ASObject> parseValue(std::vector<tiny_string>& stringMap,
-			std::vector<ASObject*>& objMap,
+	asAtom parseVector(uint8_t marker, std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
 			std::vector<TraitsRef>& traitsMap) const;
-	_R<ASObject> parseInteger() const;
-	_R<ASObject> parseDouble() const;
-	_R<ASObject> parseXML(std::vector<ASObject*>& objMap, bool legacyXML) const;
+	asAtom parseDictionary(std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
+			std::vector<TraitsRef>& traitsMap) const;
+	asAtom parseByteArray(std::vector<asAtom>& objMap) const;
+	asAtom parseValue(std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
+			std::vector<TraitsRef>& traitsMap) const;
+	asAtom parseInteger() const;
+	asAtom parseDouble() const;
+	asAtom parseDate() const;
+	asAtom parseXML(std::vector<asAtom>& objMap, bool legacyXML) const;
+
+
+	asAtom parseECMAArrayAMF0(std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
+			std::vector<TraitsRef>& traitsMap) const;
+	asAtom parseStrictArrayAMF0(std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
+			std::vector<TraitsRef>& traitsMap) const;
+	asAtom parseObjectAMF0(std::vector<tiny_string>& stringMap,
+			std::vector<asAtom>& objMap,
+			std::vector<TraitsRef>& traitsMap, const tiny_string& clsname="") const;
 public:
 	Amf3Deserializer(ByteArray* i):input(i) {}
-	_R<ASObject> readObject() const;
+	asAtom readObject() const;
+	tiny_string parseStringAMF0() const;
 };
 
-};
+}
 #endif /* PARSING_AMF3_GENERATOR_H */
