@@ -27,6 +27,9 @@
 #include <cstring>
 #include <assert.h>
 
+
+extern lightspark::SystemState* getSys();
+
 #define LZMA_PROP_LENGTH 5
 
 using namespace std;
@@ -112,7 +115,7 @@ int zlib_filter::fillBuffer()
 	return sizeof(buffer) - strm.avail_out;
 }
 
-bytes_buf::bytes_buf(const uint8_t* b, int l):buf(b),offset(0),len(l)
+bytes_buf::bytes_buf(const uint8_t* b, int l):buf(b),len(l)
 {
 	setg((char*)buf,(char*)buf,(char*)buf+len);
 }
@@ -206,108 +209,7 @@ int liblzma_filter::fillBuffer()
 	return sizeof(buffer) - strm.avail_out;
 }
 
-memorystream::memorystream(const char* const b, unsigned int l)
-  : code(b), len(l), pos(0), read_past_end(false)
+void memorystream::handleError(const char* msg)
 {
-}
-
-unsigned int memorystream::tellg() const
-{
-	return pos;
-}
-
-void memorystream::seekg(unsigned int offset)
-{
-	if (offset > len)
-		pos = len;
-	else
-		pos = offset;
-}
-
-unsigned int memorystream::size() const
-{
-	return len;
-}
-
-void memorystream::read(char *out, unsigned int nbytes)
-{
-	if ((nbytes == 1) && (pos+1 > len))
-	{
-		// fastpath for one bytes reads
-		*out = code[pos];
-		pos++;
-	}
-	else if (pos+nbytes > len)
-	{
-		memcpy(out, code+pos, len-pos);
-		pos = len;
-		read_past_end = true;
-	}
-	else
-	{
-		memcpy(out, code+pos, nbytes);
-		pos += nbytes;
-	}
-}
-
-bool memorystream::eof() const
-{
-	return read_past_end;
-}
-
-memorystream& lightspark::operator>>(memorystream& in, lightspark::u8& v)
-{
-	uint8_t t;
-	in.read((char*)&t,1);
-	v.val= t;
-	return in;
-}
-
-memorystream& lightspark::operator>>(memorystream& in, lightspark::s24& v)
-{
-	uint32_t ret=0;
-	in.read((char*)&ret,3);
-	v.val=LittleEndianToSignedHost24(ret);
-	return in;
-}
-
-memorystream& lightspark::operator>>(memorystream& in, lightspark::u30& v)
-{
-	lightspark::u32 vv;
-	in >> vv;
-	uint32_t val = vv;
-	if(val&0xc0000000)
-		throw lightspark::Class<lightspark::VerifyError>::getInstanceS("Invalid u30");
-	v.val = val;
-	return in;
-}
-
-memorystream& lightspark::operator>>(memorystream& in, lightspark::u32& v)
-{
-	int i=0;
-	uint32_t val=0;
-	uint8_t t;
-	do
-	{
-		in.read((char*)&t,1);
-		//No more than 5 bytes should be read
-		if(i==28)
-		{
-			//Only the first 4 bits should be used to reach 32 bits
-			if((t&0xf0))
-				LOG(LOG_ERROR,"Error in u32");
-			uint8_t t2=(t&0xf);
-			val|=(t2<<i);
-			break;
-		}
-		else
-		{
-			uint8_t t2=(t&0x7f);
-			val|=(t2<<i);
-			i+=7;
-		}
-	}
-	while(t&0x80);
-	v.val=val;
-	return in;
+	throw lightspark::Class<lightspark::VerifyError>::getInstanceS(lightspark::getSys(),msg);
 }
