@@ -25,13 +25,18 @@
 #include <iostream>
 #include "swftypes.h"
 #include "backends/geometry.h"
-#include "scripting/flash/utils/flashutils.h"
-#include "scripting/class.h"
+#include "backends/decoder.h"
+#include "scripting/flash/display/flashdisplay.h"
 
 namespace lightspark
 {
+class Class_base;
+class RootMovieClip;
+class DisplayObjectContainer;
+class DefineSpriteTag;
+class AdditionalDataTag;
 
-enum TAGTYPE {TAG=0,DISPLAY_LIST_TAG,SHOW_TAG,CONTROL_TAG,DICT_TAG,FRAMELABEL_TAG,SYMBOL_CLASS_TAG,ABC_TAG,END_TAG};
+enum TAGTYPE {TAG=0,DISPLAY_LIST_TAG,SHOW_TAG,CONTROL_TAG,DICT_TAG,FRAMELABEL_TAG,SYMBOL_CLASS_TAG,ACTION_TAG,ABC_TAG,END_TAG,AVM1ACTION_TAG,AVM1INITACTION_TAG};
 
 void ignore(std::istream& i, int count);
 
@@ -53,15 +58,15 @@ class EndTag:public Tag
 {
 public:
 	EndTag(RECORDHEADER h, std::istream& s):Tag(h){}
-	virtual TAGTYPE getType() const{ return END_TAG; }
+	TAGTYPE getType() const override { return END_TAG; }
 };
 
 class DisplayListTag: public Tag
 {
 public:
 	DisplayListTag(RECORDHEADER h):Tag(h){}
-	virtual TAGTYPE getType() const{ return DISPLAY_LIST_TAG; }
-	virtual void execute(DisplayObjectContainer* parent) const=0;
+	TAGTYPE getType() const override { return DISPLAY_LIST_TAG; }
+	virtual void execute(DisplayObjectContainer* parent,bool inskipping) =0;
 };
 
 class DictionaryTag: public Tag
@@ -75,11 +80,16 @@ public:
 	   the first one is reported here. This behaviour is tested.
 	*/
 	Class_base* bindedTo;
+	tiny_string bindingclassname;
 	RootMovieClip* loadedFrom;
-	DictionaryTag(RECORDHEADER h, RootMovieClip* root):Tag(h),bindedTo(NULL),loadedFrom(root){ }
-	virtual TAGTYPE getType()const{ return DICT_TAG; }
+	uint32_t nameID;
+	DictionaryTag(RECORDHEADER h, RootMovieClip* root):Tag(h),bindedTo(nullptr),loadedFrom(root),nameID(UINT32_MAX) { }
+	TAGTYPE getType() const override { return DICT_TAG; }
 	virtual int getId() const=0;
-	virtual ASObject* instance(Class_base* c=NULL) const { return NULL; };
+	virtual ASObject* instance(Class_base* c=nullptr) { return nullptr; }
+	virtual MATRIX MapToBounds(const MATRIX& mat) { return mat; }
+	virtual MATRIX MapToBoundsForButton(const MATRIX& mat) { return MapToBounds(mat); }
+	virtual void resizeCompleted() {}
 };
 
 /*
@@ -90,35 +100,69 @@ class ControlTag: public Tag
 {
 public:
 	ControlTag(RECORDHEADER h):Tag(h){}
-	virtual TAGTYPE getType()const{ return CONTROL_TAG; }
+	TAGTYPE getType() const override { return CONTROL_TAG; }
 	virtual void execute(RootMovieClip* root) const=0;
 };
 
+/*
+ * Initiates an action. Action is executed after a frame is parsed.
+ */
+class ActionTag: public ControlTag
+{
+public:
+	ActionTag(RECORDHEADER h):ControlTag(h){}
+	TAGTYPE getType() const override { return ACTION_TAG; }
+	void execute(RootMovieClip* root) const override =0;
+};
+
+class AVM1ActionTag: public Tag
+{
+private:
+	std::vector<uint8_t> actions;
+	uint32_t startactionpos;
+public:
+	AVM1ActionTag(RECORDHEADER h, std::istream& s,RootMovieClip* root, AdditionalDataTag* datatag);
+	TAGTYPE getType() const override { return AVM1ACTION_TAG; }
+	void execute(MovieClip* clip, AVM1context *context);
+	bool empty() { return actions.empty(); }
+};
+class AVM1InitActionTag: public ControlTag
+{
+private:
+	UI16_SWF SpriteId;
+	std::vector<uint8_t> actions;
+	uint32_t startactionpos;
+public:
+	AVM1InitActionTag(RECORDHEADER h, std::istream& s,RootMovieClip* root, AdditionalDataTag* datatag);
+	TAGTYPE getType() const override { return AVM1INITACTION_TAG; }
+	void execute(RootMovieClip* root) const override;
+	bool empty() { return actions.empty(); }
+	void executeDirect(MovieClip *clip) const;
+};
+
 class DefineShapeTag: public DictionaryTag
 {
+friend class Shape;
 protected:
 	UI16_SWF ShapeId;
 	RECT ShapeBounds;
 	SHAPEWITHSTYLE Shapes;
-	/* tokens are computed from Shapes */
-	tokensVector tokens;
+	tokensVector* tokens;
+	TextureChunk chunk;
 	DefineShapeTag(RECORDHEADER h,int v,RootMovieClip* root);
 public:
 	DefineShapeTag(RECORDHEADER h,std::istream& in, RootMovieClip* root);
+	~DefineShapeTag();
 	virtual int getId() const{ return ShapeId; }
-	ASObject* instance(Class_base* c=NULL) const
-	{
-		if(c==NULL)
-			c=Class<Shape>::getClass();
-		Shape* ret=new (c->memoryAccount) Shape(c, tokens, 1.0f/20.0f);
-		return ret;
-	}
+	ASObject* instance(Class_base* c=nullptr);
+	MATRIX MapToBoundsForButton(const MATRIX& mat) override;
+	void resizeCompleted() override;
 };
 
 class DefineShape2Tag: public DefineShapeTag
 {
 protected:
-	DefineShape2Tag(RECORDHEADER h, int v, RootMovieClip* root):DefineShapeTag(h,v,root){};
+	DefineShape2Tag(RECORDHEADER h, int v, RootMovieClip* root):DefineShapeTag(h,v,root){}
 public:
 	DefineShape2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 };
@@ -126,7 +170,7 @@ public:
 class DefineShape3Tag: public DefineShape2Tag
 {
 protected:
-	DefineShape3Tag(RECORDHEADER h, int v, RootMovieClip* root):DefineShape2Tag(h,v,root){};
+	DefineShape3Tag(RECORDHEADER h, int v, RootMovieClip* root):DefineShape2Tag(h,v,root){}
 public:
 	DefineShape3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	virtual int getId() const { return ShapeId; }
@@ -146,6 +190,7 @@ public:
 
 class DefineMorphShapeTag: public DictionaryTag
 {
+	friend class TokenContainer;
 protected:
 	UI16_SWF CharacterId;
 	RECT StartBounds;
@@ -158,7 +203,7 @@ protected:
 public:
 	DefineMorphShapeTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	int getId() const { return CharacterId; }
-	virtual ASObject* instance(Class_base* c=NULL) const;
+	virtual ASObject* instance(Class_base* c=NULL);
 };
 
 class DefineMorphShape2Tag: public DefineMorphShapeTag
@@ -208,10 +253,13 @@ private:
 	TextData textData;
 public:
 	DefineEditTextTag(RECORDHEADER h, std::istream& s, RootMovieClip* root);
-	int getId() const { return CharacterID; }
-	ASObject* instance(Class_base* c=NULL) const;
+	int getId() const override { return CharacterID; }
+	ASObject* instance(Class_base* c=nullptr) override;
+	MATRIX MapToBounds(const MATRIX& mat) override;
 };
 
+class MemoryStreamCache;
+
 class DefineSoundTag: public DictionaryTag
 {
 private:
@@ -221,29 +269,65 @@ private:
 	char SoundSize;
 	char SoundType;
 	UI32_SWF SoundSampleCount;
+	_R<MemoryStreamCache> SoundData;
+	int realSampleRate;
 public:
 	DefineSoundTag(RECORDHEADER h, std::istream& s, RootMovieClip* root);
 	virtual int getId() const { return SoundId; }
-	ASObject* instance(Class_base* c=NULL) const;
+	ASObject* instance(Class_base* c=nullptr) override;
+	LS_AUDIO_CODEC getAudioCodec() const;
+	number_t getDurationInMS() const;
+	int getSampleRate() const;
+	int getChannels() const;
+	_R<MemoryStreamCache> getSoundData() const;
+	std::streambuf *createSoundStream() const;
+	_NR<SoundChannel> soundchanel;
+	// indicates if this channel is attached to a Sound object
+	bool isAttached;
 };
 
-class StartSoundTag: public Tag
+class StartSoundTag: public DisplayListTag
 {
+private:
+	UI16_SWF SoundId;
+	SOUNDINFO SoundInfo;
+
+	void play(DefineSoundTag *soundTag);
 public:
 	StartSoundTag(RECORDHEADER h, std::istream& s);
+	void execute(DisplayObjectContainer* parent,bool inskipping) override;
 };
 
-class SoundStreamHeadTag: public Tag
+class SoundStreamHeadTag: public DisplayListTag
+{
+friend class SoundStreamBlockTag;
+private:
+	int StreamSoundCompression;
+	int StreamSoundRate;
+	char StreamSoundSize;
+	char StreamSoundType;
+	UI16_SWF StreamSoundSampleCount;
+	SI16_SWF LatencySeek;
+public:
+	SoundStreamHeadTag(RECORDHEADER h, std::istream& s, RootMovieClip* root,DefineSpriteTag* sprite);
+	_R<MemoryStreamCache> SoundData;
+	void setSoundChannel(Sprite* spr, bool autoplay);
+	void execute(DisplayObjectContainer *parent,bool inskipping) override {}
+};
+
+class SoundStreamBlockTag: public DisplayListTag
 {
 public:
-	SoundStreamHeadTag(RECORDHEADER h, std::istream& s);
+	SoundStreamBlockTag(RECORDHEADER h, std::istream& in, RootMovieClip* root,DefineSpriteTag* sprite);
+	static void decodeSoundBlock(StreamCache *cache, LS_AUDIO_CODEC codec, unsigned char* buf, int len);
+	void execute(DisplayObjectContainer *parent,bool inskipping) override {}
 };
 
 class ShowFrameTag: public Tag
 {
 public:
 	ShowFrameTag(RECORDHEADER h, std::istream& in);
-	virtual TAGTYPE getType()const{ return SHOW_TAG; }
+	TAGTYPE getType() const override { return SHOW_TAG; }
 };
 
 /*class PlaceObjectTag: public Tag
@@ -264,7 +348,7 @@ private:
 
 public:
 	RemoveObject2Tag(RECORDHEADER h, std::istream& in);
-	void execute(DisplayObjectContainer* parent) const;
+	void execute(DisplayObjectContainer* parent,bool inskipping) override;
 };
 
 class PlaceObject2Tag: public DisplayListTag
@@ -281,22 +365,25 @@ protected:
 	UI16_SWF Depth;
 	UI16_SWF CharacterId;
 	MATRIX Matrix;
-	CXFORMWITHALPHA ColorTransform;
+	CXFORMWITHALPHA ColorTransformWithAlpha;
 	UI16_SWF Ratio;
 	UI16_SWF ClipDepth;
 	CLIPACTIONS ClipActions;
-	PlaceObject2Tag(RECORDHEADER h):DisplayListTag(h){}
-	void setProperties(DisplayObject* obj, DisplayObjectContainer* parent) const;
-	const DictionaryTag* placedTag;
+	PlaceObject2Tag(RECORDHEADER h,uint32_t v):DisplayListTag(h),ClipActions(v,nullptr),placedTag(nullptr),NameID(BUILTIN_STRINGS::EMPTY){}
+	virtual void setProperties(DisplayObject* obj, DisplayObjectContainer* parent) const;
+	DictionaryTag* placedTag;
 public:
 	STRING Name;
-	PlaceObject2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	void execute(DisplayObjectContainer* parent) const;
+	uint32_t NameID;
+	PlaceObject2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root, AdditionalDataTag* datatag);
+	void execute(DisplayObjectContainer* parent,bool inskipping) override;
 };
 
 class PlaceObject3Tag: public PlaceObject2Tag
 {
 private:
+	bool PlaceFlagOpaqueBackground;
+	bool PlaceFlagHasVisible;
 	bool PlaceFlagHasImage;
 	bool PlaceFlagHasClassName;
 	bool PlaceFlagHasCacheAsBitmap;
@@ -306,9 +393,12 @@ private:
 	FILTERLIST SurfaceFilterList;
 	UI8 BlendMode;
 	UI8 BitmapCache;
+	UI8 Visible;
+	RGBA BackgroundColor;
 
 public:
 	PlaceObject3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
+	void setProperties(DisplayObject* obj, DisplayObjectContainer* parent) const;
 };
 
 class FrameLabelTag: public Tag
@@ -316,7 +406,7 @@ class FrameLabelTag: public Tag
 public:
 	STRING Name;
 	FrameLabelTag(RECORDHEADER h, std::istream& in);
-	virtual TAGTYPE getType()const{ return FRAMELABEL_TAG; }
+	TAGTYPE getType() const override { return FRAMELABEL_TAG; }
 };
 
 class SetBackgroundColorTag: public ControlTag
@@ -328,24 +418,18 @@ public:
 	void execute(RootMovieClip* root) const;
 };
 
-class SoundStreamHead2Tag: public Tag
-{
-public:
-	SoundStreamHead2Tag(RECORDHEADER h, std::istream& in);
-};
-
 class DefineButtonTag: public DictionaryTag
 {
 private:
 	UI16_SWF ButtonId;
 	UB ReservedFlags;
 	bool TrackAsMenu;
-	UI16_SWF ActionOffset;
 	std::vector<BUTTONRECORD> Characters;
 public:
-	DefineButtonTag(RECORDHEADER h, std::istream& in, int version, RootMovieClip* root);
+	std::vector<BUTTONCONDACTION> condactions;
+	DefineButtonTag(RECORDHEADER h, std::istream& in, int version, RootMovieClip* root, AdditionalDataTag* datatag);
 	virtual int getId() const { return ButtonId; }
-	ASObject* instance(Class_base* c=NULL) const;
+	ASObject* instance(Class_base* c=NULL);
 };
 
 class KERNINGRECORD
@@ -363,14 +447,22 @@ public:
 	DefineBinaryDataTag(RECORDHEADER h,std::istream& s,RootMovieClip* root);
 	~DefineBinaryDataTag() { delete[] bytes; }
 	virtual int getId() const {return Tag;}
-	ASObject* instance(Class_base* c=NULL) const;
+	ASObject* instance(Class_base* c=NULL);
 };
 
 class FontTag: public DictionaryTag
 {
+friend class DefineFontInfoTag;
 protected:
 	UI16_SWF FontID;
 	std::vector<SHAPE> GlyphShapeTable;
+	std::vector<uint16_t> CodeTable;
+	tiny_string fontname;
+	bool FontFlagsSmallText;
+	bool FontFlagsShiftJIS;
+	bool FontFlagsANSI;
+	bool FontFlagsItalic;
+	bool FontFlagsBold;
 public:
 	/* Multiply the coordinates of the SHAPEs by this
 	 * value to get a resolution of 1024*20th pixel
@@ -383,6 +475,10 @@ public:
 		return GlyphShapeTable;
 	}
 	int getId() const { return FontID; }
+	ASObject* instance(Class_base* c=NULL);
+	const tiny_string getFontname() const { return fontname;}
+	virtual void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const=0;
+	bool hasGlyphs(const tiny_string text) const;
 };
 
 class DefineFontTag: public FontTag
@@ -392,13 +488,13 @@ protected:
 	std::vector<uint16_t> OffsetTable;
 public:
 	DefineFontTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	ASObject* instance(Class_base* c=NULL) const;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const;
 };
 
 class DefineFontInfoTag: public Tag
 {
 public:
-	DefineFontInfoTag(RECORDHEADER h, std::istream& in);
+	DefineFontInfoTag(RECORDHEADER h, std::istream& in,RootMovieClip* root);
 };
 
 class DefineFont2Tag: public FontTag
@@ -407,19 +503,11 @@ class DefineFont2Tag: public FontTag
 private:
 	std::vector<uint32_t> OffsetTable;
 	bool FontFlagsHasLayout;
-	bool FontFlagsShiftJIS;
-	bool FontFlagsSmallText;
-	bool FontFlagsANSI;
 	bool FontFlagsWideOffsets;
 	bool FontFlagsWideCodes;
-	bool FontFlagsItalic;
-	bool FontFlagsBold;
 	LANGCODE LanguageCode;
-	UI8 FontNameLen;
-	std::vector <UI8> FontName;
 	UI16_SWF NumGlyphs;
 	uint32_t CodeTableOffset;
-	std::vector <uint16_t> CodeTable;
 	SI16_SWF FontAscent;
 	SI16_SWF FontDescent;
 	SI16_SWF FontLeading;
@@ -430,7 +518,7 @@ private:
 
 public:
 	DefineFont2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	ASObject* instance(Class_base* c=NULL) const;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const;
 };
 
 class DefineFont3Tag: public FontTag
@@ -438,19 +526,11 @@ class DefineFont3Tag: public FontTag
 private:
 	std::vector<uint32_t> OffsetTable;
 	UB FontFlagsHasLayout;
-	UB FontFlagsShiftJIS;
-	UB FontFlagsSmallText;
-	UB FontFlagsANSI;
 	UB FontFlagsWideOffsets;
 	UB FontFlagsWideCodes;
-	UB FontFlagsItalic;
-	UB FontFlagsBold;
 	LANGCODE LanguageCode;
-	UI8 FontNameLen;
-	std::vector <UI8> FontName;
 	UI16_SWF NumGlyphs;
 	uint32_t CodeTableOffset;
-	std::vector <UI16_SWF> CodeTable;
 	SI16_SWF FontAscent;
 	SI16_SWF FontDescent;
 	SI16_SWF FontLeading;
@@ -461,7 +541,7 @@ private:
 
 public:
 	DefineFont3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	ASObject* instance(Class_base* c=NULL) const;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const;
 };
 
 class DefineFont4Tag : public DictionaryTag
@@ -475,7 +555,7 @@ private:
 public:
 	DefineFont4Tag(RECORDHEADER h, std::istream& in,RootMovieClip* root);
 	virtual int getId() const { return FontID; }
-	ASObject* instance(Class_base* c=NULL) const;
+	ASObject* instance(Class_base* c=nullptr);
 };
 
 class DefineTextTag: public DictionaryTag
@@ -494,7 +574,7 @@ public:
 	int version;
 	DefineTextTag(RECORDHEADER h, std::istream& in,RootMovieClip* root,int v=1);
 	int getId() const { return CharacterId; }
-	ASObject* instance(Class_base* c=NULL) const;
+	ASObject* instance(Class_base* c=nullptr);
 };
 
 class DefineText2Tag: public DefineTextTag
@@ -508,18 +588,25 @@ class DefineSpriteTag: public DictionaryTag, public FrameContainer
 private:
 	UI16_SWF SpriteID;
 	UI16_SWF FrameCount;
+	uint32_t soundstartframe;
 public:
+	SoundStreamHeadTag* soundheadtag;
 	DefineSpriteTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	~DefineSpriteTag();
 	virtual int getId() const { return SpriteID; }
-	virtual ASObject* instance(Class_base* c=NULL) const;
+	virtual ASObject* instance(Class_base* c=nullptr);
+	void setSoundStartFrame(uint32_t frame) 
+	{
+		if (soundstartframe == UINT32_MAX)
+			soundstartframe=frame;
+	}
 };
 
 class ProtectTag: public ControlTag
 {
 public:
 	ProtectTag(RECORDHEADER h, std::istream& in);
-	void execute(RootMovieClip* root) const{};
+	void execute(RootMovieClip* root) const{}
 };
 
 class BitmapContainer;
@@ -528,9 +615,10 @@ class BitmapTag: public DictionaryTag
 {
 protected:
         _R<BitmapContainer> bitmap;
+    void loadBitmap(uint8_t* inData, int datasize, const uint8_t *tablesData=nullptr, int tablesLen=0);
 public:
 	BitmapTag(RECORDHEADER h,RootMovieClip* root);
-	ASObject* instance(Class_base* c=NULL) const;
+	ASObject* instance(Class_base* c=nullptr);
         _R<BitmapContainer> getBitmap() const;
 };
 
@@ -596,6 +684,17 @@ public:
 	DefineScalingGridTag(RECORDHEADER h, std::istream& in);
 };
 
+class AdditionalDataTag: public Tag
+{
+public:
+	AdditionalDataTag(RECORDHEADER h, std::istream& in);
+	~AdditionalDataTag()
+	{
+		delete bytes;
+	}
+	uint8_t* bytes;
+	uint32_t numbytes;
+};
 class UnimplementedTag: public Tag
 {
 public:
@@ -612,7 +711,7 @@ public:
 	std::vector<u32> FrameNum;
 	std::vector<STRING> FrameLabel;
 	DefineSceneAndFrameLabelDataTag(RECORDHEADER h, std::istream& in);
-	void execute(RootMovieClip* root) const;
+	void execute(RootMovieClip* root) const override;
 };
 
 class DefineFontNameTag: public Tag
@@ -629,6 +728,7 @@ public:
 
 class DefineVideoStreamTag: public DictionaryTag
 {
+friend class Video;
 private:
 	UI16_SWF CharacterID;
 	UI16_SWF NumFrames;
@@ -637,19 +737,23 @@ private:
 	UB VideoFlagsReserved;
 	UB VideoFlagsDeblocking;
 	UB VideoFlagsSmoothing;
-	UI8 CodecID;
+	UI8 VideoCodecID;
 public:
 	DefineVideoStreamTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	int getId() const{ return CharacterID; }
-	ASObject* instance(Class_base* c=NULL) const;
+	int getId() const override { return CharacterID; }
+	ASObject* instance(Class_base* c=nullptr) override;
 };
-
-class SoundStreamBlockTag: public Tag
+class VideoFrameTag: public DisplayListTag
 {
+private:
+	UI16_SWF StreamID;
+	UI16_SWF FrameNum;
+	uint8_t* framedata;
+	uint32_t numbytes;
 public:
-	SoundStreamBlockTag(RECORDHEADER h, std::istream& in);
+	VideoFrameTag(RECORDHEADER h, std::istream& in);
+	void execute(DisplayObjectContainer* parent,bool inskipping) override;
 };
-
 class MetadataTag: public Tag
 {
 private:
@@ -671,8 +775,8 @@ private:
 	UI16_SWF ScriptTimeoutSeconds;
 public:
 	ScriptLimitsTag(RECORDHEADER h, std::istream& in);
-	TAGTYPE getType() const{ return ABC_TAG; }
-	void execute(RootMovieClip* root) const;
+	TAGTYPE getType() const override { return ABC_TAG; }
+	void execute(RootMovieClip* root) const override;
 };
 
 class ProductInfoTag: public Tag
@@ -725,21 +829,32 @@ public:
 	DebugIDTag(RECORDHEADER h, std::istream& in);
 };
 
+class ExportAssetsTag: public Tag
+{
+public:
+	ExportAssetsTag(RECORDHEADER h, std::istream& in, RootMovieClip *root);
+};
+
+class NameCharacterTag: public Tag
+{
+public:
+	NameCharacterTag(RECORDHEADER h, std::istream& in, RootMovieClip *root);
+};
+
 class TagFactory
 {
 private:
 	std::istream& f;
 	bool firstTag;
-	bool topLevel;
 public:
-	TagFactory(std::istream& in, bool t):f(in),firstTag(true),topLevel(t){}
+	TagFactory(std::istream& in):f(in),firstTag(true){}
 	/**
 	 * The RootMovieClip that is the owner of the content.
 	 * It is needed to solve references to other tags during construction
 	 */
-	Tag* readTag(RootMovieClip* root);
+	Tag* readTag(RootMovieClip* root,DefineSpriteTag* sprite=nullptr);
 };
 
-};
+}
 
 #endif /* PARSING_TAGS_H */
