@@ -18,6 +18,7 @@
 **************************************************************************/
 
 #include "scripting/abctypes.h"
+#include "swf.h"
 
 using namespace std;
 using namespace lightspark;
@@ -113,7 +114,7 @@ istream& lightspark::operator>>(istream& in, string_info& v)
 {
 	u30 size;
 	in >> size;
-	v.val=tiny_string(in,size);
+	v.val = getSys()->getUniqueStringId(tiny_string(in,size));
 	return in;
 }
 
@@ -163,9 +164,9 @@ istream& lightspark::operator>>(istream& in, method_body_info& v)
 {
 	u30 code_length;
 	in >> v.method >> v.max_stack >> v.local_count >> v.init_scope_depth >> v.max_scope_depth >> code_length;
+	v.returnvaluepos=v.local_count+1;
 	v.code.resize(code_length);
 	in.read(&v.code[0],code_length);
-
 	u30 exception_count;
 	in >> exception_count;
 	v.exceptions.resize(exception_count);
@@ -206,9 +207,11 @@ istream& lightspark::operator>>(istream& in, multiname_info& v)
 		case 0x0f:
 		case 0x10:
 			in >> v.name;
+			v.runtimeargs = 1;
 			break;
 		case 0x11:
 		case 0x12:
+			v.runtimeargs = 2;
 			break;
 		case 0x09:
 		case 0x0e:
@@ -217,6 +220,7 @@ istream& lightspark::operator>>(istream& in, multiname_info& v)
 		case 0x1b:
 		case 0x1c:
 			in >> v.ns_set;
+			v.runtimeargs = 1;
 			break;
 		case 0x1d:
 		{
@@ -265,9 +269,14 @@ istream& lightspark::operator>>(istream& in, metadata_info& v)
 	in >> v.item_count;
 
 	v.items.resize(v.item_count);
+	// it seems that metadata is stored as key1,key2,key3...,value2,value2,value3...
 	for(unsigned int i=0;i<v.item_count;i++)
 	{
-		in >> v.items[i].key >> v.items[i].value;
+		in >> v.items[i].key;
+	}
+	for(unsigned int i=0;i<v.item_count;i++)
+	{
+		in >> v.items[i].value;
 	}
 	return in;
 }
@@ -309,7 +318,7 @@ istream& lightspark::operator>>(istream& in, traits_info& v)
 	return in;
 }
 
-istream& lightspark::operator >>(istream& in, exception_info& v)
+istream& lightspark::operator >>(istream& in, exception_info_abc& v)
 {
 	u30 from, to, target;
 	in >> from >> to >> target >> v.exc_type >> v.var_name;
@@ -340,6 +349,7 @@ istream& lightspark::operator>>(istream& in, instance_info& v)
 	v.traits.resize(v.trait_count);
 	for(unsigned int i=0;i<v.trait_count;i++)
 		in >> v.traits[i];
+	v.overriddenmethods=nullptr;
 	return in;
 }
 
