@@ -22,11 +22,13 @@
 
 #include "swftypes.h"
 #include "memory_support.h"
+#include <unordered_set>
 
 class memorystream;
 
 namespace lightspark
 {
+struct variable;
 
 class u8
 {
@@ -96,15 +98,28 @@ class string_info
 {
 friend std::istream& operator>>(std::istream& in, string_info& v);
 private:
-	tiny_string val;
+	uint32_t val;
 public:
-	operator const tiny_string&() const{return val;}
+	operator uint32_t() const{return val;}
 };
 
 struct namespace_info
 {
 	u8 kind;
 	u30 name;
+	bool cachefilled;
+	nsNameAndKind nscached;
+	namespace_info():cachefilled(false) {}
+	~namespace_info(){ cachefilled=false; }
+	const nsNameAndKind& getNS(ABCContext * c, uint32_t nsContextIndex)
+	{
+		if (!cachefilled)
+		{
+			cachefilled = true;
+			nscached = nsNameAndKind(c,nsContextIndex);
+		}
+		return nscached;
+	}
 };
 
 struct ns_set_info
@@ -121,9 +136,11 @@ struct multiname_info
 	u30 ns_set;
 	u30 type_definition;
 	std::vector<u30> param_types;
+	int runtimeargs;
 	multiname* cached;
-	multiname_info():cached(NULL){}
-	~multiname_info(){delete cached;}
+	multiname* dynamic;
+	multiname_info():runtimeargs(0),cached(NULL),dynamic(NULL){}
+	~multiname_info(){delete cached;if (dynamic) {delete dynamic;};}
 	bool isAttributeName() const;
 };
 
@@ -201,6 +218,7 @@ struct instance_info
 	u30 init;
 	u30 trait_count;
 	std::vector<traits_info> traits;
+	std::unordered_set<uint32_t>* overriddenmethods;
 };
 
 struct class_info
@@ -217,7 +235,7 @@ struct script_info
 	std::vector<traits_info> traits;
 };
 
-struct exception_info
+struct exception_info_abc
 {
 	uint32_t from;
 	uint32_t to;
@@ -244,30 +262,85 @@ struct method_info_simple
 	std::vector<option_detail> options;
 	std::vector<u30> param_names;
 };
+typedef void (*abc_function)(struct call_context*);
+
+struct preloadedcodedata
+{
+	abc_function func;
+	union
+	{
+		ASObject* cacheobj1;
+		asAtom* arg1_constant;
+		uint32_t local_pos1;
+		int32_t arg1_int;
+		uint32_t arg1_uint;
+	};
+	union
+	{
+		ASObject* cacheobj2;
+		multiname* cachedmultiname2;
+		variable* cachedvar2;
+		asAtom* arg2_constant;
+		struct
+		{
+			uint16_t pos;
+			uint16_t flags;
+		} local2;
+		uint32_t local_pos2;
+		int32_t arg2_int;
+		uint32_t arg2_uint;
+	};
+	union
+	{
+		ASObject* cacheobj3;
+		multiname* cachedmultiname3;
+		asAtom* arg3_constant;
+		struct
+		{
+			uint16_t pos;
+			uint16_t flags;
+		} local3;
+		int32_t arg3_int;
+		uint32_t arg3_uint;
+	};
+	preloadedcodedata():func(nullptr),cacheobj1(nullptr),cacheobj2(nullptr),cacheobj3(nullptr) {}
+};
+struct localconstantslot
+{
+	uint32_t local_pos;
+	uint32_t slot_number;
+};
 
 struct method_body_info
 {
-	method_body_info():hit_count(0),codeStatus(ORIGINAL){}
+	method_body_info():localresultcount(0),hit_count(0),codeStatus(ORIGINAL){}
 	u30 method;
 	u30 max_stack;
 	u30 local_count;
 	u30 init_scope_depth;
 	u30 max_scope_depth;
 	std::string code;
-	std::vector<exception_info> exceptions;
+	std::vector<exception_info_abc> exceptions;
 	u30 trait_count;
 	std::vector<traits_info> traits;
+	uint16_t localresultcount;
 	//The hit_count belongs here, since it is used to manipulate the code
 	uint16_t hit_count;
+	uint16_t returnvaluepos;
 	//The code status
-	enum CODE_STATUS { ORIGINAL = 0, USED, OPTIMIZED, JITTED };
+	enum CODE_STATUS { ORIGINAL = 0, USED, OPTIMIZED, JITTED, PRELOADED };
 	CODE_STATUS codeStatus;
+	// list of local/slot pairs that were optimized away
+	std::vector<localconstantslot> localconstantslots;
+	std::vector<preloadedcodedata> preloadedcode;
+	inline uint16_t getReturnValuePos() const { return returnvaluepos; }
 };
 
 std::istream& operator>>(std::istream& in, u8& v);
 std::istream& operator>>(std::istream& in, u16& v);
 std::istream& operator>>(std::istream& in, u30& v);
 std::istream& operator>>(std::istream& in, u32& v);
+std::istream& operator>>(std::istream& in, s24& v);
 std::istream& operator>>(std::istream& in, s32& v);
 std::istream& operator>>(std::istream& in, d64& v);
 std::istream& operator>>(std::istream& in, string_info& v);
@@ -275,7 +348,7 @@ std::istream& operator>>(std::istream& in, namespace_info& v);
 std::istream& operator>>(std::istream& in, ns_set_info& v);
 std::istream& operator>>(std::istream& in, multiname_info& v);
 std::istream& operator>>(std::istream& in, cpool_info& v);
-std::istream& operator>>(std::istream& in, exception_info& v);
+std::istream& operator>>(std::istream& in, exception_info_abc& v);
 std::istream& operator>>(std::istream& in, method_info_simple& v);
 std::istream& operator>>(std::istream& in, method_body_info& v);
 std::istream& operator>>(std::istream& in, instance_info& v);
