@@ -28,11 +28,16 @@ namespace lightspark
 class AccessibilityProperties : public ASObject
 {
 private:
-	ASPROPERTY_GETTER_SETTER(tiny_string,name);
+	ASPROPERTY_GETTER_SETTER(tiny_string, description);
+	ASPROPERTY_GETTER_SETTER(bool, forceSimple);
+	ASPROPERTY_GETTER_SETTER(tiny_string, name);
+	ASPROPERTY_GETTER_SETTER(bool, noAutoLabeling);
+	ASPROPERTY_GETTER_SETTER(tiny_string, shortcut);
+	ASPROPERTY_GETTER_SETTER(bool, silent);
 public:
-	AccessibilityProperties(Class_base* c):ASObject(c){}
+	AccessibilityProperties(Class_base* c);
 	static void sinit(Class_base*);
-	ASFUNCTION(_constructor);
+	ASFUNCTION_ATOM(_constructor);
 };
 
 class AccessibilityImplementation : public ASObject
@@ -40,7 +45,17 @@ class AccessibilityImplementation : public ASObject
 public:
 	AccessibilityImplementation(Class_base* c):ASObject(c){}
 	static void sinit(Class_base*);
-	ASFUNCTION(_constructor);
+	ASFUNCTION_ATOM(_constructor);
+};
+
+class Accessibility : public ASObject
+{
+private:
+	_NR<AccessibilityProperties> properties;
+public:
+	Accessibility(Class_base* c):ASObject(c),properties(NULL){}
+	static void sinit(Class_base*);
+	ASFUNCTION_ATOM(updateProperties);
 };
 
 }
