@@ -32,11 +32,20 @@ public:
 	NativeApplication(Class_base* c):EventDispatcher(c){}
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_getNativeApplication);
-	ASFUNCTION(addEventListener);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(_getNativeApplication);
+	ASFUNCTION_ATOM(addEventListener);
 };
 
+class NativeDragManager: public ASObject
+{
+public:
+	NativeDragManager(Class_base* c):ASObject(c),isSupported(false){}
+	static void sinit(Class_base* c);
+	ASPROPERTY_GETTER(bool,isSupported);
+};
+
+
 };
 
 #endif /* SCRIPTING_FLASH_DESKTOP_FLASHDESKTOP_H */
