@@ -24,16 +24,20 @@
 #include "backends/geometry.h"
 #include "backends/graphics.h"
 #include "scripting/flash/display/DisplayObject.h"
+#include "scripting/flash/display/Graphics.h"
 
 namespace lightspark
 {
 
 class DisplayObject;
 class InteractiveObject;
+class DefineMorphShapeTag;
 
 class TokenContainer
 {
 	friend class Graphics;
+	friend class MorphShape;
+	friend class TextField;
 public:
 	DisplayObject* owner;
 	/* multiply shapes' coordinates by this
@@ -45,22 +49,25 @@ public:
 	 * the tokens are cleared and scaling is set
 	 * to 1.0f.
 	 */
-	std::vector<GeomToken> tokens;
+	tokensVector tokens;
 	static void FromShaperecordListToShapeVector(const std::vector<SHAPERECORD>& shapeRecords,
 					 tokensVector& tokens, const std::list<FILLSTYLE>& fillStyles,
-					 const MATRIX& matrix = MATRIX());
-	void getTextureSize(int *width, int *height) const;
+					 const MATRIX& matrix = MATRIX(), const std::list<LINESTYLE2>& lineStyles = std::list<LINESTYLE2>(), const RECT &shapebounds= RECT());
+	static void FromDefineMorphShapeTagToShapeVector(SystemState *sys, DefineMorphShapeTag *tag,
+					 tokensVector& tokens, uint16_t ratio);
+	static void getTextureSize(std::vector<_NR<GeomToken>, reporter_allocator<_NR<GeomToken>>> &tokens, int *width, int *height);
+	uint16_t getCurrentLineWidth() const;
 	float scaling;
 protected:
-	TokenContainer(DisplayObject* _o);
-	TokenContainer(DisplayObject* _o, const tokensVector& _tokens, float _scaling);
-	IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix);
-	void requestInvalidation(InvalidateQueue* q);
+	TokenContainer(DisplayObject* _o, MemoryAccount* _m);
+	TokenContainer(DisplayObject* _o, MemoryAccount* _m, const tokensVector& _tokens, float _scaling);
+	IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix, bool smoothing);
+	void requestInvalidation(InvalidateQueue* q, bool forceTextureRefresh=false);
 	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const;
 	_NR<DisplayObject> hitTestImpl(_NR<DisplayObject> last, number_t x, number_t y, DisplayObject::HIT_TYPE type) const;
-	void renderImpl(RenderContext& ctxt) const;
+	bool renderImpl(RenderContext& ctxt) const;
 	bool tokensEmpty() const { return tokens.empty(); }
 };
 
-};
+}
 #endif /* SCRIPTING_FLASH_DISPLAY_TOKENCONTAINER_H */
