@@ -30,15 +30,21 @@ namespace lightspark
 {
 
 class URLStream;
+class ByteArray;
 
-class URLStreamThread : public DownloaderThreadBase
+class URLStreamThread : public DownloaderThreadBase, public ILoadable
 {
 private:
 	_R<URLStream> loader;
 	_R<ByteArray> data;
+	std::streambuf *streambuffer;
+	uint64_t timestamp_last_progress;
+	uint32_t bytes_total;
 	void execute();
 public:
 	URLStreamThread(_R<URLRequest> request, _R<URLStream> ldr, _R<ByteArray> bytes);
+	void setBytesTotal(uint32_t b);
+	void setBytesLoaded(uint32_t b);
 };
 
 class URLStream: public EventDispatcher, public IDataInput, public IDownloaderThreadListener
@@ -47,32 +53,33 @@ private:
 	URLInfo url;
 	_NR<ByteArray> data;
 	URLStreamThread *job;
-	Spinlock spinlock;
+	Mutex spinlock;
 	void finalize();
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_getEndian);
-	ASFUNCTION(_setEndian);
-	ASFUNCTION(_getObjectEncoding);
-	ASFUNCTION(_setObjectEncoding);
-	ASFUNCTION(load);
-	ASFUNCTION(close);
-	ASFUNCTION(bytesAvailable);
-	ASFUNCTION(readBoolean);
-	ASFUNCTION(readByte);
-	ASFUNCTION(readBytes);
-	ASFUNCTION(readDouble);
-	ASFUNCTION(readFloat);
-	ASFUNCTION(readInt);
-	ASFUNCTION(readMultiByte);
-	ASFUNCTION(readObject);
-	ASFUNCTION(readShort);
-	ASFUNCTION(readUnsignedByte);
-	ASFUNCTION(readUnsignedInt);
-	ASFUNCTION(readUnsignedShort);
-	ASFUNCTION(readUTF);
-	ASFUNCTION(readUTFBytes);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(_getEndian);
+	ASFUNCTION_ATOM(_setEndian);
+	ASFUNCTION_ATOM(_getObjectEncoding);
+	ASFUNCTION_ATOM(_setObjectEncoding);
+	ASFUNCTION_ATOM(load);
+	ASFUNCTION_ATOM(close);
+	ASFUNCTION_ATOM(bytesAvailable);
+	ASFUNCTION_ATOM(readBoolean);
+	ASFUNCTION_ATOM(readByte);
+	ASFUNCTION_ATOM(readBytes);
+	ASFUNCTION_ATOM(readDouble);
+	ASFUNCTION_ATOM(readFloat);
+	ASFUNCTION_ATOM(readInt);
+	ASFUNCTION_ATOM(readMultiByte);
+	ASFUNCTION_ATOM(readObject);
+	ASFUNCTION_ATOM(readShort);
+	ASFUNCTION_ATOM(readUnsignedByte);
+	ASFUNCTION_ATOM(readUnsignedInt);
+	ASFUNCTION_ATOM(readUnsignedShort);
+	ASFUNCTION_ATOM(readUTF);
+	ASFUNCTION_ATOM(readUTFBytes);
+	ASPROPERTY_GETTER(bool,connected);
 public:
-	URLStream(Class_base* c):EventDispatcher(c),data(_MNR(Class<ByteArray>::getInstanceS())),job(NULL) {}
+	URLStream(Class_base* c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	void threadFinished(IThreadJob *job);
