@@ -27,7 +27,9 @@
 #include "backends/netutils.h"
 #include "timer.h"
 #include "backends/decoder.h"
-#include "backends/interfaces/audio/IAudioPlugin.h"
+#include "backends/audio.h"
+#include "NetStreamInfo.h"
+#include "scripting/flash/utils/ByteArray.h"
 
 namespace lightspark
 {
@@ -39,23 +41,26 @@ private:
 	METHOD method;
 	tiny_string url;
 	_NR<ASObject> data;
+	tiny_string digest;
 	tiny_string validatedContentType() const;
 	tiny_string getContentTypeHeader() const;
 	void validateHeaderName(const tiny_string& headerName) const;
 	ASPROPERTY_GETTER_SETTER(tiny_string,contentType);
 	ASPROPERTY_GETTER_SETTER(_R<Array>,requestHeaders);
 public:
-	URLRequest(Class_base* c);
+	URLRequest(Class_base* c, const tiny_string u="");
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_getURL);
-	ASFUNCTION(_setURL);
-	ASFUNCTION(_getMethod);
-	ASFUNCTION(_setMethod);
-	ASFUNCTION(_setData);
-	ASFUNCTION(_getData);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(_getURL);
+	ASFUNCTION_ATOM(_setURL);
+	ASFUNCTION_ATOM(_getMethod);
+	ASFUNCTION_ATOM(_setMethod);
+	ASFUNCTION_ATOM(_setData);
+	ASFUNCTION_ATOM(_getData);
+	ASFUNCTION_ATOM(_getDigest);
+	ASFUNCTION_ATOM(_setDigest);
 	URLInfo getRequestURL() const;
 	std::list<tiny_string> getHeaders() const;
 	void getPostData(std::vector<uint8_t>& data) const;
@@ -78,9 +83,9 @@ public:
 	URLVariables(Class_base* c,const tiny_string& s);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(decode);
-	ASFUNCTION(_toString);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(decode);
+	ASFUNCTION_ATOM(_toString);
 	tiny_string toString();
 };
 
@@ -91,13 +96,58 @@ public:
 	static void sinit(Class_base*);
 };
 
-class SharedObject: public EventDispatcher
+
+class SharedObjectFlushStatus: public ASObject
 {
 public:
-	SharedObject(Class_base* c);
+	SharedObjectFlushStatus(Class_base* c):ASObject(c){}
 	static void sinit(Class_base*);
-	ASFUNCTION(getLocal);
+};
+
+class SharedObject: public EventDispatcher
+{
+private:
+	tiny_string name;
+	bool hasData;
+public:
+	SharedObject(Class_base* c);
+	bool doFlush();
+	static void sinit(Class_base*);
+	ASFUNCTION_ATOM(getLocal);
+	ASFUNCTION_ATOM(getRemote);
+	ASFUNCTION_ATOM(flush);
+	ASFUNCTION_ATOM(clear);
+	ASFUNCTION_ATOM(close);
+	ASFUNCTION_ATOM(connect);
+	ASFUNCTION_ATOM(setProperty);
+	ASPROPERTY_GETTER_SETTER(_R<ASObject>,client);
 	ASPROPERTY_GETTER(_NR<ASObject>,data);
+	ASFUNCTION_ATOM(_getDefaultObjectEncoding);
+	ASFUNCTION_ATOM(_setDefaultObjectEncoding);
+	ASPROPERTY_SETTER(number_t,fps);
+	ASPROPERTY_GETTER_SETTER(uint32_t,objectEncoding);
+	ASFUNCTION_ATOM(_getPreventBackup);
+	ASFUNCTION_ATOM(_setPreventBackup);
+	ASFUNCTION_ATOM(_getSize);
+};
+
+class IDynamicPropertyWriter
+{
+public:
+	static void linkTraits(Class_base* c);
+};
+class IDynamicPropertyOutput
+{
+public:
+	static void linkTraits(Class_base* c);
+};
+// this is an internal class not described in the specs, but avmplus defines it for dynamic property writing
+class DynamicPropertyOutput:public ASObject
+{
+public:
+	DynamicPropertyOutput(Class_base* c):ASObject(c){}
+	static void sinit(Class_base*);
+	ASFUNCTION_ATOM(writeDynamicProperty);
 };
 
 class ObjectEncoding: public ASObject
@@ -106,6 +156,7 @@ public:
 	ObjectEncoding(Class_base* c):ASObject(c){}
 	enum ENCODING { AMF0=0, AMF3=3, DEFAULT=3 };
 	static void sinit(Class_base*);
+	ASPROPERTY_GETTER_SETTER(_NR<ASObject>, dynamicPropertyWriter);
 };
 
 class URLLoader;
@@ -124,8 +175,9 @@ class URLLoader: public EventDispatcher, public IDownloaderThreadListener, publi
 private:
 	tiny_string dataFormat;
 	_NR<ASObject> data;
-	Spinlock spinlock;
+	Mutex spinlock;
 	URLLoaderThread *job;
+	uint64_t timestamp_last_progress;
 public:
 	URLLoader(Class_base* c);
 	void finalize();
@@ -133,17 +185,21 @@ public:
 	static void buildTraits(ASObject* o);
 	void threadFinished(IThreadJob *job);
 	void setData(_NR<ASObject> data);
+	ASObject* getData() const
+	{
+		return data.getPtr();
+	}
 	tiny_string getDataFormat();
 	void setDataFormat(const tiny_string& newFormat);
 	void setBytesTotal(uint32_t b);
 	void setBytesLoaded(uint32_t b);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(load);
-	ASFUNCTION(close);
-	ASFUNCTION(_getDataFormat);
-	ASFUNCTION(_getData);
-	ASFUNCTION(_setData);
-	ASFUNCTION(_setDataFormat);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(load);
+	ASFUNCTION_ATOM(close);
+	ASFUNCTION_ATOM(_getDataFormat);
+	ASFUNCTION_ATOM(_getData);
+	ASFUNCTION_ATOM(_setData);
+	ASFUNCTION_ATOM(_setDataFormat);
 	ASPROPERTY_GETTER_SETTER(uint32_t, bytesLoaded);
 	ASPROPERTY_GETTER_SETTER(uint32_t, bytesTotal);
 };
@@ -151,20 +207,21 @@ public:
 class Responder: public ASObject
 {
 private:
-	_NR<IFunction> result;
-	_NR<IFunction> status;
+	asAtom result;
+	asAtom status;
 public:
-	Responder(Class_base* c):ASObject(c){}
+	Responder(Class_base* c):ASObject(c),result(asAtomHandler::invalidAtom),status(asAtomHandler::invalidAtom){}
 	static void sinit(Class_base*);
 	void finalize();
-	ASFUNCTION(_constructor);
-	ASFUNCTION(onResult);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(onResult);
 };
 
 class NetConnection: public EventDispatcher, public IThreadJob
 {
 friend class NetStream;
 private:
+	enum PROXY_TYPE { PT_NONE, PT_HTTP, PT_CONNECT_ONLY, PT_CONNECT, PT_BEST };
 	//Indicates whether the application is connected to a server through a persistent RMTP connection/HTTP server with Flash Remoting
 	bool _connected;
 	tiny_string protocol;
@@ -172,12 +229,13 @@ private:
 	//Data for remoting support (NetConnection::call)
 	// The message data to be sent asynchronously
 	std::vector<uint8_t> messageData;
-	Spinlock downloaderLock;
+	Mutex downloaderLock;
 	Downloader* downloader;
 	_NR<Responder> responder;
 	uint32_t messageCount;
 	//The connection is to a flash media server
 	ObjectEncoding::ENCODING objectEncoding;
+	PROXY_TYPE proxyType;
 	//IThreadJob interface
 	void execute();
 	void threadAbort();
@@ -187,19 +245,34 @@ public:
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(connect);
-	ASFUNCTION(call);
-	ASFUNCTION(_getConnected);
-	ASFUNCTION(_getDefaultObjectEncoding);
-	ASFUNCTION(_setDefaultObjectEncoding);
-	ASFUNCTION(_getObjectEncoding);
-	ASFUNCTION(_setObjectEncoding);
-	ASFUNCTION(_getProtocol);
-	ASFUNCTION(_getURI);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(connect);
+	ASFUNCTION_ATOM(call);
+	ASFUNCTION_ATOM(_getConnected);
+	ASFUNCTION_ATOM(_getConnectedProxyType);
+	ASFUNCTION_ATOM(_getDefaultObjectEncoding);
+	ASFUNCTION_ATOM(_setDefaultObjectEncoding);
+	ASFUNCTION_ATOM(_getObjectEncoding);
+	ASFUNCTION_ATOM(_setObjectEncoding);
+	ASFUNCTION_ATOM(_getProtocol);
+	ASFUNCTION_ATOM(_getProxyType);
+	ASFUNCTION_ATOM(_setProxyType);
+	ASFUNCTION_ATOM(_getURI);
+	ASFUNCTION_ATOM(close);
 	ASPROPERTY_GETTER_SETTER(NullableRef<ASObject>,client);
+	ASPROPERTY_GETTER_SETTER(uint32_t,maxPeerConnections);
+	ASPROPERTY_GETTER(tiny_string,nearID);
+	void afterExecution(_R<Event> e);
 };
 
+class NetStreamAppendBytesAction: public ASObject
+{
+public:
+	NetStreamAppendBytesAction(Class_base* c):ASObject(c){}
+	static void sinit(Class_base*);
+};
+
+
 class SoundTransform;
 class NetStream: public EventDispatcher, public IThreadJob, public ITickJob
 {
@@ -220,7 +293,11 @@ private:
 	VideoDecoder* videoDecoder;
 	AudioDecoder* audioDecoder;
 	AudioStream *audioStream;
+	// only used when in DataGenerationMode
+	StreamCache* datagenerationfile;
+	bool datagenerationthreadstarted;
 	Mutex mutex;
+	Mutex countermutex;
 	//IThreadJob interface for long jobs
 	void execute();
 	void threadAbort();
@@ -240,36 +317,56 @@ private:
 	bool checkPolicyFile;
 	bool rawAccessAllowed;
 
-	ASObject *createMetaDataObject(StreamDecoder* streamDecoder);
-	ASObject *createPlayStatusObject(const tiny_string& code);
-	void sendClientNotification(const tiny_string& name, ASObject *args);
+	uint32_t framesdecoded;
+	uint32_t prevstreamtime;
+	number_t playbackBytesPerSecond;
+	number_t maxBytesPerSecond;
+
+	struct bytespertime {
+		uint64_t timestamp;
+		uint32_t bytesread;
+	};
+	std::deque<bytespertime> currentBytesPerSecond;
+	enum DATAGENERATION_EXPECT_TYPE { DATAGENERATION_HEADER=0,DATAGENERATION_PREVTAG,DATAGENERATION_FLVTAG };
+	DATAGENERATION_EXPECT_TYPE datagenerationexpecttype;
+	_NR<ByteArray> datagenerationbuffer;
+	StreamDecoder* streamDecoder;
 public:
 	NetStream(Class_base* c);
 	~NetStream();
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(play);
-	ASFUNCTION(resume);
-	ASFUNCTION(pause);
-	ASFUNCTION(togglePause);
-	ASFUNCTION(close);
-	ASFUNCTION(seek);
-	ASFUNCTION(_getBytesLoaded);
-	ASFUNCTION(_getBytesTotal);
-	ASFUNCTION(_getTime);
-	ASFUNCTION(_getCurrentFPS);
-	ASFUNCTION(_getClient);
-	ASFUNCTION(_setClient);
-	ASFUNCTION(_getCheckPolicyFile);
-	ASFUNCTION(_setCheckPolicyFile);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(play);
+	ASFUNCTION_ATOM(play2);
+	ASFUNCTION_ATOM(resume);
+	ASFUNCTION_ATOM(pause);
+	ASFUNCTION_ATOM(togglePause);
+	ASFUNCTION_ATOM(close);
+	ASFUNCTION_ATOM(seek);
+	ASFUNCTION_ATOM(_getBytesLoaded);
+	ASFUNCTION_ATOM(_getBytesTotal);
+	ASFUNCTION_ATOM(_getTime);
+	ASFUNCTION_ATOM(_getCurrentFPS);
+	ASFUNCTION_ATOM(_getClient);
+	ASFUNCTION_ATOM(_setClient);
+	ASFUNCTION_ATOM(_getCheckPolicyFile);
+	ASFUNCTION_ATOM(_setCheckPolicyFile);
+	ASFUNCTION_ATOM(attach);
+	ASFUNCTION_ATOM(appendBytes);
+	ASFUNCTION_ATOM(appendBytesAction);
+	ASFUNCTION_ATOM(_getInfo);
+	ASFUNCTION_ATOM(publish);
 	ASPROPERTY_GETTER(number_t, backBufferLength);
 	ASPROPERTY_GETTER_SETTER(number_t, backBufferTime);
 	ASPROPERTY_GETTER(number_t, bufferLength);
 	ASPROPERTY_GETTER_SETTER(number_t, bufferTime);
 	ASPROPERTY_GETTER_SETTER(number_t, bufferTimeMax);
 	ASPROPERTY_GETTER_SETTER(number_t, maxPauseBufferTime);
+	ASPROPERTY_GETTER_SETTER(bool, useHardwareDecoder);
+
+	void sendClientNotification(const tiny_string& name, std::list<asAtom>& arglist);
 
 	//Interface for video
 	/**
@@ -334,11 +431,61 @@ public:
 	void unlock();
 };
 
-ASObject* navigateToURL(ASObject* obj,ASObject* const* args, const unsigned int argslen);
-ASObject* sendToURL(ASObject* obj,ASObject* const* args, const unsigned int argslen);
-ASObject* registerClassAlias(ASObject* obj,ASObject* const* args, const unsigned int argslen);
-ASObject* getClassByAlias(ASObject* obj,ASObject* const* args, const unsigned int argslen);
-
+class LocalConnection: public EventDispatcher
+{
+public:
+	LocalConnection(Class_base* c);
+	static void sinit(Class_base*);
+	ASFUNCTION_ATOM(_constructor);
+	ASPROPERTY_GETTER(bool,isSupported);
+	ASFUNCTION_ATOM(allowDomain);
+	ASFUNCTION_ATOM(allowInsecureDomain);
+	ASFUNCTION_ATOM(send);
+	ASFUNCTION_ATOM(connect);
+	ASFUNCTION_ATOM(close);
+	ASFUNCTION_ATOM(domain);
+	ASPROPERTY_GETTER_SETTER(_NR<ASObject>,client);
 };
 
+class NetGroup: public EventDispatcher
+{
+public:
+	NetGroup(Class_base* c);
+	static void sinit(Class_base*);
+	ASFUNCTION_ATOM(_constructor);
+};
+
+class FileReference: public EventDispatcher
+{
+public:
+	FileReference(Class_base* c);
+	static void sinit(Class_base*);
+	ASFUNCTION_ATOM(_constructor);
+};
+class FileFilter: public ASObject
+{
+public:
+	FileFilter(Class_base* c);
+	static void sinit(Class_base*);
+	ASFUNCTION_ATOM(_constructor);
+	ASPROPERTY_GETTER_SETTER(tiny_string,description);
+	ASPROPERTY_GETTER_SETTER(tiny_string,extension);
+	ASPROPERTY_GETTER_SETTER(tiny_string,macType);
+};
+
+class DRMManager: public EventDispatcher
+{
+public:
+	DRMManager(Class_base* c);
+	static void sinit(Class_base*);
+	ASPROPERTY_GETTER(bool,isSupported);
+};
+
+void navigateToURL(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
+void sendToURL(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
+void registerClassAlias(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
+void getClassByAlias(asAtom& ret,SystemState* sys,asAtom& obj,asAtom* args, const unsigned int argslen);
+
+}
+
 #endif /* SCRIPTING_FLASH_NET_FLASHNET_H */
