@@ -26,16 +26,15 @@
 #include "thread_pool.h"
 #include "backends/netutils.h"
 #include "timer.h"
-#include "backends/interfaces/audio/IAudioPlugin.h"
 
 namespace lightspark
 {
-class Accelerometer: public ASObject {
+class Accelerometer: public EventDispatcher {
 	public:
 		Accelerometer(Class_base* c);
 		static void sinit(Class_base* c);
 		static void buildTraits(ASObject* o);
-		ASFUNCTION(_isSupported);
+		ASFUNCTION_ATOM(_isSupported);
 };
 }
 #endif /* SCRIPTING_FLASH_SENSORS_FLASHSENSORS_H */
