@@ -36,35 +36,37 @@ template<class T>
 void throwError(int errorID, const tiny_string& arg1="", const tiny_string& arg2="", const tiny_string& arg3="")
 {
 	tiny_string message = createErrorMessage(errorID, arg1, arg2, arg3);
-	throw Class<T>::getInstanceS(message, errorID);
+	throw Class<T>::getInstanceS(getSys(),message, errorID);
 }
 
 class ASError: public ASObject
 {
 private:
+	tiny_string stacktrace;
 	ASPROPERTY_GETTER(int32_t, errorID);
 	ASPROPERTY_GETTER_SETTER(tiny_string, name);
 protected:
 	ASPROPERTY_GETTER_SETTER(tiny_string, message);
 	void setErrorID(int32_t id) { errorID=id; }
-	static void errorGenerator(ASError *obj, ASObject* const* args, const unsigned int argslen);
+	static void errorGenerator(ASError *obj, asAtom *args, const unsigned int argslen);
 public:
 	ASError(Class_base* c, const tiny_string& error_message = "", int id = 0, const tiny_string& error_name="Error");
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
-	ASFUNCTION(getStackTrace);
-	ASFUNCTION(_toString);
-	tiny_string toString(bool debugMsg=false);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
+	ASFUNCTION_ATOM(_getStackTrace);
+	ASFUNCTION_ATOM(_toString);
+	tiny_string toString();
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
+	tiny_string getStackTraceString();
 };
 
 class SecurityError: public ASError
 {
 public:
 	SecurityError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "SecurityError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -73,8 +75,8 @@ class ArgumentError: public ASError
 {
 public:
 	ArgumentError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "ArgumentError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -83,8 +85,8 @@ class DefinitionError: public ASError
 {
 public:
 	DefinitionError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "DefinitionError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -93,8 +95,8 @@ class EvalError: public ASError
 {
 public:
 	EvalError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "EvalError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -103,8 +105,8 @@ class RangeError: public ASError
 {
 public:
 	RangeError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "RangeError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -113,8 +115,8 @@ class ReferenceError: public ASError
 {
 public:
 	ReferenceError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "ReferenceError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -123,8 +125,8 @@ class SyntaxError: public ASError
 {
 public:
 	SyntaxError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "SyntaxError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -133,8 +135,8 @@ class TypeError: public ASError
 {
 public:
 	TypeError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "TypeError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -143,8 +145,8 @@ class URIError: public ASError
 {
 public:
 	URIError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "URIError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -153,8 +155,8 @@ class VerifyError: public ASError
 {
 public:
 	VerifyError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "VerifyError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
@@ -164,8 +166,8 @@ class UninitializedError: public ASError
 {
 public:
 	UninitializedError(Class_base* c, const tiny_string& error_message = "", int id = 0) : ASError(c, error_message, id, "UninitializedError"){}
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 };
