@@ -31,24 +31,34 @@ friend class Number;
 friend class Array;
 friend class ABCVm;
 friend class ABCContext;
-friend ASObject* abstract_i(int32_t i);
+friend ASObject* abstract_i(SystemState* sys, int32_t i);
 public:
-	Integer(Class_base* c,int32_t v=0):ASObject(c),val(v){type=T_INTEGER;}
+	Integer(Class_base* c,int32_t v=0):ASObject(c,T_INTEGER),val(v){}
 	int32_t val;
-	static void buildTraits(ASObject* o){};
+	static void buildTraits(ASObject* o){}
 	static void sinit(Class_base* c);
-	ASFUNCTION(_toString);
+	inline number_t toNumber() { return val; }
+	inline bool destruct() { val=0; return destructIntern(); }
+	ASFUNCTION_ATOM(_toString);
 	tiny_string toString();
 	static tiny_string toString(int32_t val);
 	int32_t toInt()
 	{
 		return val;
 	}
+	int64_t toInt64()
+	{
+		return val;
+	}
 	TRISTATE isLess(ASObject* r);
+	TRISTATE isLessAtom(asAtom& r);
 	bool isEqual(ASObject* o);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
-	ASFUNCTION(_valueOf);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
+	ASFUNCTION_ATOM(_valueOf);
+	ASFUNCTION_ATOM(_toExponential);
+	ASFUNCTION_ATOM(_toFixed);
+	ASFUNCTION_ATOM(_toPrecision);
 	std::string toDebugString() { return toString()+"i"; }
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
@@ -57,12 +67,13 @@ public:
 	/*
 	 * This method skips trailing spaces and zeroes
 	 */
-	static bool fromStringFlashCompatible(const char* str, int64_t& ret,int radix);
+	static bool fromStringFlashCompatible(const char* str, number_t &ret, int radix, bool strict=false);
 	/* Converts a string into AS (32 bit) integer value using ECMA
 	 * script conversion algorithm (i.e. accepts hex, skips
 	 * whitespace, zeroes). Returns 0 if conversion fails.
 	 */
-	static int32_t stringToASInteger(const char* cur, int radix);
+	static int32_t stringToASInteger(const char* cur, int radix, bool strict=false);
+	
 };
 
 }
