@@ -29,37 +29,73 @@ namespace lightspark
 
 class Number : public ASObject
 {
-friend ASObject* abstract_d(number_t i);
+friend ASObject* abstract_d(SystemState* sys,number_t i);
+friend ASObject* abstract_d_constant(SystemState* sys,number_t i);
+friend ASObject* abstract_di(SystemState* sys,int32_t i);
 friend class ABCContext;
 friend class ABCVm;
 private:
 	static void purgeTrailingZeroes(char* buf);
+	static tiny_string purgeExponentLeadingZeros(const tiny_string& exponentialForm);
+	static int32_t countSignificantDigits(double v);
+	enum DTOSTRMODE { DTOSTR_NORMAL, DTOSTR_FIXED, DTOSTR_PRECISION, DTOSTR_EXPONENTIAL };
+	static tiny_string toString(number_t value, DTOSTRMODE mode, int32_t precision);
 public:
-	Number(Class_base* c, double v=0.):ASObject(c),val(v){type=T_NUMBER;}
+	Number(Class_base* c, double v=Number::NaN):ASObject(c,T_NUMBER),dval(v),isfloat(true){}
 	static const number_t NaN;
-	double val;
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_toString);
-	ASFUNCTION(toFixed);
-	ASFUNCTION(_valueOf);
+	union {
+		number_t dval;
+		int64_t ival;
+	};
+	bool isfloat:1;
+	inline void setNumber(number_t v)
+	{
+		isfloat = true;
+		dval=v;
+	}
+	inline number_t toNumber() override { return isfloat ? dval : ival; }
+	inline bool destruct() override { dval=Number::NaN; isfloat = true; return destructIntern(); }
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(_toString);
+	ASFUNCTION_ATOM(_toLocaleString);
+	ASFUNCTION_ATOM(toExponential);
+	ASFUNCTION_ATOM(toPrecision);
+	ASFUNCTION_ATOM(toFixed);
+	ASFUNCTION_ATOM(_valueOf);
 	tiny_string toString();
-	static tiny_string toString(number_t val);
+	static tiny_string toString(number_t val) { return toString(val,DTOSTR_NORMAL,15);}
 	static tiny_string toStringRadix(number_t val, int radix);
+	static tiny_string toExponentialString(double v, int32_t fractionDigits);
+	static tiny_string toFixedString(double v, int32_t fractionDigits);
+	static tiny_string toPrecisionString(double v, int32_t precision);
 	static bool isInteger(number_t val)
 	{
-		return floor(val) == val;
+		return trunc(val) == val;
 	}
-	unsigned int toUInt()
+	unsigned int toUInt() override
 	{
-		return (unsigned int)(val);
+		return isfloat ? (unsigned int)(dval) : ival;
 	}
+	int64_t toInt64() override
+	{
+		if (!isfloat) return ival;
+		if(std::isnan(dval) || std::isinf(dval))
+			return INT64_MAX;
+		return (int64_t)dval;
+	}
+
 	/* ECMA-262 9.5 ToInt32 */
-	int32_t toInt()
+	int32_t toInt() override
+	{
+		if (!isfloat) return ival;
+		return toInt(dval);
+	}
+	static int32_t toInt(number_t val)
 	{
 		double posInt;
 
 		/* step 2 */
-		if(std::isnan(val) || std::isinf(val) || val == 0)
+		if(std::isnan(val) || std::isinf(val) || val == 0.0)
 			return 0;
 		/* step 3 */
 		posInt = floor(fabs(val));
@@ -74,21 +110,21 @@ public:
 			else
 				return 0x80000000 + (int32_t)(posInt - 2147483648.0);
 		}
-		return (int32_t)copysign(posInt, val);
+		return (int32_t)(val < 0.0 ? -posInt : posInt);
 	}
-	TRISTATE isLess(ASObject* o);
-	bool isEqual(ASObject* o);
-	static void buildTraits(ASObject* o){};
+	TRISTATE isLess(ASObject* o) override;
+	bool isEqual(ASObject* o) override;
+	static void buildTraits(ASObject* o){}
 	static void sinit(Class_base* c);
-	ASFUNCTION(generator);
-	std::string toDebugString() { return toString()+"d"; }
+	ASFUNCTION_ATOM(generator);
+	std::string toDebugString() override;
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap);
+				std::map<const Class_base*, uint32_t>& traitsMap) override;
 };
 
 
-};
+}
 
 #endif /* SCRIPTING_TOPLEVEL_NUMBER_H */
