@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -35,11 +35,11 @@ public:
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	ASObject *match(const tiny_string& str);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
-	ASFUNCTION(exec);
-	ASFUNCTION(test);
-	ASFUNCTION(_toString);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(generator);
+	ASFUNCTION_ATOM(exec);
+	ASFUNCTION_ATOM(test);
+	ASFUNCTION_ATOM(_toString);
 	ASPROPERTY_GETTER(bool, dotall);
 	ASPROPERTY_GETTER(bool, global);
 	ASPROPERTY_GETTER(bool, ignoreCase);
