@@ -29,35 +29,69 @@ namespace lightspark
 class RefCountable {
 private:
 	ATOMIC_INT32(ref_count);
-
+	int32_t activation_refcount;
+	bool isConstant:1;
+	bool inDestruction:1;
+	bool cached:1;
 protected:
-	RefCountable() : ref_count(1) {}
+	RefCountable() : ref_count(1),activation_refcount(1),isConstant(false),inDestruction(false),cached(false) {}
 
 public:
 	virtual ~RefCountable() {}
 
-#ifndef NDEBUG
 	int getRefCount() const { return ref_count; }
-#endif
-	void incRef()
+	inline bool isLastRef() const { return !isConstant && ref_count == activation_refcount; }
+	inline void setConstant()
 	{
-		ATOMIC_INCREMENT(ref_count);
-		assert(ref_count>0);
+		isConstant=true;
 	}
-	void decRef()
+	inline bool getConstant() const { return isConstant; }
+	inline bool getInDestruction() const { return inDestruction; }
+	inline bool getCached() const { return cached; }
+	inline void setCached() { cached=true; }
+	inline void resetCached() { cached=false; }
+	inline void incActivationCount() { activation_refcount++; }
+	inline void decActivationCount() { activation_refcount--; }
+	inline void setActivationCount(int32_t c) { activation_refcount=c; }
+	inline int32_t getActivationCount() const { return activation_refcount; }
+	inline void incRef()
 	{
-		assert(ref_count>0);
-		uint32_t t=ATOMIC_DECREMENT(ref_count);
-		if(t==0)
+		if (!isConstant)
+			++ref_count;
+	}
+	bool handleDestruction()
+	{
+		if (inDestruction)
+			return true;
+		inDestruction = true;
+		activation_refcount=1;
+		ref_count=1;
+		if (destruct())
 		{
 			//Let's make refcount very invalid
 			ref_count=-1024;
+			inDestruction = false;
 			delete this;
 		}
+		else
+			inDestruction = false;
+		return true;
 	}
-	void fake_decRef()
+	inline bool decRef()
 	{
-		ATOMIC_DECREMENT(ref_count);
+		if (!isConstant && !cached)
+		{
+			assert(ref_count>0);
+			if (ref_count == activation_refcount)
+				return handleDestruction();
+			else
+				--ref_count;
+		}
+		return cached;
+	}
+	virtual bool destruct()
+	{
+		return true;
 	}
 };
 
@@ -103,7 +137,7 @@ public:
 
 		return *this;
 	}
-	template<class D> Ref<T>& operator=(const Ref<D>& r)
+	template<class D> inline Ref<T>& operator=(const Ref<D>& r)
 	{
 		//incRef before decRef to make sure this works even if the pointer is the same
 		r.m->incRef();
@@ -115,25 +149,25 @@ public:
 
 		return *this;
 	}
-	template<class D> bool operator==(const Ref<D>& r) const
+	template<class D> inline bool operator==(const Ref<D>& r) const
 	{
 		return m==r.getPtr();
 	}
-	template<class D> bool operator!=(const Ref<D>& r) const
+	template<class D> inline bool operator!=(const Ref<D>& r) const
 	{
 		return m!=r.getPtr();
 	}
-	template<class D> bool operator==(const NullableRef<D>&r) const;
-	bool operator==(T* r) const
+	template<class D> inline bool operator==(const NullableRef<D>&r) const;
+	inline bool operator==(T* r) const
 	{
 		return m==r;
 	}
 	//Order operator for Dictionary map
-	bool operator<(const Ref<T>& r) const
+	inline bool operator<(const Ref<T>& r) const
 	{
 		return m<r.m;
 	}
-	template<class D> Ref<D> cast() const
+	template<class D> inline Ref<D> cast() const
 	{
 		D* p = static_cast<D*>(m);
 		p->incRef();
@@ -143,8 +177,14 @@ public:
 	{
 		m->decRef();
 	}
-	T* operator->() const {return m;}
-	T* getPtr() const { return m; }
+	inline T* operator->() const 
+	{
+		return m;
+	}
+	inline T* getPtr() const 
+	{ 
+		return m; 
+	}
 };
 
 #define _R Ref
@@ -192,7 +232,7 @@ public:
 		//The right hand Ref object is guaranteed to be valid
 		m->incRef();
 	}
-	NullableRef<T>& operator=(const NullableRef<T>& r)
+	inline NullableRef<T>& operator=(const NullableRef<T>& r)
 	{
 		if(r.m)
 			r.m->incRef();
@@ -203,7 +243,7 @@ public:
 			old->decRef();
 		return *this;
 	}
-	template<class D> NullableRef<T>& operator=(const NullableRef<D>& r)
+	template<class D> inline NullableRef<T>& operator=(const NullableRef<D>& r)
 	{
 		if(r.getPtr())
 			r->incRef();
@@ -214,7 +254,7 @@ public:
 			old->decRef();
 		return *this;
 	}
-	template<class D> NullableRef<T>& operator=(const Ref<D>& r)
+	template<class D> inline NullableRef<T>& operator=(const Ref<D>& r)
 	{
 		r.getPtr()->incRef();
 
@@ -224,37 +264,37 @@ public:
 			old->decRef();
 		return *this;
 	}
-	template<class D> bool operator==(const NullableRef<D>& r) const
+	template<class D> inline bool operator==(const NullableRef<D>& r) const
 	{
 		return m==r.getPtr();
 	}
-	template<class D> bool operator==(const Ref<D>& r) const
+	template<class D> inline bool operator==(const Ref<D>& r) const
 	{
 		return m==r.getPtr();
 	}
 	template<class D>
-	bool operator==(const D* r) const
+	inline bool operator==(const D* r) const
 	{
 		return m==r;
 	}
-	bool operator==(NullRef_t) const
+	inline bool operator==(NullRef_t) const
 	{
 		return m==NULL;
 	}
-	template<class D> bool operator!=(const NullableRef<D>& r) const
+	template<class D> inline bool operator!=(const NullableRef<D>& r) const
 	{
 		return m!=r.getPtr();
 	}
-	template<class D> bool operator!=(const Ref<D>& r) const
+	template<class D> inline bool operator!=(const Ref<D>& r) const
 	{
 		return m!=r.getPtr();
 	}
 	template<class D>
-	bool operator!=(const D* r) const
+	inline bool operator!=(const D* r) const
 	{
 		return m!=r;
 	}
-	bool operator!=(NullRef_t) const
+	inline bool operator!=(NullRef_t) const
 	{
 		return m!=NULL;
 	}
@@ -267,27 +307,34 @@ public:
 		if(m)
 			m->decRef();
 	}
-	T* operator->() const
+	inline T* operator->() const
 	{
 		if(m != NULL)
 			return m;
 		else
 			throw std::runtime_error("LS smart pointer: NULL pointer access");
 	}
-	T* getPtr() const { return m; }
-	bool isNull() const { return m==NULL; }
-	void reset()
+	inline T* getPtr() const { return m; }
+	inline bool isNull() const { return m==NULL; }
+	inline void reset()
 	{
 		T* old=m;
 		m=NULL;
 		if(old)
 			old->decRef();
 	}
-	void fakeRelease()
+	inline void fakeRelease()
 	{
 		m=NULL;
 	}
-	template<class D> NullableRef<D> cast() const
+	inline void forceDestruct()
+	{
+		assert(m->getConstant());
+		if (m)
+			delete m;
+		m = NULL;
+	}
+	template<class D> inline NullableRef<D> cast() const
 	{
 		if(!m)
 			return NullRef;
