@@ -25,11 +25,12 @@
 #include <list>
 #include <queue>
 #include <map>
-#include <boost/bimap.hpp>
+#include <unordered_set>
 #include <string>
 #include "swftypes.h"
 #include "scripting/flash/display/flashdisplay.h"
 #include "scripting/flash/net/flashnet.h"
+#include "scripting/flash/utils/IntervalManager.h"
 #include "timer.h"
 #include "memory_support.h"
 #include "platforms/engineutils.h"
@@ -52,9 +53,15 @@ class ParseThread;
 class PluginManager;
 class RenderThread;
 class SecurityManager;
+class LocaleManager;
 class Tag;
 class ApplicationDomain;
+class ASWorker;
+class WorkerDomain;
 class SecurityDomain;
+class Class_inherit;
+class FontTag;
+class SoundTransform;
 
 class RootMovieClip: public MovieClip
 {
@@ -63,9 +70,14 @@ protected:
 	URLInfo origin;
 private:
 	bool parsingIsFailed;
+	bool waitingforparser;
 	RGB Background;
-	Spinlock dictSpinlock;
-	std::list < DictionaryTag* > dictionary;
+	Mutex dictSpinlock;
+	std::unordered_map < uint32_t, DictionaryTag* > dictionary;
+	std::list< std::pair<tiny_string, DictionaryTag*> > classesToBeBound;
+	std::map < tiny_string,FontTag* > embeddedfonts;
+	std::map < uint32_t,FontTag* > embeddedfontsByID;
+
 	//frameSize and frameRate are valid only after the header has been parsed
 	RECT frameSize;
 	float frameRate;
@@ -73,16 +85,27 @@ private:
 	/* those are private because you shouldn't call mainClip->*,
 	 * but mainClip->getStage()->* instead.
 	 */
-	void initFrame();
-	void advanceFrame();
+	void initFrame() override;
+	void advanceFrame() override;
+	void executeFrameScript() override;
 	ACQUIRE_RELEASE_FLAG(finishedLoading);
+	
+	unordered_map<uint32_t,_NR<IFunction>> avm1ClassConstructors;
+	unordered_map<uint32_t,AVM1InitActionTag*> avm1InitActionTags;
 public:
 	RootMovieClip(_NR<LoaderInfo> li, _NR<ApplicationDomain> appDomain, _NR<SecurityDomain> secDomain, Class_base* c);
 	~RootMovieClip();
-	void finalize();
-	bool hasFinishedLoading() { return ACQUIRE_READ(finishedLoading); }
+	bool destruct() override;
+	bool hasFinishedLoading() override { return ACQUIRE_READ(finishedLoading); }
+	bool isWaitingForParser() { return waitingforparser; }
+	void constructionComplete() override;
+	void afterConstruction() override;
+	bool needsActionScript3() const override { return this->usesActionScript3;}
 	uint32_t version;
 	uint32_t fileLength;
+	bool hasSymbolClass;
+	bool hasMainClass;
+	bool usesActionScript3;
 	RGB getBackground();
 	void setBackground(const RGB& bg);
 	void setFrameSize(const RECT& f);
@@ -91,18 +114,20 @@ public:
 	void setFrameRate(float f);
 	void addToDictionary(DictionaryTag* r);
 	DictionaryTag* dictionaryLookup(int id);
+	DictionaryTag* dictionaryLookupByName(uint32_t nameID);
+	void resizeCompleted();
 	void labelCurrentFrame(const STRING& name);
 	void commitFrame(bool another);
 	void revertFrame();
 	void parsingFailed();
-	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const;
+	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const override;
 	void DLL_PUBLIC setOrigin(const tiny_string& u, const tiny_string& filename="");
-	URLInfo& getOrigin() { return origin; };
+	URLInfo& getOrigin() { return origin; }
 	void DLL_PUBLIC setBaseURL(const tiny_string& url);
 	const URLInfo& getBaseURL();
 /*	ASObject* getVariableByQName(const tiny_string& name, const tiny_string& ns);
 	void setVariableByQName(const tiny_string& name, const tiny_string& ns, ASObject* o);
-	void setVariableByMultiname(multiname& name, ASObject* o);
+	void setVariableByMultiname(multiname& name, asAtom o);
 	void setVariableByString(const std::string& s, ASObject* o);*/
 	static RootMovieClip* getInstance(_NR<LoaderInfo> li, _R<ApplicationDomain> appDomain, _R<SecurityDomain> secDomain);
 	/*
@@ -114,8 +139,20 @@ public:
 	 */
 	_NR<SecurityDomain> securityDomain;
 	//DisplayObject interface
-	_NR<RootMovieClip> getRoot();
+	_NR<RootMovieClip> getRoot() override;
+	_NR<Stage> getStage() override;
 	void addBinding(const tiny_string& name, DictionaryTag *tag);
+	void bindClass(const QName &classname, Class_inherit* cls);
+	void checkBinding(DictionaryTag* tag);
+	void registerEmbeddedFont(const tiny_string fontname, FontTag *tag);
+	FontTag* getEmbeddedFont(const tiny_string fontname) const;
+	FontTag* getEmbeddedFontByID(uint32_t fontID) const;
+	void setupAVM1RootMovie();
+	// map AVM1 class constructors to named tags
+	bool AVM1registerTagClass(const tiny_string& name, _NR<IFunction> theClassConstructor);
+	AVM1Function* AVM1getClassConstructor(uint32_t spriteID);
+	void AVM1registerInitActionTag(uint32_t spriteID, AVM1InitActionTag* tag);
+	void AVM1checkInitActions(MovieClip *sprite);
 };
 
 class ThreadProfile
@@ -136,17 +173,15 @@ private:
 	RGB color;
 	int32_t len;
 	uint32_t tickCount;
+	EngineData* engineData;
 public:
-	ThreadProfile(const RGB& c,uint32_t l):color(c),len(l),tickCount(0){}
+	ThreadProfile(const RGB& c,uint32_t l,EngineData* _engineData):color(c),len(l),tickCount(0),engineData(_engineData){}
 	void accountTime(uint32_t time);
 	void setTag(const std::string& tag);
 	void tick();
 	void plot(uint32_t max, cairo_t *cr);
 };
 
-enum BUILTIN_STRINGS { EMPTY=0, ANY, VOID, PROTOTYPE, LAST_BUILTIN_STRING };
-enum BUILTIN_NAMESPACES { EMPTY_NS=0 };
-
 class SystemState: public ITickJob, public InvalidateQueue
 {
 private:
@@ -159,15 +194,18 @@ private:
 	};
 	friend class SystemState::EngineCreator;
 	ThreadPool* threadPool;
+	ThreadPool* downloadThreadPool;
 	TimerThread* timerThread;
+	TimerThread* frameTimerThread;
 	Semaphore terminated;
 	float renderRate;
 	bool error;
 	bool shutdown;
+	bool firsttick;
+	bool localstorageallowed;
 	RenderThread* renderThread;
 	InputThread* inputThread;
 	EngineData* engineData;
-	Thread* mainThread;
 	void startRenderTicks();
 	Mutex rootMutex;
 	/**
@@ -183,7 +221,7 @@ private:
 	*/
 	void stopEngines();
 
-	void delayedCreation();
+	static void delayedCreation(SystemState* sys);
 	void delayedStopping();
 
 	/* dumpedSWFPathAvailable is signaled after dumpedSWFPath has been set */
@@ -204,6 +242,7 @@ private:
 	_NR<Undefined> undefined;
 	_NR<Boolean> trueRef;
 	_NR<Boolean> falseRef;
+	Class_base* objClassRef;
 
 	//Parameters/FlashVars
 	_NR<ASObject> parameters;
@@ -218,7 +257,7 @@ private:
 	char* cookiesFileName;
 
 	URLInfo url;
-	Spinlock profileDataSpinlock;
+	Mutex profileDataSpinlock;
 
 	Mutex mutexFrameListeners;
 	std::set<_R<DisplayObject>> frameListeners;
@@ -233,7 +272,11 @@ private:
 	/*
 	   The lock for the invalidate queue
 	*/
-	Spinlock invalidateQueueLock;
+	Mutex invalidateQueueLock;
+	
+	Mutex drawjobLock;
+	std::unordered_set<AsyncDrawJob*> drawJobsNew;
+	std::unordered_set<AsyncDrawJob*> drawJobsPending;
 #ifdef PROFILING_SUPPORT
 	/*
 	   Output file for the profiling data
@@ -248,22 +291,35 @@ private:
 	 * Pooling support
 	 */
 	mutable Mutex poolMutex;
-	boost::bimap<tiny_string, uint32_t> uniqueStringMap;
+	map<tiny_string, uint32_t> uniqueStringMap;
+	vector<tiny_string> uniqueStringIDMap;
 	uint32_t lastUsedStringId;
-	boost::bimap<nsNameAndKindImpl, uint32_t> uniqueNamespaceMap;
+	map<nsNameAndKindImpl, uint32_t> uniqueNamespaceImplMap;
+	unordered_map<uint32_t,nsNameAndKindImpl> uniqueNamespaceIDMap;
 	//This needs to be atomic because it's decremented without the mutex held
 	ATOMIC_INT32(lastUsedNamespaceId);
+	
+	Mutex mainsignalMutex;
+	Cond mainsignalCond;
 	void systemFinalize();
+	std::map<tiny_string, Class_base *> classnamemap;
 public:
 	void setURL(const tiny_string& url) DLL_PUBLIC;
+	tiny_string getDumpedSWFPath() const { return dumpedSWFPath;}
 
 	//Interative analysis flags
 	bool showProfilingData;
 	bool standalone;
+	bool allowFullscreen;
+	bool allowFullscreenInteractive;
 	//Flash for execution mode
-	enum FLASH_MODE { FLASH=0, AIR };
+	enum FLASH_MODE { FLASH=0, AIR, AVMPLUS };
 	const FLASH_MODE flashMode;
-	
+	uint32_t swffilesize;
+	asAtom nanAtom;
+	// the global object for AVM1
+	Global* avm1global;
+	void setupAVM1();
 	// Error types used to decide when to exit, extend as a bitmap
 	enum ERROR_TYPE { ERROR_NONE    = 0x0000,
 			  ERROR_PARSING = 0x0001,
@@ -275,10 +331,13 @@ public:
 	std::string& getErrorCause() { return errorCause; }
 	bool shouldTerminate() const;
 	bool isShuttingDown() const DLL_PUBLIC;
-	bool isOnError() const;
+	bool isOnError() const DLL_PUBLIC;
 	void setShutdownFlag() DLL_PUBLIC;
-	void tick();
-	void tickFence();
+	std::map<tiny_string, _R<SharedObject> > sharedobjectmap;
+	bool localStorageAllowed() const { return localstorageallowed; }
+	void setLocalStorageAllowed(bool allowed);
+	void tick() override;
+	void tickFence() override;
 	RenderThread* getRenderThread() const { return renderThread; }
 	InputThread* getInputThread() const { return inputThread; }
 	void setParamsAndEngine(EngineData* e, bool s) DLL_PUBLIC;
@@ -305,16 +364,35 @@ public:
 	ThreadProfile* allocateProfiler(const RGB& color);
 	std::list<ThreadProfile*> profilingData;
 	
-	Null* getNullRef() const;
-	Undefined* getUndefinedRef() const;
-	Boolean* getTrueRef() const;
-	Boolean* getFalseRef() const;
+	inline Null* getNullRef() const
+	{
+		return null.getPtr();
+	}
+	
+	inline Undefined* getUndefinedRef() const
+	{
+		return undefined.getPtr();
+	}
+	
+	inline Boolean* getTrueRef() const
+	{
+		return trueRef.getPtr();
+	}
+	
+	inline Boolean* getFalseRef() const
+	{
+		return falseRef.getPtr();
+	}
+
+	inline Class_base* getObjectClassRef() const
+	{
+		return objClassRef;
+	}
 
 	RootMovieClip* mainClip;
 	Stage* stage;
 	ABCVm* currentVm;
 
-	PluginManager* pluginManager;
 	AudioManager* audioManager;
 
 	//Application starting time in milliseconds
@@ -325,12 +403,14 @@ public:
 	//This is an array of fixed size, we can avoid using std::vector
 	Class_base** builtinClasses;
 	std::map<QName, Template_base*> templates;
-	std::map<QName, Class_base*> instantiatedTemplates;
 
+	std::map<QName,std::unordered_set<uint32_t>*> customclassoverriddenmethods;
+	
 	//Flags for command line options
 	bool useInterpreter;
 	bool useFastInterpreter;
 	bool useJit;
+	bool ignoreUnhandledExceptions;
 	ERROR_TYPE exitOnError;
 
 	//Parameters/FlashVars
@@ -346,7 +426,11 @@ public:
 
 	//Interfaces to the internal thread pool and timer thread
 	void addJob(IThreadJob* j) DLL_PUBLIC;
+	// downloaders may be executed from inside a job from the main threadpool,
+	// so we use a second threadpool for them, to avoid deadlocks
+	void addDownloadJob(IThreadJob* j) DLL_PUBLIC;
 	void addTick(uint32_t tickTime, ITickJob* job);
+	void addFrameTick(uint32_t tickTime, ITickJob* job);
 	void addWait(uint32_t waitTime, ITickJob* job);
 	void removeJob(ITickJob* job);
 
@@ -358,6 +442,13 @@ public:
 	 */
 	_NR<ApplicationDomain> systemDomain;
 
+	_NR<ASWorker> worker;
+	_NR<WorkerDomain> workerDomain;
+	bool singleworker;
+	Mutex workerMutex;
+	void addWorker(ASWorker* w);
+	void removeWorker(ASWorker* w);
+
 	//Stuff to be done once for process and not for plugin instance
 	static void staticInit() DLL_PUBLIC;
 	static void staticDeinit() DLL_PUBLIC;
@@ -365,6 +456,7 @@ public:
 	DownloadManager* downloadManager;
 	IntervalManager* intervalManager;
 	SecurityManager* securityManager;
+	LocaleManager* localeManager;
 	ExtScriptObject* extScriptObject;
 
 	enum SCALE_MODE { EXACT_FIT=0, NO_BORDER=1, NO_SCALE=2, SHOW_ALL=3 };
@@ -376,20 +468,21 @@ public:
 	//	NetConnection
 	ObjectEncoding::ENCODING staticNetConnectionDefaultObjectEncoding;
 	ObjectEncoding::ENCODING staticByteArrayDefaultObjectEncoding;
+	ObjectEncoding::ENCODING staticSharedObjectDefaultObjectEncoding;
+	bool staticSharedObjectPreventBackup;
 	
 	//enterFrame event management
 	void registerFrameListener(_R<DisplayObject> clip);
 	void unregisterFrameListener(_R<DisplayObject> clip);
 
-	//tags management
-	void registerTag(Tag* t);
-
 	//Invalidation queue management
-	void addToInvalidateQueue(_R<DisplayObject> d);
+	void addToInvalidateQueue(_R<DisplayObject> d) override;
 	void flushInvalidationQueue();
+	void AsyncDrawJobCompleted(AsyncDrawJob* j);
+	void swapAsyncDrawJobQueue();
 
 	//Resize support
-	void resizeCompleted() const;
+	void resizeCompleted();
 
 	/*
 	 * Support for class aliases in AMF3 serialization
@@ -405,6 +498,11 @@ public:
 	MemoryAccount* unaccountedMemory;
 	MemoryAccount* tagsMemory;
 	MemoryAccount* stringMemory;
+	MemoryAccount* textTokenMemory;
+	MemoryAccount* shapeTokenMemory;
+	MemoryAccount* morphShapeTokenMemory;
+	MemoryAccount* bitmapTokenMemory;
+	MemoryAccount* spriteTokenMemory;
 #ifdef MEMORY_USAGE_PROFILING
 	void saveMemoryUsageInformation(std::ofstream& out, int snapshotCount) const;
 #endif
@@ -429,6 +527,31 @@ public:
 
 	//Opening web pages
 	void openPageInBrowser(const tiny_string& url, const tiny_string& window);
+
+	void showMouseCursor(bool visible);
+	void waitRendering() DLL_PUBLIC;
+	EngineData* getEngineData() { return engineData;}
+	uint32_t getSwfVersion();
+
+	// these methods ensure that externalcallevents are executed as soon and as fast as possible
+	// the ppapi plugin needs this because external call events are blocking the main plugin thread 
+	// so we need to make sure external call events are executed even if another plugin method is currently running in another thread
+	void checkExternalCallEvent() DLL_PUBLIC;
+	void waitMainSignal() DLL_PUBLIC;
+	void sendMainSignal() DLL_PUBLIC;
+
+	void dumpStacktrace();
+	
+	// static class properties are named static_<classname>_<propertyname>
+	_NR<SoundTransform> static_SoundMixer_soundTransform;
+	int static_SoundMixer_bufferTime;
+	_NR<ASObject> static_ObjectEncoding_dynamicPropertyWriter;
+
+	ACQUIRE_RELEASE_FLAG(isinitialized);
+	Mutex initializedMutex;
+	Cond initializedCond;
+	void waitInitialized();
+	void getClassInstanceByName(asAtom &ret, const tiny_string& clsname);
 };
 
 class ParseThread: public IThreadJob
@@ -456,11 +579,11 @@ private:
 	std::streambuf* backend;
 	Loader *loader;
 	_NR<DisplayObject> parsedObject;
-	Spinlock objectSpinlock;
+	Mutex objectSpinlock;
 	tiny_string url;
 	FILE_TYPE fileType;
 	void threadAbort();
-	void jobFence() {};
+	void jobFence() {}
 	void parseSWFHeader(RootMovieClip *root, UI8 ver);
 	void parseSWF(UI8 ver);
 	void parseBitmap();
@@ -473,6 +596,9 @@ SystemState* getSys() DLL_PUBLIC;
 void setTLSSys(SystemState* sys) DLL_PUBLIC;
 
 ParseThread* getParseThread();
+/* Returns the thread-specific SystemState */
+ASWorker* getWorker() DLL_PUBLIC;
+void setTLSWorker(ASWorker* worker) DLL_PUBLIC;
 
-};
+}
 #endif /* SWF_H */
