@@ -27,26 +27,28 @@
 #include <cmath>
 #include <cairo.h>
 #include <glib.h>
-#include <iomanip> 
+#include <iomanip>
+#include <algorithm>
 
 #include "exceptions.h"
 #include "compat.h"
 #include "scripting/toplevel/ASString.h"
 #include "scripting/flash/display/BitmapData.h"
+#include "scripting/flash/geom/flashgeom.h"
+#include "scripting/toplevel/toplevel.h"
+
 
 using namespace std;
 using namespace lightspark;
 
-multiname::multiname(MemoryAccount* m):name_o(NULL),ns(reporter_allocator<nsNameAndKind>(m)),name_type(NAME_OBJECT),isAttribute(false)
+const tiny_string multiname::qualifiedString(SystemState* sys, bool forDescribeType) const
 {
-}
-
-tiny_string multiname::qualifiedString() const
-{
-	assert_and_throw(ns.size()==1);
+	assert_and_throw(ns.size()>=1);
 	assert_and_throw(name_type==NAME_STRING);
-	const tiny_string nsName=ns[0].getImpl().name;
-	const tiny_string& name=getSys()->getStringFromUniqueId(name_s_id);
+	if (forDescribeType && name_s_id == BUILTIN_STRINGS::ANY)
+		return "*";
+	const tiny_string nsName=sys->getStringFromUniqueId(ns[0].nsNameId);
+	const tiny_string name=sys->getStringFromUniqueId(name_s_id);
 	if(nsName.empty())
 		return name;
 	else
@@ -54,22 +56,31 @@ tiny_string multiname::qualifiedString() const
 		tiny_string ret=nsName;
 		ret+="::";
 		ret+=name;
+		if (forDescribeType && ret.startsWith("__AS3__.vec::Vector$"))
+		{
+			tiny_string ret2 = "__AS3__.vec::Vector.<";
+			ret2 += ret.substr(strlen("__AS3__.vec::Vector$"),ret.numChars());
+			ret2 += ">";
+			return ret2;
+		}
 		return ret;
 	}
 }
 
-tiny_string multiname::normalizedName() const
+const tiny_string multiname::normalizedName(SystemState* sys) const
 {
 	switch(name_type)
 	{
 		case multiname::NAME_INT:
 			return Integer::toString(name_i);
+		case multiname::NAME_UINT:
+			return UInteger::toString(name_ui);
 		case multiname::NAME_NUMBER:
 			return Number::toString(name_d);
 		case multiname::NAME_STRING:
-			return getSys()->getStringFromUniqueId(name_s_id);
+			return sys->getStringFromUniqueId(name_s_id);
 		case multiname::NAME_OBJECT:
-			return name_o->toString();
+			return name_o ? name_o->toString() : "*";
 		default:
 			assert("Unexpected name kind" && false);
 			//Should never reach this
@@ -77,16 +88,20 @@ tiny_string multiname::normalizedName() const
 	}
 }
 
-uint32_t multiname::normalizedNameId() const
+uint32_t multiname::normalizedNameId(SystemState* sys) const
 {
 	switch(name_type)
 	{
 		case multiname::NAME_STRING:
 			return name_s_id;
 		case multiname::NAME_INT:
+		case multiname::NAME_UINT:
 		case multiname::NAME_NUMBER:
 		case multiname::NAME_OBJECT:
-			return getSys()->getUniqueStringId(normalizedName());
+			if (name_s_id != UINT32_MAX)
+				return name_s_id;
+			else
+				return sys->getUniqueStringId(normalizedName(sys));
 		default:
 			assert("Unexpected name kind" && false);
 			//Should never reach this
@@ -94,45 +109,94 @@ uint32_t multiname::normalizedNameId() const
 	}
 }
 
-void multiname::setName(ASObject* n)
+const tiny_string multiname::normalizedNameUnresolved(SystemState* sys) const
+{
+	switch(name_type)
+	{
+		case multiname::NAME_INT:
+			return Integer::toString(name_i);
+		case multiname::NAME_UINT:
+			return UInteger::toString(name_ui);
+		case multiname::NAME_NUMBER:
+			return Number::toString(name_d);
+		case multiname::NAME_STRING:
+			return sys->getStringFromUniqueId(name_s_id);
+		case multiname::NAME_OBJECT:
+			return name_o ? name_o->getClassName() : "*";
+		default:
+			assert("Unexpected name kind" && false);
+			//Should never reach this
+			return "";
+	}
+}
+
+void multiname::setName(asAtom& n,SystemState* sys)
 {
 	if (name_type==NAME_OBJECT && name_o!=NULL) {
 		name_o->decRef();
 		name_o = NULL;
 	}
 
-	if(n->is<Integer>())
+	switch(asAtomHandler::getObjectType(n))
 	{
-		name_i=n->as<Integer>()->val;
+	case T_INTEGER:
+		name_i=asAtomHandler::getInt(n);
 		name_type = NAME_INT;
-	}
-	else if(n->is<UInteger>())
-	{
-		name_i=n->as<UInteger>()->val;
-		name_type = NAME_INT;
-	}
-	else if(n->is<Number>())
-	{
-		name_d=n->as<Number>()->val;
+		name_s_id = UINT32_MAX;
+		isInteger=true;
+		break;
+	case T_UINTEGER:
+		name_ui=asAtomHandler::getUInt(n);
+		name_type = NAME_UINT;
+		name_s_id = UINT32_MAX;
+		isInteger=true;
+		break;
+	case T_NUMBER:
+		name_d=asAtomHandler::toNumber(n);
 		name_type = NAME_NUMBER;
-	}
-	else if(n->getObjectType()==T_QNAME)
-	{
-		ASQName* qname=static_cast<ASQName*>(n);
-		name_s_id=getSys()->getUniqueStringId(qname->local_name);
-		name_type = NAME_STRING;
-	}
-	else if(n->getObjectType()==T_STRING)
-	{
-		ASString* o=static_cast<ASString*>(n);
-		name_s_id=getSys()->getUniqueStringId(o->data);
-		name_type = NAME_STRING;
-	}
-	else
-	{
-		n->incRef();
-		name_o=n;
+		name_s_id = UINT32_MAX;
+		isInteger=false;
+		break;
+	case T_BOOLEAN:
+		name_i=asAtomHandler::toInt(n);
+		name_type = NAME_INT;
+		name_s_id = UINT32_MAX;
+		isInteger=true;
+		break;
+	case T_QNAME:
+		{
+			ASQName* qname=asAtomHandler::as<ASQName>(n);
+			name_s_id=qname->local_name;
+			name_type = NAME_STRING;
+			isInteger=Array::isIntegerWithoutLeadingZeros(sys->getStringFromUniqueId(name_s_id));
+		}
+		break;
+	case T_STRING:
+		{
+			name_s_id=asAtomHandler::toStringId(n,sys);
+			name_type = NAME_STRING;
+			isInteger=Array::isIntegerWithoutLeadingZeros(sys->getStringFromUniqueId(name_s_id));
+		}
+		break;
+	case T_NULL:
+		name_o=sys->getNullRef();
 		name_type = NAME_OBJECT;
+		name_s_id = UINT32_MAX;
+		isInteger=false;
+		break;
+	case T_UNDEFINED:
+		name_o=sys->getUndefinedRef();
+		name_type = NAME_OBJECT;
+		name_s_id = UINT32_MAX;
+		isInteger=false;
+		break;
+	default:
+		ASATOM_INCREF(n);
+		name_o=asAtomHandler::getObject(n);
+		name_type = NAME_OBJECT;
+		name_s_id = UINT32_MAX;
+		isInteger=false;
+		break;
 	}
 }
 
@@ -145,8 +209,10 @@ void multiname::resetNameIfObject()
 	}
 }
 
-bool multiname::toUInt(uint32_t& index, bool acceptStringFractions) const
+bool multiname::toUInt(SystemState* sys, uint32_t& index, bool acceptStringFractions, bool *isNumber) const
 {
+	if (isNumber)
+		*isNumber = false;
 	switch(name_type)
 	{
 		//We try to convert this to an index, otherwise bail out
@@ -155,13 +221,19 @@ bool multiname::toUInt(uint32_t& index, bool acceptStringFractions) const
 		{
 			tiny_string str;
 			if(name_type==multiname::NAME_STRING)
-				str=getSys()->getStringFromUniqueId(name_s_id);
+			{
+				if (name_s_id < BUILTIN_STRINGS::LAST_BUILTIN_STRING &&
+					(name_s_id < 0x30 || name_s_id > 0x39)) // char 0-9
+					return false;
+				str=sys->getStringFromUniqueId(name_s_id);
+			}
 			else
 				str=name_o->toString();
 
 			if(str.empty())
 				return false;
 			index=0;
+			uint64_t parsed = 0;
 			for(auto i=str.begin(); i!=str.end(); ++i)
 			{
 				if (*i == '.' && acceptStringFractions)
@@ -174,15 +246,25 @@ bool multiname::toUInt(uint32_t& index, bool acceptStringFractions) const
 					++i;
 					for (; i!=str.end(); ++i)
 						if (*i != '0')
+						{
+							if (isNumber)
+								*isNumber = true;
 							return false;
-					return true;
+						}
+					break;
 				}
 				else if(!i.isdigit())
 					return false;
-
-				index*=10;
-				index+=i.digit_value();
+				parsed*=10;
+				parsed+=i.digit_value();
+				if (parsed > UINT32_MAX)
+					break;
 			}
+
+			if (parsed > UINT32_MAX)
+				return false;
+
+			index = (uint32_t)parsed;
 			break;
 		}
 		//This is already an int, so its good enough
@@ -191,8 +273,11 @@ bool multiname::toUInt(uint32_t& index, bool acceptStringFractions) const
 				return false;
 			index=name_i;
 			break;
+		case multiname::NAME_UINT:
+			index=name_ui;
+			break;
 		case multiname::NAME_NUMBER:
-			if(!Number::isInteger(name_d) || name_d < 0)
+			if(!Number::isInteger(name_d) || name_d < 0 || name_d > UINT32_MAX)
 				return false;
 			index=name_d;
 			break;
@@ -206,7 +291,7 @@ bool multiname::toUInt(uint32_t& index, bool acceptStringFractions) const
 
 std::ostream& lightspark::operator<<(std::ostream& s, const QName& r)
 {
-	s << r.ns << ':' << r.name;
+	s << getSys()->getStringFromUniqueId(r.nsStringId) << ':' << getSys()->getStringFromUniqueId(r.nameId);
 	return s;
 }
 
@@ -220,7 +305,7 @@ std::ostream& lightspark::operator<<(std::ostream& s, const tiny_string& r)
 std::ostream& lightspark::operator<<(std::ostream& s, const nsNameAndKind& r)
 {
 	const char* prefix;
-	switch(r.getImpl().kind)
+	switch(r.kind)
 	{
 		case NAMESPACE:
 			prefix="ns:";
@@ -249,7 +334,7 @@ std::ostream& lightspark::operator<<(std::ostream& s, const nsNameAndKind& r)
 			prefix="";
 			break;
 	}
-	s << prefix << r.getImpl().name;
+	s << prefix << getSys()->getStringFromUniqueId(r.nsNameId);
 	return s;
 }
 
@@ -261,6 +346,8 @@ std::ostream& lightspark::operator<<(std::ostream& s, const multiname& r)
 	}
 	if(r.name_type==multiname::NAME_INT)
 		s << r.name_i;
+	else if(r.name_type==multiname::NAME_UINT)
+		s << r.name_ui;
 	else if(r.name_type==multiname::NAME_NUMBER)
 		s << r.name_d;
 	else if(r.name_type==multiname::NAME_STRING)
@@ -270,7 +357,7 @@ std::ostream& lightspark::operator<<(std::ostream& s, const multiname& r)
 	return s;
 }
 
-lightspark::RECT::RECT()
+lightspark::RECT::RECT():Xmin(0),Xmax(0),Ymin(0),Ymax(0)
 {
 }
 
@@ -443,8 +530,7 @@ void LINESTYLEARRAY::appendStyles(const LINESTYLEARRAY& r)
 	assert_and_throw(r.version==version);
 	if(version<4)
 		LineStyles.insert(LineStyles.end(),r.LineStyles.begin(),r.LineStyles.end());
-	else
-		LineStyles2.insert(LineStyles2.end(),r.LineStyles2.begin(),r.LineStyles2.end());
+	LineStyles2.insert(LineStyles2.end(),r.LineStyles2.begin(),r.LineStyles2.end());
 }
 
 std::istream& lightspark::operator>>(std::istream& s, LINESTYLEARRAY& v)
@@ -461,6 +547,10 @@ std::istream& lightspark::operator>>(std::istream& s, LINESTYLEARRAY& v)
 			LINESTYLE tmp(v.version);
 			s >> tmp;
 			v.LineStyles.push_back(tmp);
+			LINESTYLE2 tmp2(v.version);
+			tmp2.Color = tmp.Color;
+			tmp2.Width = tmp.Width;
+			v.LineStyles2.push_back(tmp2);
 		}
 	}
 	else
@@ -489,6 +579,12 @@ std::istream& lightspark::operator>>(std::istream& s, MORPHLINESTYLEARRAY& v)
 			MORPHLINESTYLE t;
 			s >> t;
 			v.LineStyles.emplace_back(t);
+			MORPHLINESTYLE2 tmp2;
+			tmp2.StartWidth = t.StartWidth;
+			tmp2.EndWidth = t.EndWidth;
+			tmp2.StartColor = t.StartColor;
+			tmp2.EndColor = t.EndColor;
+			v.LineStyles2.push_back(tmp2);
 		}
 	}
 	else
@@ -515,10 +611,14 @@ std::istream& lightspark::operator>>(std::istream& s, FILLSTYLEARRAY& v)
 	assert(v.version!=0xff);
 	UI8 FillStyleCount;
 	s >> FillStyleCount;
+	int fsc = FillStyleCount;
 	if(FillStyleCount==0xff)
-		LOG(LOG_ERROR,_("Fill array extended not supported"));
-
-	for(int i=0;i<FillStyleCount;i++)
+	{
+		UI16_SWF ExtendedFillStyleCount;
+		s >> ExtendedFillStyleCount;
+		fsc = ExtendedFillStyleCount;
+	}
+	for(int i=0;i<fsc;i++)
 	{
 		FILLSTYLE t(v.version);
 		s >> t;
@@ -547,6 +647,9 @@ std::istream& lightspark::operator>>(std::istream& s, SHAPE& v)
 	BitStream bs(s);
 	v.NumFillBits=UB(4,bs);
 	v.NumLineBits=UB(4,bs);
+	// this is not mentioned in the specs, but it seems if NumFillBits and NumLineBits are 0 in font tags, no shaperecords are read.
+	if (v.forfont && v.NumFillBits == 0 && v.NumLineBits==0)
+		return s;
 	do
 	{
 		v.ShapeRecords.push_back(SHAPERECORD(&v,bs));
@@ -786,9 +889,11 @@ std::istream& lightspark::operator>>(std::istream& s, FILLSTYLE& v)
 				if(!b)
 				{
 					LOG(LOG_ERROR,"Invalid bitmap ID " << bitmapId);
-					throw ParseException("Invalid ID for bitmap");
+					v.bitmap.reset();
+					//throw ParseException("Invalid ID for bitmap");
 				}
-				v.bitmap = b->getBitmap();
+				else
+					v.bitmap = b->getBitmap();
 			}
 			catch(RunTimeException& e)
 			{
@@ -806,7 +911,7 @@ std::istream& lightspark::operator>>(std::istream& s, FILLSTYLE& v)
 	else
 	{
 		LOG(LOG_ERROR,_("Not supported fill style ") << (int)v.FillStyleType);
-		throw ParseException("Not supported fill style");
+	throw ParseException("Not supported fill style");
 	}
 	return s;
 }
@@ -821,11 +926,13 @@ std::istream& lightspark::operator>>(std::istream& s, MORPHFILLSTYLE& v)
 	{
 		s >> v.StartColor >> v.EndColor;
 	}
-	else if(v.FillStyleType==LINEAR_GRADIENT || v.FillStyleType==RADIAL_GRADIENT)
+	else if(v.FillStyleType==LINEAR_GRADIENT || v.FillStyleType==RADIAL_GRADIENT || v.FillStyleType==FOCAL_RADIAL_GRADIENT)
 	{
 		s >> v.StartGradientMatrix >> v.EndGradientMatrix;
-		UI8 NumGradients;
-		s >> NumGradients;
+		BitStream bs(s);
+		v.SpreadMode = UB(2,bs);
+		v.InterpolationMode = UB(2,bs);
+		int NumGradients = UB(4,bs);
 		UI8 t;
 		RGBA t2;
 		for(int i=0;i<NumGradients;i++)
@@ -837,6 +944,8 @@ std::istream& lightspark::operator>>(std::istream& s, MORPHFILLSTYLE& v)
 			v.EndRatios.push_back(t);
 			v.EndColors.push_back(t2);
 		}
+		if (v.FillStyleType==FOCAL_RADIAL_GRADIENT)
+			s >> v.StartFocalPoint >> v.EndFocalPoint;
 	}
 	else if(v.FillStyleType==REPEATING_BITMAP
 		|| v.FillStyleType==CLIPPED_BITMAP
@@ -848,7 +957,7 @@ std::istream& lightspark::operator>>(std::istream& s, MORPHFILLSTYLE& v)
 	}
 	else
 	{
-		LOG(LOG_ERROR,_("Not supported fill style 0x") << hex << (int)v.FillStyleType << dec << _("... Aborting"));
+		LOG(LOG_ERROR,_("Not supported fill style 0x") << hex << (int)v.FillStyleType << " at "<< s.tellg()<< dec <<  _("... Aborting"));
 	}
 	return s;
 }
@@ -907,15 +1016,15 @@ SHAPERECORD::SHAPERECORD(SHAPE* p,BitStream& bs):parent(p),MoveBits(0),MoveDelta
 		}
 		if(StateFillStyle0)
 		{
-			FillStyle0=UB(parent->NumFillBits,bs)+p->fillOffset;
+			FillStyle0=UB(parent->NumFillBits,bs);
 		}
 		if(StateFillStyle1)
 		{
-			FillStyle1=UB(parent->NumFillBits,bs)+p->fillOffset;
+			FillStyle1=UB(parent->NumFillBits,bs);
 		}
 		if(StateLineStyle)
 		{
-			LineStyle=UB(parent->NumLineBits,bs)+p->lineOffset;
+			LineStyle=UB(parent->NumLineBits,bs);
 		}
 		if(StateNewStyles && parent->version >= 2)
 		{
@@ -925,17 +1034,28 @@ SHAPERECORD::SHAPERECORD(SHAPE* p,BitStream& bs):parent(p),MoveBits(0),MoveDelta
 			bs.pos=0;
 			FILLSTYLEARRAY a(ps->FillStyles.version);
 			bs.f >> a;
-			p->fillOffset=ps->FillStyles.FillStyles.size();
-			ps->FillStyles.appendStyles(a);
+			if (!a.FillStyles.empty())
+			{
+				p->fillOffset=ps->FillStyles.FillStyles.size();
+				ps->FillStyles.appendStyles(a);
+			}
 
 			LINESTYLEARRAY b(ps->LineStyles.version);
 			bs.f >> b;
-			p->lineOffset=ps->LineStyles.LineStyles.size();
-			ps->LineStyles.appendStyles(b);
-
+			if (!b.LineStyles2.empty())
+			{
+				p->lineOffset=ps->LineStyles.LineStyles2.size();
+				ps->LineStyles.appendStyles(b);
+			}
 			parent->NumFillBits=UB(4,bs);
 			parent->NumLineBits=UB(4,bs);
 		}
+		if(StateFillStyle0 && (FillStyle0 != 0))
+			FillStyle0+=p->fillOffset;
+		if(StateFillStyle1 && (FillStyle1 != 0))
+			FillStyle1+=p->fillOffset;
+		if(StateLineStyle && (LineStyle != 0))
+			LineStyle+=p->lineOffset;
 	}
 }
 
@@ -948,12 +1068,14 @@ void CXFORMWITHALPHA::getParameters(number_t& redMultiplier,
 				    number_t& blueOffset,
 				    number_t& alphaOffset) const
 {
+	// multipliers are stored as values from 0.0 to 1.0
 	if (HasMultTerms)
 	{
-		redMultiplier = RedMultTerm;
-		greenMultiplier = GreenMultTerm;
-		blueMultiplier = BlueMultTerm;
-		alphaMultiplier = AlphaMultTerm;
+		redMultiplier = RedMultTerm/256.0;
+		greenMultiplier = GreenMultTerm/256.0;
+		blueMultiplier = BlueMultTerm/256.0;
+		alphaMultiplier = AlphaMultTerm/256.0;
+		
 	}
 	else
 	{
@@ -1022,6 +1144,11 @@ std::istream& lightspark::operator>>(std::istream& stream, MATRIX& v)
 		v.xx=FB(NScaleBits,bs);
 		v.yy=FB(NScaleBits,bs);
 	}
+	else
+	{
+		v.xx = 1;
+		v.yy = 1;
+	}
 	int HasRotate=UB(1,bs);
 	if(HasRotate)
 	{
@@ -1029,6 +1156,11 @@ std::istream& lightspark::operator>>(std::istream& stream, MATRIX& v)
 		v.yx=FB(NRotateBits,bs);
 		v.xy=FB(NRotateBits,bs);
 	}
+	else
+	{
+		v.yx = 0;
+		v.xy = 0;
+	}
 	int NTranslateBits=UB(5,bs);
 	v.x0=SB(NTranslateBits,bs)/20;
 	v.y0=SB(NTranslateBits,bs)/20;
@@ -1080,9 +1212,8 @@ std::istream& lightspark::operator>>(std::istream& stream, FILTERLIST& v)
 
 std::istream& lightspark::operator>>(std::istream& stream, FILTER& v)
 {
-	UI8 FilterID;
-	stream >> FilterID;
-	switch(FilterID)
+	stream >> v.FilterID;
+	switch(v.FilterID)
 	{
 		case 0:
 			stream >> v.DropShadowFilter;
@@ -1109,7 +1240,7 @@ std::istream& lightspark::operator>>(std::istream& stream, FILTER& v)
 			stream >> v.GradientBevelFilter;
 			break;
 		default:
-			LOG(LOG_ERROR,_("Unsupported Filter Id ") << (int)FilterID);
+			LOG(LOG_ERROR,_("Unsupported Filter Id ") << (int)v.FilterID);
 			throw ParseException("Unsupported Filter Id");
 	}
 	return stream;
@@ -1195,6 +1326,8 @@ std::istream& lightspark::operator>>(std::istream& stream, GRADIENTGLOWFILTER& v
 	}
 	stream >> v.BlurX;
 	stream >> v.BlurY;
+	stream >> v.Angle;
+	stream >> v.Distance;
 	stream >> v.Strength;
 	BitStream bs(stream);
 	v.InnerGlow = UB(1,bs);
@@ -1267,19 +1400,72 @@ std::istream& lightspark::operator>>(std::istream& stream, GRADIENTBEVELFILTER&
 
 std::istream& lightspark::operator>>(std::istream& s, CLIPEVENTFLAGS& v)
 {
-	/* In SWF version <=5 this was UI16_SWF,
-	 * but parsing will then stop before we get here
-	 * to fallback on gnash
-	 */
-	UI32_SWF t;
-	s >> t;
-	v.toParse=t;
+	BitStream bs(s);
+	v.ClipEventKeyUp=UB(1,bs);
+	v.ClipEventKeyDown=UB(1,bs);
+	v.ClipEventMouseUp=UB(1,bs);
+	v.ClipEventMouseDown=UB(1,bs);
+	v.ClipEventMouseMove=UB(1,bs);
+	v.ClipEventUnload=UB(1,bs);
+	v.ClipEventEnterFrame=UB(1,bs);
+	v.ClipEventLoad=UB(1,bs);
+	v.ClipEventDragOver=UB(1,bs);
+	v.ClipEventRollOut=UB(1,bs);
+	v.ClipEventRollOver=UB(1,bs);
+	v.ClipEventReleaseOutside=UB(1,bs);
+	v.ClipEventRelease=UB(1,bs);
+	v.ClipEventPress=UB(1,bs);
+	v.ClipEventInitialize=UB(1,bs);
+	v.ClipEventData=UB(1,bs);
+	if (v.getSWFVersion() <= 5)
+	{
+		v.ClipEventConstruct=false;
+		v.ClipEventKeyPress=false;
+		v.ClipEventDragOut=false;
+	}
+	else
+	{
+		UB(5,bs);
+		v.ClipEventConstruct=UB(1,bs);
+		v.ClipEventKeyPress=UB(1,bs);
+		v.ClipEventDragOut=UB(1,bs);
+		UB(8,bs);
+	}
+	if (v.ClipEventUnload)
+		LOG(LOG_NOT_IMPLEMENTED,"CLIPEVENTFLAG ClipEventUnload not handled");
+	if (v.ClipEventDragOver)
+		LOG(LOG_NOT_IMPLEMENTED,"CLIPEVENTFLAG ClipEventDragOver not handled");
+	if (v.ClipEventData)
+		LOG(LOG_NOT_IMPLEMENTED,"CLIPEVENTFLAG ClipEventData not handled");
+	if (v.ClipEventKeyPress)
+		LOG(LOG_NOT_IMPLEMENTED,"CLIPEVENTFLAG ClipEventKeyPress not handled");
+	if (v.ClipEventDragOut)
+		LOG(LOG_NOT_IMPLEMENTED,"CLIPEVENTFLAG ClipEventDragOut not handled");
 	return s;
 }
 
 bool CLIPEVENTFLAGS::isNull()
 {
-	return toParse==0;
+	return
+			!ClipEventKeyUp &&
+			!ClipEventKeyDown &&
+			!ClipEventMouseUp &&
+			!ClipEventMouseDown &&
+			!ClipEventMouseMove &&
+			!ClipEventUnload &&
+			!ClipEventEnterFrame &&
+			!ClipEventLoad &&
+			!ClipEventDragOver &&
+			!ClipEventRollOut &&
+			!ClipEventRollOver &&
+			!ClipEventReleaseOutside &&
+			!ClipEventRelease &&
+			!ClipEventPress &&
+			!ClipEventInitialize &&
+			!ClipEventData &&
+			!ClipEventConstruct &&
+			!ClipEventKeyPress &&
+			!ClipEventDragOut;
 }
 
 std::istream& lightspark::operator>>(std::istream& s, CLIPACTIONRECORD& v)
@@ -1288,8 +1474,21 @@ std::istream& lightspark::operator>>(std::istream& s, CLIPACTIONRECORD& v)
 	if(v.EventFlags.isNull())
 		return s;
 	s >> v.ActionRecordSize;
-	LOG(LOG_NOT_IMPLEMENTED,_("Skipping ") << v.ActionRecordSize << _(" of action data"));
-	ignore(s,v.ActionRecordSize);
+	uint32_t len = v.ActionRecordSize;
+	if (v.EventFlags.ClipEventKeyPress)
+	{
+		s >> v.KeyCode;
+		len -= 1;
+	}
+	if (v.datatag)
+	{
+		v.startactionpos=v.datatag->numbytes;//+datatagskipbytes;
+		v.actions.resize(len+v.startactionpos);
+		memcpy(v.actions.data(),v.datatag->bytes,v.datatag->numbytes);
+	}
+	else
+		v.actions.resize(len);
+	s.read((char*)(v.actions.data()+v.startactionpos),len);
 	return s;
 }
 
@@ -1304,7 +1503,9 @@ std::istream& lightspark::operator>>(std::istream& s, CLIPACTIONS& v)
 	s >> Reserved >> v.AllEventFlags;
 	while(1)
 	{
-		CLIPACTIONRECORD t;
+		CLIPACTIONRECORD t(v.AllEventFlags.getSWFVersion(),v.datatag);
+		// use datatag only on first clipaction
+		v.datatag=nullptr;
 		s >> t;
 		if(t.isLast())
 			break;
@@ -1312,37 +1513,117 @@ std::istream& lightspark::operator>>(std::istream& s, CLIPACTIONS& v)
 	}
 	return s;
 }
-
-ASObject* lightspark::abstract_d(number_t i)
+ASObject* lightspark::abstract_s(SystemState *sys)
 {
-	Number* ret=Class<Number>::getInstanceS(i);
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ret->stringId = BUILTIN_STRINGS::EMPTY;
+	ret->hasId = true;
+	ret->datafilled=true;
+	return ret;
+}
+ASObject* lightspark::abstract_s(SystemState *sys, const char* s, uint32_t len)
+{
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ret->data = std::string(s,len);
+	ret->stringId = UINT32_MAX;
+	ret->hasId = false;
+	ret->datafilled=true;
+	return ret;
+}
+ASObject* lightspark::abstract_s(SystemState *sys, const char* s)
+{
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ret->data = s;
+	ret->stringId = UINT32_MAX;
+	ret->hasId = false;
+	ret->datafilled=true;
+	return ret;
+}
+ASObject* lightspark::abstract_s(SystemState *sys, const tiny_string& s)
+{
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ret->data = s;
+	ret->stringId = UINT32_MAX;
+	ret->hasId = false;
+	ret->datafilled=true;
+	return ret;
+}
+ASObject* lightspark::abstract_s(SystemState *sys, uint32_t stringId)
+{
+	ASString* ret= Class<ASString>::getInstanceSNoArgs(sys);
+	ret->stringId = stringId;
+	ret->hasId = true;
+	ret->datafilled=false;
+	return ret;
+}
+ASObject* lightspark::abstract_d(SystemState* sys,number_t i)
+{
+	Number* ret=Class<Number>::getInstanceSNoArgs(sys);
+	ret->dval = i;
+	ret->isfloat = true;
+	return ret;
+}
+ASObject* lightspark::abstract_d_constant(SystemState* sys,number_t i)
+{
+	Number* ret=new (sys->unaccountedMemory) Number(Class<Number>::getRef(sys).getPtr());
+	ret->dval = i;
+	ret->isfloat = true;
+	ret->setConstant();
+	return ret;
+}
+ASObject* lightspark::abstract_di(SystemState* sys,int64_t i)
+{
+	Number* ret=Class<Number>::getInstanceSNoArgs(sys);
+	ret->ival = i;
+	ret->isfloat = false;
+	return ret;
+}
+ASObject* lightspark::abstract_i(SystemState *sys, int32_t i)
+{
+	Integer* ret=Class<Integer>::getInstanceSNoArgs(sys);
+	ret->val = i;
 	return ret;
 }
 
-ASObject* lightspark::abstract_i(int32_t i)
+ASObject* lightspark::abstract_ui(SystemState *sys, uint32_t i)
 {
-	Integer* ret=Class<Integer>::getInstanceS(i);
+	UInteger* ret=Class<UInteger>::getInstanceSNoArgs(sys);
+	ret->val = i;
 	return ret;
 }
-
-ASObject* lightspark::abstract_ui(uint32_t i)
+ASObject* lightspark::abstract_null(SystemState *sys)
 {
-	UInteger* ret=Class<UInteger>::getInstanceS(i);
-	return ret;
+	return sys->getNullRef();
+}
+ASObject* lightspark::abstract_undefined(SystemState *sys)
+{
+	return sys->getUndefinedRef();
+}
+ASObject *lightspark::abstract_b(SystemState *sys, bool v)
+{
+	if(v==true)
+		return sys->getTrueRef();
+	else
+		return sys->getFalseRef();
 }
 
 void lightspark::stringToQName(const tiny_string& tmp, tiny_string& name, tiny_string& ns)
 {
 	//Ok, let's split our string into namespace and name part
-	char* collon=tmp.strchrr(':');
+	char* collon=tmp.strchr(':');
 	if(collon)
 	{
 		/* collon is not the first character and there is
 		 * another collon before it */
-		assert_and_throw(collon != tmp.raw_buf() && *(collon-1) == ':');
+		if (collon == tmp.raw_buf() || *(collon+1) != ':')
+		{
+			name=tmp;
+			ns="";
+			return;
+		}
 		uint32_t collon_offset = collon-tmp.raw_buf();
-		ns = tmp.substr_bytes(0,collon_offset-1);
-		name = tmp.substr_bytes(collon_offset+1,tmp.numChars()-collon_offset-1);
+		ns = tmp.substr_bytes(0,collon_offset);
+		name = tmp.substr_bytes(collon_offset+2,tmp.numBytes()-collon_offset-2);
 		return;
 	}
 	// No namespace, look for a package name
@@ -1351,7 +1632,7 @@ void lightspark::stringToQName(const tiny_string& tmp, tiny_string& name, tiny_s
 	{
 		uint32_t dot_offset = dot-tmp.raw_buf();
 		ns = tmp.substr_bytes(0,dot_offset);
-		name = tmp.substr_bytes(dot_offset+1,tmp.numChars()-dot_offset-1);
+		name = tmp.substr_bytes(dot_offset+1,tmp.numBytes()-dot_offset-1);
 		return;
 	}
 	//No namespace or package in the string
@@ -1359,19 +1640,26 @@ void lightspark::stringToQName(const tiny_string& tmp, tiny_string& name, tiny_s
 	ns="";
 }
 
-RunState::RunState():last_FP(-1),FP(0),next_FP(0),stop_FP(0),explicit_FP(false)
+RunState::RunState():last_FP(-1),FP(0),next_FP(0),stop_FP(false),explicit_FP(false),creatingframe(false),frameadvanced(false),avm1ScriptExecutedAfterStop(false)
 {
 }
 
-tiny_string QName::getQualifiedName() const
+tiny_string QName::getQualifiedName(SystemState *sys,bool forDescribeType) const
 {
 	tiny_string ret;
-	if(!ns.empty())
+	if(nsStringId != BUILTIN_STRINGS::EMPTY)
 	{
-		ret+=ns;
+		ret+=sys->getStringFromUniqueId(nsStringId);
 		ret+="::";
 	}
-	ret+=name;
+	ret+=sys->getStringFromUniqueId(nameId);
+	if (forDescribeType && ret.startsWith("__AS3__.vec::Vector$"))
+	{
+		tiny_string ret2 = "__AS3__.vec::Vector.<";
+		ret2 += ret.substr(strlen("__AS3__.vec::Vector$"),ret.numChars());
+		ret2 += ">";
+		return ret2;
+	}
 	return ret;
 }
 
@@ -1379,8 +1667,8 @@ QName::operator multiname() const
 {
 	multiname ret(NULL);
 	ret.name_type = multiname::NAME_STRING;
-	ret.name_s_id = getSys()->getUniqueStringId(name);
-	ret.ns.push_back( nsNameAndKind(ns, NAMESPACE) );
+	ret.name_s_id = nameId;
+	ret.ns.emplace_back(getSys(),nsStringId, NAMESPACE);
 	ret.isAttribute = false;
 	return ret;
 }
@@ -1390,7 +1678,7 @@ FILLSTYLE::FILLSTYLE(uint8_t v):Gradient(v),version(v)
 }
 
 FILLSTYLE::FILLSTYLE(const FILLSTYLE& r):Matrix(r.Matrix),Gradient(r.Gradient),FocalGradient(r.FocalGradient),
-	bitmap(r.bitmap),Color(r.Color),FillStyleType(r.FillStyleType),version(r.version)
+	bitmap(r.bitmap),ShapeBounds(r.ShapeBounds),Color(r.Color),FillStyleType(r.FillStyleType),version(r.version)
 {
 }
 
@@ -1398,51 +1686,77 @@ FILLSTYLE::~FILLSTYLE()
 {
 }
 
-nsNameAndKind::nsNameAndKind(const tiny_string& _name, NS_KIND _kind)
+FILLSTYLE& FILLSTYLE::operator=(FILLSTYLE r)
 {
-	nsNameAndKindImpl tmp(_name, _kind);
-	getSys()->getUniqueNamespaceId(tmp, nsRealId, nsId);
-	nameIsEmpty=_name.empty();
+	Matrix = r.Matrix;
+	Gradient = r.Gradient;
+	FocalGradient = r.FocalGradient;
+	bitmap = r.bitmap;
+	ShapeBounds = r.ShapeBounds;
+	Color = r.Color;
+	FillStyleType = r.FillStyleType;
+	version = r.version;
+	return *this;
 }
 
-nsNameAndKind::nsNameAndKind(const char* _name, NS_KIND _kind)
+nsNameAndKind::nsNameAndKind(SystemState* sys,const tiny_string& _name, NS_KIND _kind)
 {
-	nsNameAndKindImpl tmp(_name, _kind);
-	getSys()->getUniqueNamespaceId(tmp, nsRealId, nsId);
-	nameIsEmpty=(_name[0]=='\0');
+	nsNameId = sys->getUniqueStringId(_name);
+	nsNameAndKindImpl tmp(nsNameId, _kind);
+	sys->getUniqueNamespaceId(tmp, nsRealId, nsId);
+	kind = _kind;
 }
 
-nsNameAndKind::nsNameAndKind(const tiny_string& _name, uint32_t _baseId, NS_KIND _kind)
+nsNameAndKind::nsNameAndKind(SystemState* sys,const char* _name, NS_KIND _kind)
+{
+	nsNameId = sys->getUniqueStringId(_name);
+	nsNameAndKindImpl tmp(nsNameId, _kind);
+	sys->getUniqueNamespaceId(tmp, nsRealId, nsId);
+	kind = _kind;
+}
+
+nsNameAndKind::nsNameAndKind(SystemState* sys,uint32_t _nameId, NS_KIND _kind)
+{
+	nsNameAndKindImpl tmp(_nameId, _kind);
+	sys->getUniqueNamespaceId(tmp, nsRealId, nsId);
+	nsNameId = _nameId;
+	kind = _kind;
+}
+nsNameAndKind::nsNameAndKind(SystemState* sys,uint32_t _nameId, NS_KIND _kind,RootMovieClip* root)
+{
+	assert(_kind==PROTECTED_NAMESPACE);
+	nsNameAndKindImpl tmp(_nameId, _kind,root);
+	sys->getUniqueNamespaceId(tmp, nsRealId, nsId);
+	nsNameId = _nameId;
+	kind = _kind;
+}
+nsNameAndKind::nsNameAndKind(SystemState* sys, uint32_t _nameId, uint32_t _baseId, NS_KIND _kind,RootMovieClip* root)
 {
 	assert(_kind==PROTECTED_NAMESPACE);
 	nsId=_baseId;
-	nsNameAndKindImpl tmp(_name, _kind, nsId);
+	nsNameAndKindImpl tmp(_nameId, _kind, root, nsId);
 	uint32_t tmpId;
-	getSys()->getUniqueNamespaceId(tmp, nsRealId, tmpId);
+	sys->getUniqueNamespaceId(tmp, nsRealId, tmpId);
 	assert(tmpId==_baseId);
-	nameIsEmpty=_name.empty();
+	nsNameId = _nameId;
+	kind = _kind;
 }
 
 nsNameAndKind::nsNameAndKind(ABCContext* c, uint32_t nsContextIndex)
 {
 	const namespace_info& ns=c->constant_pool.namespaces[nsContextIndex];
-	const tiny_string& nsName=c->getString(ns.name);
-	nsNameAndKindImpl tmp(nsName, (NS_KIND)(int)ns.kind);
+	nsNameId=c->getString(ns.name);
+	nsNameAndKindImpl tmp(nsNameId, (NS_KIND)(int)ns.kind,((NS_KIND)(int)ns.kind)==PROTECTED_NAMESPACE ? c->root.getPtr() : nullptr);
 	//Give an id hint, in case the namespace is created in the map
-	getSys()->getUniqueNamespaceId(tmp, c->namespaceBaseId+nsContextIndex, nsRealId, nsId);
+	c->root->getSystemState()->getUniqueNamespaceId(tmp, c->namespaceBaseId+nsContextIndex, nsRealId, nsId);
 	//Special handling for private namespaces, they are always compared by id
 	if(ns.kind==PRIVATE_NAMESPACE)
 		nsId=c->namespaceBaseId+nsContextIndex;
-	nameIsEmpty=nsName.empty();
+	kind = (NS_KIND)(int)ns.kind;
 }
 
-const nsNameAndKindImpl& nsNameAndKind::getImpl() const
-{
-	return getSys()->getNamespaceFromUniqueId(nsRealId);
-}
-
-nsNameAndKindImpl::nsNameAndKindImpl(const tiny_string& _name, NS_KIND _kind, uint32_t b)
-  : name(_name),kind(_kind),baseId(b)
+nsNameAndKindImpl::nsNameAndKindImpl(uint32_t _nameId, NS_KIND _kind, RootMovieClip *r, uint32_t b)
+  : nameId(_nameId),kind(_kind),baseId(b),root(r)
 {
 	if (kind != NAMESPACE &&
 	    kind != PACKAGE_NAMESPACE &&
@@ -1454,26 +1768,7 @@ nsNameAndKindImpl::nsNameAndKindImpl(const tiny_string& _name, NS_KIND _kind, ui
 	{
 		//I have seen empty namespace with kind 0. For other
 		//namespaces we should not get here.
-		if (!name.empty())
-			LOG(LOG_ERROR, "Invalid namespace kind, converting to public namespace");
-		kind = NAMESPACE;
-	}
-}
-
-nsNameAndKindImpl::nsNameAndKindImpl(const char* _name, NS_KIND _kind, uint32_t b)
- : name(_name),kind(_kind),baseId(b)
-{
-	if (kind != NAMESPACE &&
-	    kind != PACKAGE_NAMESPACE &&
-	    kind != PACKAGE_INTERNAL_NAMESPACE &&
-	    kind != PROTECTED_NAMESPACE &&
-	    kind != EXPLICIT_NAMESPACE &&
-	    kind != STATIC_PROTECTED_NAMESPACE &&
-	    kind != PRIVATE_NAMESPACE)
-	{
-		//I have seen empty namespace with kind 0. For other
-		//namespaces we should not get here.
-		if (!name.empty())
+		if (nameId != BUILTIN_STRINGS::EMPTY)
 			LOG(LOG_ERROR, "Invalid namespace kind, converting to public namespace");
 		kind = NAMESPACE;
 	}
@@ -1504,3 +1799,55 @@ tiny_string RGB::toString() const
 
 	return ss.str();
 }
+
+std::istream& lightspark::operator>>(std::istream& stream, SOUNDINFO& v)
+{
+	BitStream bs(stream);
+	UB(2,bs); // reserved
+	v.SyncStop = UB(1,bs);
+	v.SyncNoMultiple = UB(1,bs);
+	v.HasEnvelope = UB(1,bs);
+	v.HasLoops = UB(1,bs);
+	v.HasOutPoint = UB(1,bs);
+	v.HasInPoint = UB(1,bs);
+	if (v.HasInPoint)
+		stream >> v.InPoint;
+	if (v.HasOutPoint)
+		stream >> v.OutPoint;
+	if (v.HasLoops)
+		stream >> v.LoopCount;
+	if (v.HasEnvelope)
+	{
+		stream >> v.EnvPoints;
+		if (v.EnvPoints)
+			LOG(LOG_NOT_IMPLEMENTED,"SOUNDENVELOPE settings are read but not used");
+			
+		for (unsigned int i = 0; i < v.EnvPoints;i++)
+		{
+			SOUNDENVELOPE env;
+			stream >> env.Pos44;
+			stream >> env.LeftLevel;
+			stream >> env.RightLevel;
+			v.SoundEnvelope.push_back(env);
+		}
+	}
+	return stream;
+}
+
+std::istream& lightspark::operator>>(std::istream& stream, BUTTONCONDACTION& v)
+{
+	stream >> v.CondActionSize;
+	BitStream bs(stream);
+	v.CondIdleToOverDown = UB(1,bs);
+	v.CondOutDownToIdle = UB(1,bs);
+	v.CondOutDownToOverDown = UB(1,bs);
+	v.CondOverDownToOutDown = UB(1,bs);
+	v.CondOverDownToOverUp = UB(1,bs);
+	v.CondOverUpToOverDown = UB(1,bs);
+	v.CondOverUpToIdle = UB(1,bs);
+	v.CondIdleToOverUp = UB(1,bs);
+	v.CondKeyPress = UB(7,bs);
+	v.CondOverDownToIdle = UB(1,bs);
+
+	return stream;
+}
