@@ -23,6 +23,8 @@
 #include "compat.h"
 #include <iostream>
 #include <vector>
+#include <map>
+#include <stack>
 #include <list>
 #include <cairo.h>
 
@@ -37,16 +39,40 @@
 #ifdef BIG_ENDIAN
 #include <algorithm>
 #endif
-
-class memorystream;
-
+#define BUILTIN_STRINGS_CHAR_MAX 0x10000 // strings 0-0xffff are one-char strings of the corresponding unicode char
 namespace lightspark
 {
+enum BUILTIN_STRINGS { EMPTY=0, STRING_WILDCARD='*', ANY=BUILTIN_STRINGS_CHAR_MAX, VOID, PROTOTYPE, STRING_FUNCTION,STRING_AS3VECTOR,STRING_CLASS,STRING_AS3NS,STRING_NAMESPACENS,STRING_XML,STRING_TOSTRING,STRING_VALUEOF,STRING_LENGTH,STRING_CONSTRUCTOR
+					   ,STRING_AVM1_TARGET,STRING_THIS,STRING_AVM1_ROOT,STRING_AVM1_PARENT,STRING_AVM1_GLOBAL,STRING_SUPER
+					   ,STRING_ONENTERFRAME,STRING_ONMOUSEMOVE,STRING_ONMOUSEDOWN,STRING_ONMOUSEUP,STRING_ONPRESS,STRING_ONRELEASE,STRING_ONRELEASEOUTSIDE,STRING_ONMOUSEWHEEL, STRING_ONLOAD
+					   ,STRING_OBJECT,STRING_UNDEFINED,STRING_BOOLEAN,STRING_NUMBER,STRING_STRING,STRING_FUNCTION_LOWERCASE,STRING_ONROLLOVER,STRING_ONROLLOUT
+					   ,STRING_PROTO
+					   ,LAST_BUILTIN_STRING };
+enum BUILTIN_NAMESPACES { EMPTY_NS=0, AS3_NS };
+
 
 enum SWFOBJECT_TYPE { T_OBJECT=0, T_INTEGER=1, T_NUMBER=2, T_FUNCTION=3, T_UNDEFINED=4, T_NULL=5, T_STRING=6, 
-	/*UNUSED=7,*/ T_BOOLEAN=8, T_ARRAY=9, T_CLASS=10, T_QNAME=11, T_NAMESPACE=12, T_UINTEGER=13, T_PROXY=14, T_TEMPLATE=15};
-
+	T_INVALID=7, T_BOOLEAN=8, T_ARRAY=9, T_CLASS=10, T_QNAME=11, T_NAMESPACE=12, T_UINTEGER=13, T_PROXY=14, T_TEMPLATE=15};
+// this is used to avoid calls to dynamic_cast when testing for some classes
+enum CLASS_SUBTYPE { SUBTYPE_NOT_SET, SUBTYPE_PROXY, SUBTYPE_REGEXP, SUBTYPE_XML, SUBTYPE_XMLLIST,SUBTYPE_DATE, SUBTYPE_INHERIT, SUBTYPE_OBJECTCONSTRUCTOR,SUBTYPE_FUNCTIONOBJECT
+					 ,SUBTYPE_GLOBAL,SUBTYPE_FUNCTION,SUBTYPE_SYNTHETICFUNCTION,SUBTYPE_EVENT,SUBTYPE_WAITABLE_EVENT, SUBTYPE_INTERACTIVE_OBJECT, SUBTYPE_STAGE
+					 ,SUBTYPE_LOADERINFO,SUBTYPE_CONTEXTMENU,SUBTYPE_BITMAP,SUBTYPE_BITMAPDATA,SUBTYPE_SOUND,SUBTYPE_SOUNDTRANSFORM,SUBTYPE_SOUNDCHANNEL,SUBTYPE_PROGRESSEVENT
+					 ,SUBTYPE_VECTOR,SUBTYPE_DISPLAYOBJECT,SUBTYPE_DISPLAYOBJECTCONTAINER,SUBTYPE_CONTEXTMENUBUILTINITEMS, SUBTYPE_SHAREDOBJECT,SUBTYPE_TEXTFIELD,SUBTYPE_TEXTFORMAT
+					 ,SUBTYPE_KEYBOARD_EVENT,SUBTYPE_MOUSE_EVENT,SUBTYPE_ROOTMOVIECLIP,SUBTYPE_MATRIX,SUBTYPE_POINT,SUBTYPE_RECTANGLE,SUBTYPE_COLORTRANSFORM,SUBTYPE_BYTEARRAY
+					 ,SUBTYPE_APPLICATIONDOMAIN,SUBTYPE_LOADERCONTEXT,SUBTYPE_SPRITE,SUBTYPE_MOVIECLIP,SUBTYPE_TEXTBLOCK,SUBTYPE_FONTDESCRIPTION,SUBTYPE_CONTENTELEMENT,SUBTYPE_ELEMENTFORMAT
+					 ,SUBTYPE_TEXTELEMENT, SUBTYPE_ACTIVATIONOBJECT,SUBTYPE_TEXTLINE,SUBTYPE_STAGE3D,SUBTYPE_MATRIX3D,SUBTYPE_INDEXBUFFER3D,SUBTYPE_PROGRAM3D,SUBTYPE_VERTEXBUFFER3D
+					 ,SUBTYPE_CONTEXT3D,SUBTYPE_TEXTUREBASE,SUBTYPE_TEXTURE,SUBTYPE_CUBETEXTURE,SUBTYPE_RECTANGLETEXTURE,SUBTYPE_VIDEOTEXTURE,SUBTYPE_VECTOR3D,SUBTYPE_NETSTREAM
+					 ,SUBTYPE_WORKER,SUBTYPE_WORKERDOMAIN,SUBTYPE_MUTEX,SUBTYPE_AVM1FUNCTION,SUBTYPE_SAMPLEDATA_EVENT
+					 ,SUBTYPE_BITMAPFILTER,SUBTYPE_GLOWFILTER,SUBTYPE_DROPSHADOWFILTER,SUBTYPE_GRADIENTGLOWFILTER,SUBTYPE_BEVELFILTER,SUBTYPE_COLORMATRIXFILTER,SUBTYPE_BLURFILTER,SUBTYPE_CONVOLUTIONFILTER,SUBTYPE_DISPLACEMENTFILTER,SUBTYPE_GRADIENTBEVELFILTER,SUBTYPE_SHADERFILTER
+					 ,SUBTYPE_THROTTLE_EVENT,SUBTYPE_CONTEXTMENUEVENT,SUBTYPE_GAMEINPUTEVENT, SUBTYPE_GAMEINPUTDEVICE, SUBTYPE_VIDEO
+				   };
+ 
 enum STACK_TYPE{STACK_NONE=0,STACK_OBJECT,STACK_INT,STACK_UINT,STACK_NUMBER,STACK_BOOLEAN};
+
+enum AS_BLENDMODE { BLENDMODE_NORMAL=0, BLENDMODE_LAYER=2, BLENDMODE_MULTIPLY=3,BLENDMODE_SCREEN=4,BLENDMODE_LIGHTEN=5,BLENDMODE_DARKEN=6, BLENDMODE_DIFFERENCE=7,
+					BLENDMODE_ADD=8, BLENDMODE_SUBTRACT=9,BLENDMODE_INVERT=10,BLENDMODE_ALPHA=11,BLENDMODE_ERASE=12,BLENDMODE_OVERLAY=13,BLENDMODE_HARDLIGHT=14
+				  };
+
 inline std::ostream& operator<<(std::ostream& s, const STACK_TYPE& st)
 {
 	switch(st)
@@ -81,25 +107,38 @@ enum FILE_TYPE { FT_UNKNOWN=0, FT_SWF, FT_COMPRESSED_SWF, FT_LZMA_COMPRESSED_SWF
 
 typedef double number_t;
 
+class Type;
+class SystemState;
 class ASObject;
+class ASString;
 class ABCContext;
-class namespace_info;
+class URLInfo;
+class DisplayObject;
+class MovieClip;
+class Frame;
+class AVM1Function;
+class AVM1context;
+struct namespace_info;
 
 struct multiname;
 class QName
 {
 public:
-	tiny_string ns;
-	tiny_string name;
-	QName(const tiny_string& _name, const tiny_string& _ns):ns(_ns),name(_name){}
+	uint32_t nsStringId;
+	uint32_t nameId;
+	QName(uint32_t _nameId, uint32_t _nsId):nsStringId(_nsId),nameId(_nameId){}
 	bool operator<(const QName& r) const
 	{
-		if(ns==r.ns)
-			return name<r.name;
+		if(nsStringId==r.nsStringId)
+			return nameId<r.nameId;
 		else
-			return ns<r.ns;
+			return nsStringId<r.nsStringId;
 	}
-	tiny_string getQualifiedName() const;
+	bool operator==(const QName& r) const
+	{
+		return nameId==r.nameId && nsStringId==r.nsStringId;
+	}
+	tiny_string getQualifiedName(SystemState* sys, bool forDescribeType = false) const;
 	operator multiname() const;
 };
 
@@ -223,11 +262,10 @@ public:
 class u32
 {
 friend std::istream& operator>>(std::istream& in, u32& v);
-friend memorystream& operator>>(memorystream& in, u32& v);
 private:
 	uint32_t val;
 public:
-	operator uint32_t() const{return val;}
+	inline operator uint32_t() const{return val;}
 };
 
 class STRING
@@ -238,10 +276,13 @@ friend class ASString;
 private:
 	std::string String;
 public:
-	STRING():String(){};
+	STRING():String(){}
 	STRING(const char* s):String(s)
 	{
 	}
+	STRING(const char* s, int len):String(s,len)
+	{
+	}
 	bool operator==(const STRING& s)
 	{
 		if(String.size()!=s.String.size())
@@ -285,25 +326,30 @@ public:
 //Numbers taken from AVM2 specs
 enum NS_KIND { NAMESPACE=0x08, PACKAGE_NAMESPACE=0x16, PACKAGE_INTERNAL_NAMESPACE=0x17, PROTECTED_NAMESPACE=0x18, 
 			EXPLICIT_NAMESPACE=0x19, STATIC_PROTECTED_NAMESPACE=0x1A, PRIVATE_NAMESPACE=0x05 };
-
+class RootMovieClip;
 struct nsNameAndKindImpl
 {
-	tiny_string name;
+	uint32_t nameId;
 	NS_KIND kind;
 	uint32_t baseId;
-	nsNameAndKindImpl(const tiny_string& _name, NS_KIND _kind, uint32_t b=-1);
-	nsNameAndKindImpl(const char* _name, NS_KIND _kind, uint32_t b=-1);
+	// this is null for all namespaces except kind PROTECTED_NAMESPACE, to ensure they are treated as different namespaces when declared in different swf files
+	RootMovieClip* root;
+	nsNameAndKindImpl(uint32_t _nameId, NS_KIND _kind, RootMovieClip* r=nullptr,uint32_t b=-1);
 	bool operator<(const nsNameAndKindImpl& r) const
 	{
+		if(root != r.root)
+			return root<r.root;
 		if(kind==r.kind)
-			return name < r.name;
+			return nameId < r.nameId;
 		else
 			return kind < r.kind;
 	}
 	bool operator>(const nsNameAndKindImpl& r) const
 	{
+		if(root != r.root)
+			return root>r.root;
 		if(kind==r.kind)
-			return name > r.name;
+			return nameId > r.nameId;
 		else
 			return kind > r.kind;
 	}
@@ -313,69 +359,95 @@ struct nsNameAndKind
 {
 	uint32_t nsId;
 	uint32_t nsRealId;
-	bool nameIsEmpty;
-	nsNameAndKind(const tiny_string& _name, NS_KIND _kind);
-	nsNameAndKind(const char* _name, NS_KIND _kind);
+	uint32_t nsNameId;
+	NS_KIND kind;
+	nsNameAndKind():nsId(0),nsRealId(0),nsNameId(BUILTIN_STRINGS::EMPTY),kind(NAMESPACE) {}
+	nsNameAndKind(SystemState *sys, const tiny_string& _name, NS_KIND _kind);
+	nsNameAndKind(SystemState* sys,const char* _name, NS_KIND _kind);
+	nsNameAndKind(SystemState* sys, uint32_t _nameId, NS_KIND _kind);
+	nsNameAndKind(SystemState* sys, uint32_t _nameId, NS_KIND _kind, RootMovieClip *root);
 	nsNameAndKind(ABCContext * c, uint32_t nsContextIndex);
 	/*
 	 * Special constructor for protected namespace, which have
 	 * different representationId
 	 */
-	nsNameAndKind(const tiny_string& _name, uint32_t _baseId, NS_KIND _kind);
+	nsNameAndKind(SystemState* sys, uint32_t _nameId, uint32_t _baseId, NS_KIND _kind, RootMovieClip *root);
 	/*
 	 * Special version to create the empty bultin namespace
 	 */
-	nsNameAndKind(uint32_t id):nsId(id),nsRealId(id),nameIsEmpty(true)
+	nsNameAndKind(uint32_t id):nsId(id),nsRealId(id)
 	{
 		assert(nsId==0);
 	}
-	bool operator<(const nsNameAndKind& r) const
+	inline bool operator<(const nsNameAndKind& r) const
 	{
 		return nsId < r.nsId;
 	}
-	bool operator>(const nsNameAndKind& r) const
+	inline bool operator>(const nsNameAndKind& r) const
 	{
 		return nsId > r.nsId;
 	}
-	bool operator==(const nsNameAndKind& r) const
+	inline bool operator==(const nsNameAndKind& r) const
 	{
 		return nsId==r.nsId;
 	}
-	const nsNameAndKindImpl& getImpl() const;
-	bool hasEmptyName() const
+	inline bool hasEmptyName() const
 	{
-		return nameIsEmpty;
+		return nsId==0;
+	}
+	inline bool hasBuiltinName() const
+	{
+		return nsNameId==STRING_AS3NS;
 	}
 };
 
 struct multiname: public memory_reporter
 {
+	uint32_t name_s_id;
 	union
 	{
-		uint32_t name_s_id;
 		int32_t name_i;
+		uint32_t name_ui;
 		number_t name_d;
 		ASObject* name_o;
 	};
 	std::vector<nsNameAndKind, reporter_allocator<nsNameAndKind>> ns;
-	enum NAME_TYPE {NAME_STRING,NAME_INT,NAME_NUMBER,NAME_OBJECT};
-	NAME_TYPE name_type;
-	bool isAttribute;
-	multiname(MemoryAccount* m);
+	const Type* cachedType;
+	std::vector<multiname*> templateinstancenames;
+	enum NAME_TYPE {NAME_STRING,NAME_INT,NAME_UINT,NAME_NUMBER,NAME_OBJECT};
+	NAME_TYPE name_type:3;
+	bool isAttribute:1;
+	bool isStatic:1;
+	bool hasEmptyNS:1;
+	bool hasBuiltinNS:1;
+	bool hasGlobalNS:1;
+	bool isInteger:1;
+	multiname(MemoryAccount* m):name_s_id(UINT32_MAX),name_o(nullptr),ns(reporter_allocator<nsNameAndKind>(m)),cachedType(nullptr),name_type(NAME_OBJECT),isAttribute(false),isStatic(true),hasEmptyNS(true),hasBuiltinNS(false),hasGlobalNS(true),isInteger(false)
+	{
+	}
+	
 	/*
 		Returns a string name whatever is the name type
 	*/
-	tiny_string normalizedName() const;
+	const tiny_string normalizedName(SystemState *sys) const;
 	/*
 	 * 	Return a string id whatever is the name type
 	 */
-	uint32_t normalizedNameId() const;
-	tiny_string qualifiedString() const;
+	uint32_t normalizedNameId(SystemState *sys) const;
+	/*
+		Returns a string name whatever is the name type, but does not resolve NAME_OBJECT names
+		this should be used for exception or debug messages to avoid calling 
+		overridden toString property of the object
+	*/
+	const tiny_string normalizedNameUnresolved(SystemState *sys) const;
+
+	const tiny_string qualifiedString(SystemState *sys, bool forDescribeType=false) const;
 	/* sets name_type, name_s/name_d based on the object n */
-	void setName(ASObject* n);
+	void setName(union asAtom &n, SystemState *sys);
 	void resetNameIfObject();
-	bool isQName() const { return ns.size() == 1; }
-	bool toUInt(uint32_t& out, bool acceptStringFractions=false) const;
+	inline bool isQName() const { return ns.size() == 1; }
+	bool toUInt(SystemState *sys, uint32_t& out, bool acceptStringFractions=false, bool* isNumber=nullptr) const;
+	inline bool isEmpty() const { return name_type == NAME_OBJECT && name_o == nullptr;}
 };
 
 class FLOAT 
@@ -387,6 +459,18 @@ public:
 	FLOAT():val(0){}
 	FLOAT(float v):val(v){}
 	operator float(){ return val; }
+	inline void read(const uint8_t* data)
+	{
+		union float_reader
+		{
+			uint32_t dump;
+			float value;
+		};
+		float_reader dummy;
+		dummy.dump = *(uint32_t*)data;
+		dummy.dump=GINT32_FROM_LE(dummy.dump);
+		val=dummy.value;
+	}
 };
 
 class DOUBLE 
@@ -398,6 +482,27 @@ public:
 	DOUBLE():val(0){}
 	DOUBLE(double v):val(v){}
 	operator double(){ return val; }
+	void read(const uint8_t* data)
+	{
+		union double_reader
+		{
+			uint64_t dump;
+			double value;
+		};
+		double_reader dummy;
+		// "Wacky format" is 45670123. Thanks to Gnash for reversing :-)
+		uint8_t* p = (uint8_t*)&dummy.dump;
+		*p++ = data[4];
+		*p++ = data[5];
+		*p++ = data[6];
+		*p++ = data[7];
+		*p++ = data[0];
+		*p++ = data[1];
+		*p++ = data[2];
+		*p++ = data[3];
+		dummy.dump=GINT64_FROM_LE(dummy.dump);
+		val=dummy.value;
+	}
 };
 
 //TODO: Really implement or suppress
@@ -420,6 +525,13 @@ public:
 		else
 			return CodeAndLen&0x3f;
 	}
+	unsigned int getHeaderSize() const
+	{
+		if((CodeAndLen&0x3f)==0x3f)
+			return sizeof(uint16_t) + sizeof(uint32_t);
+		else
+			return sizeof(uint16_t);
+	}
 	unsigned int getTagType() const
 	{
 		return CodeAndLen>>6;
@@ -429,8 +541,8 @@ public:
 class RGB
 {
 public:
-	RGB(){};
-	RGB(int r,int g, int b):Red(r),Green(g),Blue(b){};
+	RGB(){}
+	RGB(int r,int g, int b):Red(r),Green(g),Blue(b){}
 	RGB(uint32_t color):Red((color>>16)&0xFF),Green((color>>8)&0xFF),Blue(color&0xFF){}
 	//Parses a color from hex triplet string #RRGGBB
 	RGB(const tiny_string& colorstr);
@@ -513,14 +625,14 @@ inline std::istream & operator>>(std::istream &s, SI16_FLV& v)
 inline std::istream& operator>>(std::istream& s, UI16_SWF& v)
 {
 	s.read((char*)&v.val,2);
-	v.val=GINT16_FROM_LE(v.val);
+	v.val=GUINT16_FROM_LE(v.val);
 	return s;
 }
 
 inline std::istream& operator>>(std::istream& s, UI16_FLV& v)
 {
 	s.read((char*)&v.val,2);
-	v.val=GINT16_FROM_BE(v.val);
+	v.val=GUINT16_FROM_BE(v.val);
 	return s;
 }
 
@@ -559,14 +671,14 @@ inline std::istream& operator>>(std::istream& s, SI24_FLV& v)
 inline std::istream& operator>>(std::istream& s, UI32_SWF& v)
 {
 	s.read((char*)&v.val,4);
-	v.val=GINT32_FROM_LE(v.val);
+	v.val=GUINT32_FROM_LE(v.val);
 	return s;
 }
 
 inline std::istream& operator>>(std::istream& s, UI32_FLV& v)
 {
 	s.read((char*)&v.val,4);
-	v.val=GINT32_FROM_BE(v.val);
+	v.val=GUINT32_FROM_BE(v.val);
 	return s;
 }
 
@@ -644,7 +756,7 @@ public:
 	unsigned char buffer;
 	unsigned char pos;
 public:
-	BitStream(std::istream& in):f(in),buffer(0),pos(0){};
+	BitStream(std::istream& in):f(in),buffer(0),pos(0){}
 	unsigned int readBits(unsigned int num)
 	{
 		unsigned int ret=0;
@@ -672,10 +784,9 @@ public:
 class FB
 {
 	int32_t buf;
-	int size;
 public:
-	FB():buf(0),size(0){}
-	FB(int s,BitStream& stream):size(s)
+	FB():buf(0){}
+	FB(int s,BitStream& stream)
 	{
 		if(s>32)
 			LOG(LOG_ERROR,_("Fixed point bit field wider than 32 bit not supported"));
@@ -795,7 +906,7 @@ public:
 	number_t getScaleX() const
 	{
 		number_t ret=sqrt(xx*xx + yx*yx);
-		if(xx>0)
+		if(xx>=0)
 			return ret;
 		else
 			return -ret;
@@ -803,7 +914,7 @@ public:
 	number_t getScaleY() const
 	{
 		number_t ret=sqrt(yy*yy + xy*xy);
-		if(yy>0)
+		if(yy>=0)
 			return ret;
 		else
 			return -ret;
@@ -885,11 +996,13 @@ class FILLSTYLE
 public:
 	FILLSTYLE(uint8_t v);
 	FILLSTYLE(const FILLSTYLE& r);
+	FILLSTYLE& operator=(FILLSTYLE r);
 	virtual ~FILLSTYLE();
 	MATRIX Matrix;
 	GRADIENT Gradient;
 	FOCALGRADIENT FocalGradient;
 	_NR<BitmapContainer> bitmap;
+	RECT ShapeBounds;
 	RGBA Color;
 	FILL_STYLE_TYPE FillStyleType;
 	uint8_t version;
@@ -909,6 +1022,13 @@ public:
 	std::vector<RGBA> EndColors;
 	RGBA StartColor;
 	RGBA EndColor;
+	
+	// FOCAL_RADIAL_GRADIENT
+	int SpreadMode;
+	int InterpolationMode;
+	FIXED8 StartFocalPoint;
+	FIXED8 EndFocalPoint;
+
 	~MORPHFILLSTYLE(){}
 };
 
@@ -924,13 +1044,13 @@ public:
 class LINESTYLE2
 {
 public:
-	LINESTYLE2(uint8_t v):FillType(v),version(v){}
-	UB StartCapStyle;
-	UB JointStyle;
-	UB HasFillFlag;
-	UB NoHScaleFlag;
-	UB NoVScaleFlag;
-	UB PixelHintingFlag;
+	LINESTYLE2(uint8_t v):HasFillFlag(false),FillType(v),version(v){}
+	int StartCapStyle;
+	int JointStyle;
+	bool HasFillFlag;
+	bool NoHScaleFlag;
+	bool NoVScaleFlag;
+	int PixelHintingFlag;
 	UB NoClose;
 	UB EndCapStyle;
 	UI16_SWF Width;
@@ -1073,8 +1193,8 @@ class SHAPE
 	friend std::istream& operator>>(std::istream& stream, SHAPE& v);
 	friend std::istream& operator>>(std::istream& stream, SHAPEWITHSTYLE& v);
 public:
-	SHAPE(uint8_t v=0):fillOffset(0),lineOffset(0),version(v){}
-	virtual ~SHAPE(){};
+	SHAPE(uint8_t v=0,bool _forfont=false):fillOffset(0),lineOffset(0),version(v),forfont(_forfont){}
+	virtual ~SHAPE(){}
 	UB NumFillBits;
 	UB NumLineBits;
 	unsigned int fillOffset;
@@ -1082,6 +1202,7 @@ public:
 	uint8_t version; /* version of the DefineShape tag, 0 if
 			  * DefineFont or other tag */
 	std::vector<SHAPERECORD> ShapeRecords;
+	bool forfont;
 };
 
 class SHAPEWITHSTYLE : public SHAPE
@@ -1118,6 +1239,7 @@ public:
 			   number_t& blueOffset,
 			   number_t& alphaOffset) const;
 	float transformedAlpha(float alpha) const;
+	bool isfilled() const {return  HasAddTerms || HasMultTerms; }
 };
 
 class CXFORM
@@ -1232,6 +1354,7 @@ public:
 class FILTER
 {
 public:
+	UI8 FilterID;
 	DROPSHADOWFILTER DropShadowFilter;
 	BLURFILTER BlurFilter;
 	GLOWFILTER GlowFilter;
@@ -1275,24 +1398,78 @@ public:
 
 class CLIPEVENTFLAGS
 {
+private:
+	uint32_t swfversion;
 public:
-	uint32_t toParse;
+	CLIPEVENTFLAGS(uint32_t v):swfversion(v) {}
+	bool ClipEventKeyUp;
+	bool ClipEventKeyDown;
+	bool ClipEventMouseUp;
+	bool ClipEventMouseDown;
+	bool ClipEventMouseMove;
+	bool ClipEventUnload;
+	bool ClipEventEnterFrame;
+	bool ClipEventLoad;
+	bool ClipEventDragOver;
+	bool ClipEventRollOut;
+	bool ClipEventRollOver;
+	bool ClipEventReleaseOutside;
+	bool ClipEventRelease;
+	bool ClipEventPress;
+	bool ClipEventInitialize;
+	bool ClipEventData;
+	bool ClipEventConstruct;
+	bool ClipEventKeyPress;
+	bool ClipEventDragOut;
 	bool isNull();
+	uint32_t getSWFVersion() const { return swfversion; }
 };
 
+class AdditionalDataTag;
+class ACTIONRECORD;
 class CLIPACTIONRECORD
 {
 public:
+	CLIPACTIONRECORD(uint32_t v, AdditionalDataTag* _datatag):EventFlags(v),startactionpos(0),datatag( _datatag) {}
 	CLIPEVENTFLAGS EventFlags;
 	UI32_SWF ActionRecordSize;
+	UI8 KeyCode;
+	std::vector<uint8_t> actions;
 	bool isLast();
+	uint32_t startactionpos;
+	AdditionalDataTag* datatag;
 };
-
 class CLIPACTIONS
 {
 public:
+	CLIPACTIONS(uint32_t v, AdditionalDataTag* _datatag=nullptr):AllEventFlags(v),datatag(_datatag) {}
 	std::vector<CLIPACTIONRECORD> ClipActionRecords;
 	CLIPEVENTFLAGS AllEventFlags;
+	AdditionalDataTag* datatag;
+};
+
+class SOUNDENVELOPE
+{
+public:
+	UI32_SWF Pos44;
+	UI16_SWF LeftLevel;
+	UI16_SWF RightLevel;
+};
+
+class SOUNDINFO
+{
+public:
+	UB SyncStop;
+	UB SyncNoMultiple;
+	UB HasEnvelope;
+	UB HasLoops;
+	UB HasOutPoint;
+	UB HasInPoint;
+	UI32_SWF InPoint;
+	UI32_SWF OutPoint;
+	UI16_SWF LoopCount;
+	UI8 EnvPoints;
+	std::vector<SOUNDENVELOPE> SoundEnvelope;
 };
 
 class RunState
@@ -1303,12 +1480,68 @@ public:
 	unsigned int next_FP;
 	bool stop_FP;
 	bool explicit_FP;
+	bool creatingframe;
+	bool frameadvanced;
+	bool avm1ScriptExecutedAfterStop;
 	RunState();
+	inline void reset()
+	{
+		last_FP = -1;
+		FP = 0;
+		next_FP = 0;
+		stop_FP = false;
+		explicit_FP = false;
+		creatingframe = false;
+		frameadvanced = false;
+		avm1ScriptExecutedAfterStop=false;
+	}
+};
+class Activation_object;
+class ACTIONRECORD
+{
+public:
+	static void PushStack(std::stack<asAtom>& stack,const asAtom& a);
+	static asAtom PopStack(std::stack<asAtom>& stack);
+	static asAtom PeekStack(std::stack<asAtom>& stack);
+	static void executeActions(DisplayObject* clip, AVM1context* context, const std::vector<uint8_t> &actionlist, uint32_t startactionpos, std::map<uint32_t, union asAtom> &scopevariables, asAtom *result = nullptr, asAtom* obj = nullptr, asAtom *args = nullptr, uint32_t num_args=0, const std::vector<uint32_t>& paramnames=std::vector<uint32_t>(), const std::vector<uint8_t>& paramregisternumbers=std::vector<uint8_t>(),
+			bool preloadParent=false, bool preloadRoot=false, bool suppressSuper=true, bool preloadSuper=false, bool suppressArguments=false, bool preloadArguments=false, bool suppressThis=true, bool preloadThis=false, bool preloadGlobal=false, AVM1Function *caller = nullptr, AVM1Function *callee = nullptr, Activation_object *actobj=nullptr, asAtom* superobj=nullptr);
+};
+class BUTTONCONDACTION
+{
+friend std::istream& operator>>(std::istream& s, BUTTONCONDACTION& v);
+public:
+	BUTTONCONDACTION():CondActionSize(0)
+	  ,CondIdleToOverDown(false),CondOutDownToIdle(false),CondOutDownToOverDown(false),CondOverDownToOutDown(false)
+	  ,CondOverDownToOverUp(false),CondOverUpToOverDown(false),CondOverUpToIdle(false),CondIdleToOverUp(false),CondOverDownToIdle(false),startactionpos(0)
+	{}
+	UI16_SWF CondActionSize;
+	bool CondIdleToOverDown;
+	bool CondOutDownToIdle;
+	bool CondOutDownToOverDown;
+	bool CondOverDownToOutDown;
+	bool CondOverDownToOverUp;
+	bool CondOverUpToOverDown;
+	bool CondOverUpToIdle;
+	bool CondIdleToOverUp;
+	bool CondOverDownToIdle;
+	uint32_t CondKeyPress;
+	uint32_t startactionpos;
+	std::vector<uint8_t> actions;
 };
 
-ASObject* abstract_i(int32_t i);
-ASObject* abstract_ui(uint32_t i);
-ASObject* abstract_d(number_t i);
+ASObject* abstract_i(SystemState *sys, int32_t i);
+ASObject* abstract_ui(SystemState *sys, uint32_t i);
+ASObject* abstract_d(SystemState *sys, number_t i);
+ASObject* abstract_d_constant(SystemState *sys, number_t i);
+ASObject* abstract_di(SystemState *sys, int64_t i);
+ASObject* abstract_s(SystemState *sys);
+ASObject* abstract_s(SystemState *sys, const char* s, uint32_t len);
+ASObject* abstract_s(SystemState *sys, const char* s);
+ASObject* abstract_s(SystemState *sys, const tiny_string& s);
+ASObject* abstract_s(SystemState *sys, uint32_t stringId);
+ASObject* abstract_null(SystemState *sys);
+ASObject* abstract_undefined(SystemState *sys);
+ASObject* abstract_b(SystemState *sys, bool b);
 
 void stringToQName(const tiny_string& tmp, tiny_string& name, tiny_string& ns);
 
@@ -1322,6 +1555,9 @@ std::ostream& operator<<(std::ostream& s, const nsNameAndKind& r);
 std::ostream& operator<<(std::ostream& s, const multiname& r);
 std::ostream& operator<<(std::ostream& s, const tiny_string& r) DLL_PUBLIC;
 std::ostream& operator<<(std::ostream& s, const QName& r);
+std::ostream& operator<<(std::ostream& s, const MATRIX& r);
+std::ostream& operator<<(std::ostream& s, const URLInfo& u);
+std::ostream& operator<<(std::ostream& s, const DisplayObject& r);
 
 std::istream& operator>>(std::istream& s, RECT& v);
 std::istream& operator>>(std::istream& s, CLIPEVENTFLAGS& v);
@@ -1329,6 +1565,9 @@ std::istream& operator>>(std::istream& s, CLIPACTIONRECORD& v);
 std::istream& operator>>(std::istream& s, CLIPACTIONS& v);
 std::istream& operator>>(std::istream& s, RGB& v);
 std::istream& operator>>(std::istream& s, RGBA& v);
+std::istream& operator>>(std::istream& s, GRADRECORD& v);
+std::istream& operator>>(std::istream& s, GRADIENT& v);
+std::istream& operator>>(std::istream& s, FOCALGRADIENT& v);
 std::istream& operator>>(std::istream& stream, SHAPEWITHSTYLE& v);
 std::istream& operator>>(std::istream& stream, SHAPE& v);
 std::istream& operator>>(std::istream& stream, FILLSTYLEARRAY& v);
@@ -1359,7 +1598,8 @@ std::istream& operator>>(std::istream& stream, GRADIENTGLOWFILTER& v);
 std::istream& operator>>(std::istream& stream, CONVOLUTIONFILTER& v);
 std::istream& operator>>(std::istream& stream, COLORMATRIXFILTER& v);
 std::istream& operator>>(std::istream& stream, GRADIENTBEVELFILTER& v);
-
-
-};
+std::istream& operator>>(std::istream& stream, SOUNDINFO& v);
+std::istream& operator>>(std::istream& stream, ACTIONRECORD& v);
+std::istream& operator>>(std::istream& stream, BUTTONCONDACTION& v);
+}
 #endif /* SWFTYPES_H */
